Package: roar
Type: Package
Title: Identify differential APA usage from RNA-seq alignments
Version: 1.46.0
Date: 2016-03-21
Author: Elena Grassi
Maintainer: Elena Grassi <grassi.e@gmail.com>
Description: Identify preferential usage of APA sites, comparing two biological conditions, starting from known alternative sites and alignments obtained from standard RNA-seq experiments.
biocViews: Sequencing, HighThroughputSequencing, RNAseq, Transcription
License: GPL-3
Depends: R (>= 3.0.1)
Imports: methods, BiocGenerics, S4Vectors, IRanges, GenomicRanges,
        SummarizedExperiment, GenomicAlignments (>= 0.99.4),
        rtracklayer, GenomeInfoDb
Suggests: RNAseqData.HNRNPC.bam.chr14, testthat
URL: https://github.com/vodkatad/roar/
git_url: https://git.bioconductor.org/packages/roar
git_branch: RELEASE_3_22
git_last_commit: bb81790
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 06:09:17 UTC; biocbuild

import(methods, BiocGenerics, S4Vectors, IRanges, GenomicRanges, SummarizedExperiment, GenomicAlignments, GenomeInfoDb)
importFrom(rtracklayer, import)
importFrom(stats, fisher.test, p.adjust, pchisq)

exportClasses("RoarDataset", "RoarDatasetMultipleAPA")

export(
  RoarDataset,
  RoarDatasetFromFiles,
  RoarDatasetMultipleAPAFromFiles,
  RoarDatasetMultipleAPA
)

exportMethods(
   countPrePost,
   computeRoars,
   computePvals,
   computePairedPvals,
   totalResults,
   fpkmResults,
   countResults,
   standardFilter,
   pvalueFilter,
   pvalueCorrectFilter
#   cores
#  "cores<-"
)

RoarDatasetMultipleAPAFromFiles <- function(treatmentBams, controlBams, gtf) {
   gtfGRanges <- import(gtf)
   treatmentBamsGenomicAlignments <- lapply(treatmentBams, readGAlignments)
   controlBamsGenomicAlignments <- lapply(controlBams, readGAlignments)
   RoarDatasetMultipleAPA(treatmentBamsGenomicAlignments, controlBamsGenomicAlignments, gtfGRanges)
}

RoarDatasetMultipleAPA <- function(treatmentBamsGenomicAlignments, controlBamsGenomicAlignments, gtfGRanges) {
   apas_melted <- gtfGRanges[mcols(gtfGRanges)$type=="apa"]
   genes_melted <- gtfGRanges[mcols(gtfGRanges)$type=="gene"]
   mcols(apas_melted)$gene <- sapply(strsplit(mcols(apas_melted)$apa, '_', 
                                              fixed=TRUE),
                                     function(x) { x[length(x)]})
   genes_ids <- sort(unique(as.character(mcols(genes_melted)$gene)))
   genes_ids_apas <- sort(unique(mcols(apas_melted)$gene))
   if (!all(genes_ids==genes_ids_apas)) {
      stop("All the genes in the gtf should have at least one apa")
   }
   genes <- do.call(GRangesList, sapply(genes_ids, 
                                        function(x) {genes_melted[mcols(genes_melted)$gene==x]}))
   apas <- do.call(GRangesList, sapply(genes_ids, 
                                       function(x) {apas_melted[mcols(apas_melted)$gene==x]}))
   names(apas) <- genes_ids
   names(genes) <- genes_ids
   new("RoarDatasetMultipleAPA", treatmentBams=treatmentBamsGenomicAlignments, 
       controlBams=controlBamsGenomicAlignments, 
       geneCoords=genes, apaCoords=apas, step=0, paired=FALSE, cores=1)
}

setMethod("countPrePost", signature(rds="RoarDatasetMultipleAPA"),
   function(rds, stranded=FALSE) {
      goOn <- checkStep(rds, 0)
      if (!goOn[[1]]) {
         return(rds)
      }
      rds <- goOn[[2]]
      allFragmentsAndPrePostDef <- mapply(getApaGenesFractions,
                              rds@geneCoords, 
                              rds@apaCoords)
      rds@fragments <- GRangesList(allFragmentsAndPrePostDef[1,]) 
      #Right subsetting? Yes because in this situation mapply gives us a matrix with 
      #columns == gene names and rows that are fragments and prePostDef (SIMPLIFY=TRUE).
      rds@prePostDef <- allFragmentsAndPrePostDef[2,]
      # A GRangesList with GRanges for all fragments defining pre/post
      # in a gene and a list with info on APA choices and which
      # fragments are to be summed.
      ResizeReadsPlus <- function(reads, width=1, fix="end", ...) {
         reads <- as(reads, "GRanges")
         #stopifnot(all(strand(reads) != "*"))
         # Bad and ugly and will do horrible things for stranded data but still...
         # FIXME: if stranded=T different resize functions that do not change read strand but
         # move them always at their ends (should be the right call because if they are on a given
         # strand they align on a feature on that strand).
         strand(reads) <- rep("+", length(reads))
         resize(reads, width=width, fix=fix, ...)
      }
      ResizeReadsMinus <- function(reads, width=1, fix="start", ...) {
         reads <- as(reads, "GRanges")
         #stopifnot(all(strand(reads) != "*"))
         strand(reads) <- rep("+", length(reads))
         resize(reads, width=width, fix=fix, ...)
      }
      # Does not work as expected as long as for GRangesList counts
      # are collapsed: need to unlist.
      summOv <- function(x) {
         frag <-  unlist(rds@fragments)
         # We unlist because 
         # When a GRanges is supplied, each row is considered a feature.
         # When a GRangesList is supplied, each higher list-level 
         # is considered a feature. 
         # This distinction is important when defining overlaps.
         # Having genes as feature would help? 
         # No, multiple counts would happen I believe.
         featPlus <- frag[strand(frag)=="+"]
         featMinus <- frag[strand(frag)=="-"]
         plus <- summarizeOverlaps(features=featPlus, reads=x, 
                                    ignore.strand=!stranded,
                                    preprocess.reads=ResizeReadsPlus)
         minus <- summarizeOverlaps(features=featMinus, reads=x, 
                                    ignore.strand=!stranded, 
                                    preprocess.reads=ResizeReadsMinus)
         return(rbind(plus, minus)) # Does this work as expected? Yes. 
         # A more insightful comment would be what I expected...fuck you past Elena.
         # It's like an rbind for counts as long as we have only counts as assays columns 
         # and rowRanges are our genes. The order of fragments inside genes is kept while
         # not their relative order but we will get them from names later on so it's ok. 
      }
      if (length(rds@treatmentBams) == 1 && length(rds@controlBams) == 1) {
         # We obtain counts for both conditions on gene fragments and them sum 
         # them to obtain PRE/POST counts.
         treatmentSE <- summOv(rds@treatmentBams[[1]])
         controlSE <- summOv(rds@controlBams[[1]])    
         rds <- generateRoarsSingleBam(rds, treatmentSE, controlSE)
         rds@corrTreatment <- mean(qwidth(rds@treatmentBams[[1]]))
         rds@corrControl <- mean(qwidth(rds@controlBams[[1]]))
      } else {
         countsControl <- vector(mode = "list", length = length(rds@controlBams))
         countsTreatment <- vector(mode = "list", length = length(rds@treatmentBams))
         for (i in 1:length(rds@treatmentBams)) {
            countsTreatment[[i]] <- summOv(rds@treatmentBams[[i]])
         }
         for (i in 1:length(rds@controlBams)) {
            countsControl[[i]] <- summOv(rds@controlBams[[i]])
         }
         rds <- generateRoarsMultipleBam(rds, countsTreatment, countsControl)
         rds@corrTreatment <- mean(unlist(lapply(rds@treatmentBams, qwidth)))
         rds@corrControl <- mean(unlist(lapply(rds@controlBams, qwidth)))
      }
      rds@step <- 1
      return(rds)
   }       
)

setMethod("generateRoarsSingleBam", signature(rds="RoarDatasetMultipleAPA", 
                                              "RangedSummarizedExperiment",
                                              "RangedSummarizedExperiment"),
   function(rds, treatmentSE, controlSE)
   {
      # treatmentSE and controlSE are MoreArgs?
      #rds@roars <- mapply(createRoarSingleBAM, rds@fragments, rds@prePostDef,
      #                treatmentSE, controlSE)
      # Could be:
      rds@roars <- lapply(names(rds@fragments), createRoarSingleBam,
                          rds, treatmentSE, controlSE)
      names(rds@roars) <- names(rds@fragments)
      # to have the names...other ways? Store them in fragments or prePostDef
      # in an accessible way another time seems a waste of space.
      # Will have to compare times!
      return(rds)
   }
)

setMethod("generateRoarsMultipleBam", signature(rds="RoarDatasetMultipleAPA", 
                                              "list",
                                              "list"),
          function(rds, treatmentSE, controlSE)
          {
             # treatmentSE and controlSE are MoreArgs?
             #rds@roars <- mapply(createRoarSingleBAM, rds@fragments, rds@prePostDef,
             #                treatmentSE, controlSE)
             # Could be:
             rds@roars <- lapply(names(rds@fragments), createRoarMultipleBam,
                                 rds, treatmentSE, controlSE)
             names(rds@roars) <- names(rds@fragments)
             # to have the names...other ways? Store them in fragments or prePostDef
             # in an accessible way another time seems a waste of space.
             # Will have to compare times!
             return(rds)
          }
)

setMethod("computeRoars", signature(rds="RoarDatasetMultipleAPA"),
      function(rds) 
      {
         goOn <- checkStep(rds, 1)
         if (!goOn[[1]]) {
            return(rds)
         }
         rds <- goOn[[2]]
         rds@roars <- lapply(rds@roars, computeRoars, 
                             rds@corrTreatment, rds@corrControl)
         rds@step <- 2
         return(rds)
      }
)

setMethod("computePvals", signature(rds="RoarDatasetMultipleAPA"),
      function(rds)
      {
         goOn <- checkStep(rds, 2)
         if (!goOn[[1]]) {
            return(rds)
         }
         rds <- goOn[[2]]
         rds@roars <- lapply(rds@roars, computePvals)
         rds@step <- 3
         return(rds)
      }
)

setMethod("computePairedPvals", 
            signature(rds="RoarDatasetMultipleAPA",
            treatmentSamples="numeric", controlSamples="numeric"),
      function(rds, treatmentSamples, controlSamples) 
      {
         goOn <- checkStep(rds, 2)
         if (!goOn[[1]]) {
            return(rds)
         }
         rds <- goOn[[2]]
         rds@roars <- lapply(rds@roars, computePairedPvals, 
                             treatmentSamples, controlSamples)
         rds@step <- 3
         return(rds)
      }
)

setMethod("totalResults", signature(rds="RoarDatasetMultipleAPA"),
      function(rds) 
      {
         goOn <- checkStep(rds, 3)
         rds <- goOn[[2]]
         totRes <- lapply(rds@roars, totalResults)
         r <- do.call(rbind, totRes)
         rownames(r) <- unlist(lapply(names(totRes), function(x) {
                                 paste(x, rownames(totRes[[x]]), sep="_") }))
         return(r)
         # For names do.call/rbind added APA ids only when there were multiple
         # choices for the same gene. There could be best ways to add names.
      }
)

setMethod("countResults", signature(rds="RoarDatasetMultipleAPA"),
      function(rds) 
      {
         totRes <- totalResults(rds)
         # We cannot call fpkmResults on the roars objects because in this
         # case we want FPKM_pre for every different PRE choice. Therefore 
         # we work on every rds@roars with an ad hoc method that
         # gets PRE counts for every choice.
         countsList <- lapply(rds@roars, sumRoarCounts)
         countsDf <- do.call(rbind, countsList)
         rownames(countsDf) <- unlist(lapply(names(countsList), function(x) {
            paste(x, rownames(countsList[[x]]), sep="_") }))
         #t(sapply(rds@roars, s1)) # the same
         res <- merge(totRes, countsDf, by="row.names", sort=FALSE)
         rownames(res) <- res$Row.names
         res$Row.names <- NULL
         # FIXED: these lengths were wrong: lengths <- sapply(rds@fragments, function(x) { sum(mcols(x)$length)}) 
         # these were lengths of all the considered fragments but we are not able to retrieve their
         # correct counts at this point.
         # We need a dataframe with gene_apa as rownames and PRE lengths as values (one foreach choice for all genes).
         lens <- t(as.data.frame(lapply(names(rds@roars), function(x) {
            names <- gsub("_PRE", "", mcols(rds@roars[[x]])$gene_id)
            len <-mcols(rds@roars[[x]])$length
            t(data.frame(length=len, row.names=paste0(x, "_", names)))
            }), check.names=FALSE))
         res <- merge(res, lens, by="row.names", sort=FALSE)
         rownames(res) <- res$Row.names
         res$Row.names <- NULL
         return(res)
      }
)

setMethod("fpkmResults", signature(rds="RoarDatasetMultipleAPA"),
      function(rds) 
      {
         counts <- countResults(rds)
         sumTreatment <- sum(counts[,"counts_treatment"])
         sumControl <- sum(counts[,"counts_control"])
         counts$treatmentValue <- (counts[,"counts_treatment"]*1000000000)/(counts[,"length"]*sumTreatment)
         counts$controlValue <- (counts[,"counts_control"]*1000000000)/(counts[,"length"]*sumControl)
         counts$counts_control <- NULL
         counts$counts_treatment <- NULL
         return(counts)
      }
)

setMethod("pvalueFilter", signature(rds="RoarDatasetMultipleAPA", fpkmCutoff="numeric", pvalCutoff="numeric"),
      function(rds, fpkmCutoff, pvalCutoff) {
         dat <- standardFilter(rds, fpkmCutoff)
         if (nrow(dat) > 0) {
            resby <- by(dat, INDICES=as.factor(sapply(rownames(dat), function(x) unlist(strsplit(x,"_", fixed=T))[1])), 
                        FUN = function(x) {res<-x[with(x, order(x[,4])),]; res[1,]})
            dat <- do.call(rbind, resby)
            # For each gene we select the APA choice that is associated with the smallest p-value (after fpkm filtering)
            # then proceed exactly like roar [NDRY ALERT FIXME].
            if ((length(rds@treatmentBams) != 1 || length(rds@controlBams) != 1) && !rds@paired) {
               # In this case we add to dat a col that says how many comparisons yielded
               # a pvalue < pvalCutoff.
               # esany <- apply(data, 1, function(x) {any(x[seq(1,12)] < 0.05)})
               if(nrow(dat) != 0) {
                  cols <- grep("^pvalue_", colnames(dat))
                  sel <- apply(dat, 1, function(x) {x[cols] < pvalCutoff})
                  # This yields a transposed dat with cols rows and TRUE/FALSE. ncol = nrows of dat
                  dat$nUnderCutoff <- apply(sel, 2, function(x){length(x[x==TRUE])})
               }
            } else {
               dat <- dat[dat$pval < pvalCutoff,]
            }   
            return(dat)
         } else {
            return(data.frame())
         }
      }                  
)

setMethod("pvalueCorrectFilter", signature(rds="RoarDatasetMultipleAPA", fpkmCutoff="numeric", pvalCutoff="numeric", method="character"),
      function(rds, fpkmCutoff, pvalCutoff, method) 
      {
         dat <- standardFilter(rds, fpkmCutoff)
         if (nrow(dat) > 0) {   
            resby <- by(dat, INDICES=as.factor(sapply(rownames(dat), function(x) unlist(strsplit(x,"_", fixed=T))[1])), 
                        FUN = function(x) {res<-x[with(x, order(x[,4])),]; res[1,]})
            dat <- do.call(rbind, resby)
            if ((length(rds@treatmentBams) != 1 || length(rds@controlBams) != 1) && !rds@paired) {
               # In this case we add to dat a col that says how many comparisons yielded
               # a pvalue < pvalCutoff.
               # esany <- apply(data, 1, function(x) {any(x[seq(1,12)] < 0.05)})
               if(nrow(dat) != 0) {
                  cols <- grep("^pvalue_", colnames(dat))
                  sel <- apply(dat, 1, function(x) {x[cols] < pvalCutoff})
                  # This yields a transposed dat with cols rows and TRUE/FALSE. ncol = nrows of dat
                  dat$nUnderCutoff <- apply(sel, 2, function(x){length(x[x==TRUE])})
               }
            } else {
               dat$pval <- p.adjust(dat$pval, method=method)
               dat <- dat[dat$pval < pvalCutoff,]
            }   
            return(dat)
         } else {
            return(data.frame())
         }    
      }
)

setMethod("show", "RoarDatasetMultipleAPA",
          function(object) {
             cat("RoarDatasetMultipleAPA object\n")
             cat("N. of treatment alignments:", length(object@treatmentBams), "\n")
             cat("N. of control alignments:", length(object@controlBams), "\n")
             cat("N. of genes in study:", length(object@geneCoords) , "\n")
             cat("N. of cores:", object@cores, "\n")
             cat("Analysis step reached [0-3]:", object@step, "\n")
             cat("Paired (TRUE is still not implemented)?", object@paired, "\n")
             cat("\n")     
          }
)

test_that("test_getApaGenesFractionsPlusStrandBasic", {
    # Here we put two APA inside the first and last exon of a three exon gene.
    chr <- rep("chr1", 3)
    strand <- rep("+", 3)
    gene <- rep("A", 3)
    type <- rep("gene", 3)
    apa <- rep(NA, 3)
    geneGr <- GRanges(seqnames = chr, strand = strand, ranges = IRanges(start = c(1000, 
        2000, 3300), width = c(500, 900, 10)), DataFrame(gene, 
        apa, type))
    apas <- GRanges(seqnames = c(chr[1], chr[1]), strand = c(strand[1], 
        strand[1]), ranges = IRanges(start = c(1100, 3305), width = c(1, 
        1)), DataFrame(gene = rep(gene[1], 2), apa = paste0(c("apa1_", 
        "apa2_"), gene[1]), type = rep("apa", 2)))
    res <- getApaGenesFractionsPlusStrand(geneGr, apas, chr[1], 
        strand[1], gene[1])
    # Two possible apa choices.
    expect_equal(2, length(res)) 
    # Four fragments.
    expect_equal(4, length(res[[1]]))
    # Two info for which fragments need to be considered for the two choices.
    expect_equal(2, length(res[[2]]))
    # For the first APA choice we sum for PRE only the first fragment counts.
    expect_equal(1, res[[2]][[1]]@PREstart)
    expect_equal(1, res[[2]][[1]]@PREend)
    # For the second one up to the third (for POST we get all the remaining fragments counts).
    expect_equal(3, res[[2]][[2]]@PREstart)
    expect_equal(3, res[[2]][[2]]@PREstart)
    expect_equal("apa2", res[[2]][[2]]@name)
    expect_equal("apa1", res[[2]][[1]]@name)
    wanted <- GRanges(seqnames = rep(chr[1], 4), strand = rep(strand[1], 
        4), ranges = IRanges(start = c(1000, 1101, 3300, 3306), 
        width = c(101, 2199, 6, 4)), DataFrame(length = c(101, 
        1299, 6, 4)))
    expect_identical(res[[1]], wanted)
})

test_that("test_getApaGenesFractionsPlusStrandIntronExon", {
    chr <- "chr1"
    strand <- "+"
    gene <- "A"
    type <- "gene"
    apa <- NA
    geneGr <- GRanges(seqnames = rep(chr, 3), strand = rep(strand, 
        3), ranges = IRanges(start = c(1000, 2000, 3300), width = c(500, 
        900, 10)), DataFrame(rep(gene, 3), rep(apa, 3), rep(type, 
        3)))
    apas <- GRanges(seqnames = rep(chr, 2), strand = rep(strand, 
        2), ranges = IRanges(start = c(2010, 2901), width = c(1, 
        1)), DataFrame(gene = rep(gene, 2), apa = paste0(c("apa1_", 
        "apa2_"), gene), type = rep("apa", 2)))
    res <- getApaGenesFractionsPlusStrand(geneGr, apas, chr, 
        strand, gene)
    expect_equal(4, length(res[[1]]))
    expect_equal(2, length(res[[2]]))
    expect_equal(1, res[[2]][[1]]@PREstart)
    expect_equal(1, res[[2]][[1]]@PREend)
    expect_equal(1, res[[2]][[2]]@PREstart)
    expect_equal(2, res[[2]][[2]]@PREend)
    wanted <- GRanges(seqnames = rep(chr[1], 4), strand = rep(strand[1], 
        4), ranges = IRanges(start = c(2000, 2011, 2902, 3300), 
        width = c(11, 891, 398, 10)), DataFrame(length = c(11, 
        889, 0, 10)))
    expect_identical(res[[1]], wanted)
})

test_that("test_getApaGenesFractionsPlusStrandSingleIntron", 
    {
        chr <- "chr1"
        strand <- "+"
        gene <- "A"
        type <- "gene"
        apa <- NA
        geneGr <- GRanges(seqnames = rep(chr, 3), strand = rep(strand, 
            3), ranges = IRanges(start = c(1000, 2000, 3300), 
            width = c(500, 900, 10)), DataFrame(rep(gene, 3), 
            rep(apa, 3), rep(type, 3)))
        apas <- GRanges(seqnames = chr, strand = strand, ranges = IRanges(start = 1510, 
            width = 1), DataFrame(gene = gene, apa = paste0("apa1_", 
            gene), type = "apa"))
        res <- getApaGenesFractionsPlusStrand(geneGr, apas, chr, 
            strand, gene)
        expect_equal(3, length(res[[1]]))
        expect_equal(1, length(res[[2]]))
        expect_equal(1, res[[2]][[1]]@PREstart)
        expect_equal(1, res[[2]][[1]]@PREend)
        expect_equal("apa1", res[[2]][[1]]@name)
        wanted <- GRanges(seqnames = rep(chr, 3), strand = rep(strand, 
            3), ranges = IRanges(start = c(1000, 1511, 3300), 
            width = c(511, 1789, 10)), DataFrame(length = c(500, 
            900, 10)))
        expect_identical(res[[1]], wanted)
})

test_that("test_RoarDatasetMultipleAPA_error_mismatch_gene_apa", 
    {
        gene <- c("A", "B", NA)
        type <- c("gene", "gene", "apa")
        apa <- c(NA, NA, "apa1_A")
        features <- GRanges(seqnames = Rle(c("chr1", "chr2", 
            "chr1")), strand = strand(rep("+", length(gene))), 
            ranges = IRanges(start = c(1000, 2000, 1300), width = c(500, 
                900, 1)), DataFrame(gene, apa, type))
        rd1 <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(1000), 
            cigar = "300M", strand = strand("+"))
        obs <- tryCatch(RoarDatasetMultipleAPA(list(c(rd1, rd1)), 
            list(c(rd1, rd1)), features), error = function(e) e)
        expect_equal("All the genes in the gtf should have at least one apa", 
            obs$message)
})

test_that("test_badIntegrationTest", 
    {
       chr <- rep("chr1", 3)
       strand <- rep("+", 3)
       gene <- rep("A", 3)
       type <- rep("gene", 3)
       apa <- rep(NA, 3)
       geneGr <- GRanges(seqnames = chr, strand = strand, ranges = IRanges(start = c(1000, 
                                                                                     2000, 3300), width = c(500, 900, 10)), DataFrame(gene, apa, type))
       gene2 <- rep("B", 3)
       chr2 = rep("chr2", 3)
       geneGr2 <- GRanges(seqnames = chr2, strand = strand, ranges = IRanges(start = c(1000, 
                                                                                       2000, 3300), width = c(500, 900, 10)), DataFrame(gene=gene2, apa, type))
       apas1 <- GRanges(seqnames = c(chr[1], chr[1]), strand = c(strand[1], strand[1]), 
                        ranges = IRanges(start = c(1100, 3305), width = c(1, 1)), 
                        DataFrame(gene = rep(gene[1], 2), apa = paste0(c("apa1_", "apa2_"), gene[1]), type = rep("apa", 2)))
       apas2 <- GRanges(seqnames = c(chr2[1], chr2[1]), strand = c(strand[1], strand[1]), 
                        ranges = IRanges(start = c(1100, 3305), width = c(1, 1)), 
                        DataFrame(gene = rep(gene2[1], 2), apa = paste0(c("apa1_", "apa2_"), gene2[1]), type = rep("apa", 2)))
       
       # We have two "identical" genes with the same two apa as the first test on chr1 and chr2
       features <- unlist(GRangesList(geneGr, geneGr2, apas1, apas2)) 
       # c() was complaining for disjoint chr (seqinfo)
       
       # FIXME if cigar=300M nothing was seen! Due to the reads converted to single base!
       rd1 <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(3301), 
                          cigar = "1M", strand = strand("+"))
       rd2 <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(3301), 
                          cigar = "1M", strand = strand("-"))
       rd3 <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(3301), 
                          cigar = "1M", strand = strand("+"))
       rd4 <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(3306), 
                          cigar = "1M", strand = strand("+"))
       rd5 <- GAlignments("a", seqnames = Rle("chr2"), pos = as.integer(3306), 
                          cigar = "1M", strand = strand("+"))
       rd6 <- GAlignments("a", seqnames = Rle("chr2"), pos = as.integer(3306), 
                          cigar = "1M", strand = strand("-"))
       rd7 <- GAlignments("a", seqnames = Rle("chr2"), pos = as.integer(3306), 
                          cigar = "1M", strand = strand("+"))
       rd8 <- GAlignments("a", seqnames = Rle("chr2"), pos = as.integer(3301), 
                          cigar = "1M", strand = strand("+"))
       
       reads <- suppressWarnings(list(c(rd1, rd2, rd3, rd4, rd5, rd6, rd7, rd8))) 
       # suppressWarnings for the same issue about different chr as before.
       # We have 3 reads on the PRE of the second choice APA and 1 on the post for gene A and the opposite for gene B.
       rds <- RoarDatasetMultipleAPA(reads, reads, features)
       rds <- countPrePost(rds)
       counts_geneA <- matrix(c(0, 4, 0, 4, 3, 1, 3, 1), byrow = TRUE, ncol = 4)
       colnames(counts_geneA) <- c("treatment_pre","treatment_post","control_pre","control_post")
       expect_equal(counts_geneA, assay(rds@roars[[1]],1))
       counts_geneB <- matrix(c(0, 4, 0, 4, 1, 3, 1, 3), byrow = TRUE, ncol = 4)
       colnames(counts_geneB) <- c("treatment_pre","treatment_post","control_pre","control_post")
       expect_equal(counts_geneB, assay(rds@roars[[2]],1))
       rds <- computeRoars(rds)
       rds <- computePvals(rds)
       res <- countResults(rds)
       mM <- c(-1,1,-1,-0.77777777)
       counts <- c(0, 3, 0, 1)
       lengths <- c(101, 6, 101, 6)
       countsResults <- data.frame(mM_treatment = mM, mM_control= mM, roar=rep(1,4), pval=rep(1,4),
                                   counts_treatment=counts, counts_control=counts, length=lengths)
       rownames(countsResults) <- c("A_apa1","A_apa2","B_apa1", "B_apa2")
       expect_equal(countsResults, res)
       fpkm <- c(0, 125000000, 0, 41666667)
       countsResults$length <- lengths
       countsResults$treatmentValue <- fpkm
       countsResults$controlValue <- fpkm
       countsResults$counts_control <- NULL
       countsResults$counts_treatment <- NULL
       res <- fpkmResults(rds)
       expect_equal(countsResults, res)
})

test_that("test_badIntegrationTestMinusStrand", 
   {
      chr <- rep("chr1", 4)
      strand <- rep("-", 4)
      gene <- rep("1", 4)
      type <- rep("gene", 4)
      apa <- rep(NA, 4)
      geneGr <- GRanges(seqnames = chr, strand = strand, ranges = IRanges(start = c(2, 4, 12, 19), width = c(1,4, 4, 3)), DataFrame(gene, apa, type))
      apas1 <- GRanges(seqnames = c(chr[1], chr[1], chr[1]), strand = c(strand[1], strand[1], strand[1]), 
                       ranges = IRanges(start = c(5, 7, 17), width = c(1, 1, 1)), 
                       DataFrame(gene = rep(gene[1], 3), apa = paste0(c("apa1_", "apa2_","apa3_"), gene[1]), type = rep("apa", 3)))
      features <- unlist(GRangesList(geneGr, apas1))
      
      r18_2 <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(18), 
                         cigar = "1M", strand = strand("-"))
      r13_3 <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(13), 
                         cigar = "4M", strand = strand("-"))
      r7_10 <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(7), 
                         cigar = "1M", strand = strand("-"))
      r2_100 <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(2), 
                         cigar = "1M", strand = strand("-"))
      treatment_reads <- list(c(rep(r18_2,2), rep(r13_3,3), rep(r7_10, 10), rep(r2_100,100)))
      control_reads <- list(c(rep(r18_2,100), rep(r13_3,10), rep(r7_10, 3), rep(r2_100,2)))
      rds <- RoarDatasetMultipleAPA(treatment_reads, control_reads, features)
      rds <- countPrePost(rds)
      
      wanted_fragments_A <- GRanges(seqnames = rep(chr[1], 6), strand = rep(strand[1], 6), 
                                  ranges = IRanges(start = c(17,8,7,5,3,2), width = c(5, 9, 1, 2, 2, 1)), 
                                  DataFrame(length=c(3, 4, 1, 2, 1, 1)))
      expect_equal(wanted_fragments_A, rds@fragments[['1']])
      
      counts_geneA <- matrix(c(2, 113, 100, 15, 10, 100, 3, 2, 10, 100, 3, 2), byrow = TRUE, ncol = 4)
      colnames(counts_geneA) <- c("treatment_pre","treatment_post","control_pre","control_post")
      expect_equal(counts_geneA, assay(rds@roars[[1]],1))
      
      rds <- computeRoars(rds)
      rds <- computePvals(rds)
      res <- fpkmResults(rds)
      
      mM_t <- c(-0.946440938822624,-0.5921739130434782,-0.9307246376811594)
      mM_c <- c(19.579710144927535, 5.391304347826088, 0.1304347826086958)
      roars <- c(-0.04833784217524875, -0.10983870967741934, -7.135555555555548)
      fishers <- c(1.505474e-45, 0.01008237, 0.01008237)
      fpkm_c <- c(314465408.8050314, 28301886.79245283, 9433962.264150944)
      fpkm_t <- c(30303030.303030305, 454545454.54545456, 151515151.5151515)
      wanted_fpkmResults <- data.frame(mM_treatment = mM_t, mM_control = mM_c, roar = roars, pval = fishers, length=c(3,1,3),
                                       treatmentValue = fpkm_t, controlValue = fpkm_c)
      rownames(wanted_fpkmResults) <- c("1_apa3", "1_apa2", "1_apa1")
      expect_equal(wanted_fpkmResults, res, tolerance = .000002)
      
      # TODO ADD checks on filters to be safe from row order issues
})

#!/usr/bin/env Rscript
# Script to perform stepwise (chr by chr) Roar analysis. 
# Requires a gtf with _PRE and _POST gene_ids and bam files from the two 
# conditions to be compared.

checkReadable <- function(filename) {
   res <- file.access(names=filename, mode=4) == 0
   if (!res) {
      warning(paste(filename, "is not readable", sep=" "))
   }
   res
}

arguments <- matrix(c(
   'help', 'h', 0, "logical",
   'debug', 'd', 1, "character",
   'gtf' , 'a', 1, "character",
   'treatment'  , 't', 1, "character",
   'control'  , 'c', 1, "character"
), ncol=4, byrow=T)

library(getopt)
opt <- getopt(arguments)

if (!is.null(opt$help)) {
   stop(getopt(arguments, command=get_Rscript_filename(), usage=TRUE))
}

if (is.null(opt$gtf)) {
   stop("Missing gtf [-a filename] annotation option\n")
}

if (is.null(opt$treatment) | is.null(opt$control)) {
   stop("Missing treatment or control [-t, -c followed by comma separated bam files] param")
}

library(roar)
library(rtracklayer)
library(Rsamtools)
library(GenomicAlignments)

treatmentBams <- as.vector(unlist(strsplit(opt$treatment, ",")))
controlBams <- as.vector(unlist(strsplit(opt$control, ",")))

if (!all(sapply(c(treatmentBams, controlBams, opt$gtf), checkReadable))) {
   stop("One of the given files does not exist or is not readable")  
}


gtfGRanges<- import(opt$gtf)
chrs <- seqlevels(gtfGRanges)

orderBam <- function(bam) {
      tmp <- tempfile()
      ordered <- sortBam(bam, tmp, byQname=FALSE, maxMemory=512)
      garbage <- indexBam(ordered)
      return(ordered)
}

orderedTreatmentBams <- lapply(treatmentBams, orderBam)
orderedControlBams <- lapply(controlBams, orderBam)

workOnChr <- function(chr) {
   write(paste("Working on", chr), stderr())
   reduced <- keepSeqlevels(gtfGRanges, chr) 
   coords <- c(start(reduced), end(reduced))  # To keep strandness in consideration! Is this needed?
   begin <- min(coords)
   end <- max(coords)
   spanChr <- GRanges(seqnames=chr,ranges=IRanges(start=begin,width=end-begin+1))
   loadBam <- function(bam) {
      param <- ScanBamParam(which=spanChr)
      res <- readGAlignments(file=bam, param = param)
      return(res)
   } 

   treatmentBamsGenomicAlignments <- lapply(orderedTreatmentBams, loadBam)
   controlBamsGenomicAlignments <- lapply(orderedControlBams, loadBam)
   
   rds <- RoarDataset(treatmentBamsGenomicAlignments, controlBamsGenomicAlignments, reduced)
   
   # Get counts
   rds <- countPrePost(rds, FALSE)
   # Get m/M and Roar
   rds <- computeRoars(rds)
   # Fisher test
   rds <- computePvals(rds)
   res <- countResults(rds)
   return(res)
}

allRes <- lapply(chrs, workOnChr)
meltedRes <- do.call("rbind", allRes)
preElems <- grep("_PRE$", mcols(gtfGRanges)$gene_id)
pre <- gtfGRanges[preElems,]
preLen <- end(pre) - start(pre) + 1
names <- sub("^\\s+","",sub("_PRE", "",mcols(pre)$gene_id))
meltedRes <- meltedRes[match(names, rownames(meltedRes)),]
sumPreTreatment <- sum(meltedRes[,"treatmentValue"])
sumPreControl <- sum(meltedRes[,"controlValue"])
meltedRes$treatmentFpkm <- (meltedRes[,"treatmentValue"]*1000000000)/(preLen*sumPreTreatment)
meltedRes$controlFpkm <- (meltedRes[,"controlValue"]*1000000000)/(preLen*sumPreControl)
write.table(meltedRes, sep="\t", quote=FALSE)

unlink(orderedTreatmentBams)
unlink(orderedControlBams)
treatmentBai <- sub(".bam", ".bai", orderedTreatmentBams)
controlBai <- sub(".bam", ".bai", orderedControlBams)
unlink(treatmentBai)
unlink(controlBai)

if (!is.null(opt$debug)) {
   save.image(file=opt$debug)
}

#!/usr/bin/env Rscript
# Script to perform Roar analysis. 
# Requires a gtf with _PRE and _POST gene_ids and bam files from the two 
# conditions to be compared.

roarAnalysis <- function(gtf, treatmentBams, controlBams)
{
   ## This will need to be run foreach gene and foreach PRE/POST choice for them.
   # Get counts
   rds <- RoarDataset(treatmentBams, controlBams, gtf)
   rds <- countPrePost(rds, FALSE)
   
   # Get m/M and Roar
   rds <- computeRoars(rds)
   
   # Fisher test
   rds <- computePvals(rds)
   
   results <- fpkmResults(rds)
   apa_used <- unique(mcols(gtf)$apa_used)
   return(cbind(apa_used, results))
   
   #return(data.frame(matrix(c(rnorm(2)), nrow=1, ncol=2), row.names=apa_used))
   
   # filteredResults <- standardFilter(rds, fpkmCutoff=1)
   # write.table(filteredResults, sep="\t", quote=FALSE)
   
   # pvals <- pvalueFilter(rds, fpkmCutoff = 1, pvalCutoff = 0.05)
   # write.table(pvals, sep="\t", quote=FALSE)
}

# A function that puts together all roar results (or choose among them)
# calling them with every possible PRE/POST choice
callRoar <- function(gtf, treatmentBams, controlBams)
{
   return(lapply(gtf, roarAnalysis, treatmentBams, controlBams))
}

# A function that for a gene and its overlapping apas producesa
# a GRangesList object with all the choices of PRE/POST as GRanges. 
getAllPrePost <- function(geneGr, apaGr)
{
   apaGr <- sort(apaGr)
   strand <- unique(as.character(strand(geneGr)))
   chr <- unique(as.character(seqnames(geneGr)))
   gene_id <- unique(as.character(mcols(geneGr)$gene))
   if (length(chr) != 1) {
      stop("A gene is on different chrs")
   }
   if (length(gene_id) != 1) {
      stop("A gene has different names")
   }
   if (strand == '+') {
      apas <- head(apaGr, n=length(apaGr)-1)
      last <- tail(apaGr, n=1)
   } else if (strand == '-') {
      apas <- tail(apaGr, n=length(apaGr)-1)
      last <- head(apaGr, n=1)
   } else {
      stop("A gene has no strand info or both + and -")
   }
   res <- lapply(apas, FUN=definePrePost, last, geneGr, strand, chr, gene_id)
   return(GRangesList(res))
}

definePrePost <- function(firstApa, secondApa, geneGr, strand, chr, gene_id)
{
   introns <- gaps(geneGr)
   # We remove the first intron (from the beginning of the chr)
   introns <- tail(introns, n=length(introns)-1)
   mcols(geneGr) <- NULL
   mcols(geneGr)$type <- 'e'
   if (length(introns) != 0) {
      mcols(introns)$type <- 'i'  
   }
   whole <- sort(c(geneGr, introns))
   hitsPre <- findOverlaps(whole, firstApa)
   #hitsPost <- findOverlaps(whole, secondApa)
   # We do not map the secondAPA: we cannot be sure that it has to map (it could
   # fall outside our gene in theory).
   foundPre <- NA
   if (!(countSubjectHits(hitsPre) == 1)) {
      # Then also the PRE is outside our gene. We use the last exon.
      #stop("Error: a given apa does not overlap its gene", 
      #     print(geneGr), print(firstApa))
      foundPre <- length(whole)
      if (strand == "-") {
         foundPre <- 1
      }
      if (whole[foundPre]$type != "e") {
         stop("Two apas outside the gene and I am unable to find the last exon", print(geneGr))
      }
   } else {
      foundPre <- queryHits(hitsPre)
   }
   # strand == "+"
   # XXX TODO reason about +-1 at begins and ends XXX TODO
   endPost <- start(secondApa)
   startPost <- end(firstApa) +1
   # We add 1 to avoid overlapping PRE/POST - APA are considered before the cut.
   # In the previous gtf we skipped the "cut" base altogether.
   startPre <- NA
   if (mcols(whole[foundPre])$type =="e") {
      startPre <- start(whole[foundPre])
      if (strand == "-") {
         startPre <- end(whole[foundPre])
      }
   } else { # if (mcols(foundPre)$type =="i") # true by costruction
      # We should not fall over boundaries if all preconditions are met.
      previousExon <- foundPre-1
      startPre <- start(whole[previousExon])
      if (strand == "-") {
         previousExon <- foundPre+1
         startPre <- end(whole[previousExon])
      }
   }
   endPre <- start(firstApa)
   if (strand == "-") {
      # We subtract 1 to avoid overlapping PRE/POST 
      # - APA are considered before the cut.
      # In the previous gtf we skipped the "cut" base altogether.
      endPost <- start(firstApa)-1
      startPost <- end(secondApa)
      sw <- startPre
      startPre <- endPre
      endPre <- sw
   }
   firstApa_name <- unlist(strsplit(mcols(firstApa)$apa, '_', fixed=TRUE))[1]
   names <- unlist(strsplit(mcols(secondApa)$apa, '_', fixed=TRUE))
   secondApa_name <- names[1]
   apa_names <- paste(firstApa_name, secondApa_name, sep="-")
   gene <- names[2]
   apa_names <- paste(gene, apa_names, sep="_")
   post <- GRanges(seqnames=chr, strand=strand, 
                   ranges=IRanges(start=startPost, end=endPost), 
                   gene_id=paste(gene_id, "POST", sep="_"),
                   apa_used=apa_names)
   pre <- GRanges(seqnames=chr, strand=strand, 
                   ranges=IRanges(start=startPre, end=endPre), 
                   gene_id=paste(gene_id, "PRE", sep="_"),
                   apa_used=apa_names)
   return(c(pre, post))
}

checkReadable <- function(filename) 
{
   res <- file.access(names=filename, mode=4) == 0
   if (!res) {
      warning(paste(filename, "is not readable", sep=" "))
   }
   res
}

printResults <- function(listRes)
{
   write.table(do.call(rbind, listRes), sep="\t", quote=FALSE, col.names=FALSE)
}

arguments <- matrix(c(
   'help', 'h', 0, "logical",
   'debug', 'd', 1, "character",
   'gtf' , 'a', 1, "character",
   'treatment'  , 't', 1, "character",
   'control'  , 'c', 1, "character"
), ncol=4, byrow=T)

library(getopt)
opt <- getopt(arguments)

if (!is.null(opt$help)) {
   stop(getopt(arguments, command=get_Rscript_filename(), usage=TRUE))
}

if (is.null(opt$gtf)) {
   stop("Missing gtf [-a filename] annotation option\n")
}

if (is.null(opt$treatment) | is.null(opt$control)) {
   stop("Missing treatment or control",
         "[-t, -c followed by comma separated bam files] param")
}

library(roar)
library(rtracklayer)
library(GenomicAlignments)
treatmentBams <- as.vector(unlist(strsplit(opt$treatment, ",")))
controlBams <- as.vector(unlist(strsplit(opt$control, ",")))
if (!all(sapply(c(treatmentBams, controlBams, opt$gtf), checkReadable))) {
   stop("One of the given files does not exist or is not readable")  
}
treatmentBamsGenomicAlignments <- lapply(treatmentBams, readGAlignments)
controlBamsGenomicAlignments <- lapply(controlBams, readGAlignments)

gtfGRanges<- import(opt$gtf, asRangedData=FALSE)
apas_melted <- gtfGRanges[mcols(gtfGRanges)$type=="apa"]
genes_melted <- gtfGRanges[mcols(gtfGRanges)$type=="gene"]
mcols(apas_melted)$gene <- as.numeric(sapply(
                                 strsplit(mcols(apas_melted)$apa, '_', 
                                          fixed=TRUE),
                                 function(x) { x[length(x)]}))
genes_ids <- sort(unique(mcols(genes_melted)$gene))
genes_ids_apas <- sort(unique(mcols(apas_melted)$gene))
if (!all(genes_ids==genes_ids_apas)) {
   stop("Lists of GAlignments could not be empty")
}
genes <- do.call(GRangesList, sapply(genes_ids, 
                                    function(x) {
                                       genes_melted[mcols(genes_melted)$gene==x]
                                    }))
apas <- do.call(GRangesList, sapply(genes_ids, 
                                    function(x) {
                                       apas_melted[mcols(apas_melted)$gene==x]
                                    }))
names(apas) <- genes_ids
names(genes) <- genes_ids

# We want a list of GRangesList: foreach gene a GRangesList 
# object with pre/post. 
# Are they sorted by name? They should be after the sapply. 
# XXX Check if that's true: a shallow check said yes.
all_pre_post <- mapply(getAllPrePost, genes, apas)
# Foreach list we have a function that puts together all roar results 
# (or choose among them)
# calling them with every possible PRE/POST choice.
res <- lapply(all_pre_post, callRoar, 
               treatmentBamsGenomicAlignments, controlBamsGenomicAlignments)
# A list with n. elements == n. genes, elements are list of results for every
# PRE/POST choice for the gene.
garbage <- lapply(res, printResults)

if (!is.null(opt$debug)) {
   save.image(file=opt$debug)
}

# This class will hold a GenomicAnnotation (pre-post GTF) and count results in a RangedSummarizedExperiment
setClass("RoarDataset",
          contains = "RangedSummarizedExperiment", # Superclasses
          representation = representation( # Slots
             treatmentBams = "list",  #list of GAlignments 
             controlBams = "list",
             prePostCoords = "GRanges",
             postCoords = "GRanges",
             countsTreatment = "RangedSummarizedExperiment",
             countsControl = "RangedSummarizedExperiment",
             pVals = "RangedSummarizedExperiment",
             step = "numeric",
             paired = "logical",
             cores = "numeric"
          )
)
#validity=function(roc) length(roc@sens)==length(roc@mspec)
#          && length(roc@sens)==length(roc@test)
# check validity of kind of alignments, existence of PRE/POST ids -> moved in other methods.
# prototype -> default values

setClass("RoarDatasetMultipleAPA",
         representation = representation( # Slots
            treatmentBams = "list",  #list of GAlignments 
            controlBams = "list",
            geneCoords = "GRangesList",
            apaCoords = "GRangesList",
            fragments = "GRangesList",
            prePostDef = "list",
            roars = "list",
            step = "numeric",
            paired = "logical",
            corrTreatment = "numeric",
            corrControl = "numeric",
            cores = "numeric"
         ),
         prototype = list(
            geneCoords = GRangesList(),
            apaCoords = GRangesList(),
            fragments = GRangesList()
         )
)

setClass("ApaFragmentPrePost",
         representation = representation (
            name = "character",
            PREstart = "numeric",
            PREend = "numeric"
         )
)

getFisher <- function(counts) {
   # treatment_pre treatment_post control_pre control_post
   mat <- matrix(counts, ncol=2, byrow=FALSE)
   # Reminders:
   #colnames(mat) <- c('treatment', 'control')
   #rownames(mat) <- c('PRE', 'POST')
   f <- fisher.test(as.table(mat), alternative="two.sided")
   return(f$p.value)
}

combineFisherMethod <- function(pvals) {
   # Computes the combined pvalue from a list of independent tests resulting pvalues
   # following the fisher method.
   # http://en.wikipedia.org/wiki/Fisher%27s_method
   chisq <- -2*sum(log(pvals))
   return(1-pchisq(chisq, df = 2*length(pvals)))
}

checkStep <- function(rds, neededStep) {
   if (rds@step > neededStep) { # Already done, not repeating. Give a reset method?
      warning("Not repeating a step already done")
      return(c(FALSE, rds)) # We hope for promotion.
   } else if (rds@step < neededStep) { # Something is missing.
      if (neededStep != rds@step) {
         if (neededStep == 1) {
            warning("Automatically calling countPrePost")
            rds <- countPrePost(rds, FALSE)    
         } else if (neededStep == 2) {
            warning("Automatically calling computeRoars")
            rds <- computeRoars(rds)
         } else if (neededStep == 3) {
            warning("Automatically calling computePvals")
            rds <- computePvals(rds)
         }
      }
   } 
   return(c(TRUE, rds))
}

meanAcrossAssays <- function(assays, wantedColumns) {
   # Is the conversion to dataframe slow? Is there a more efficient way without
   # changing the SE/Assays structure?
   wantedCols <- lapply(assays, function(x) { x[,wantedColumns] } )
   return(rowMeans(as.data.frame(wantedCols)))
}

getApaGenesFractions <- function(geneGr, apaGr)
{
   strand <- unique(as.character(strand(geneGr)))
   chr <- unique(as.character(seqnames(geneGr)))
   gene_id <- unique(as.character(mcols(geneGr)$gene))
   if (length(chr) != 1) {
      stop("A gene is on different chrs")
   }
   if (length(gene_id) != 1) {
      stop("A gene has different names")
   }
   if (strand == '+') {
      getApaGenesFractionsPlusStrand(geneGr, apaGr, chr, strand, gene_id)
   } else if (strand == '-') {
      getApaGenesFractionsMinusStrand(geneGr, apaGr, chr, strand, gene_id)
   } else {
      stop("A gene has no strand info or both + and -")
   } 
}

getApaGenesFractionsPlusStrand <- function(geneGr, apaGr, chr, strand, gene_id)
{   
   geneGr <- sort(geneGr)
   apaGr <- sort(apaGr)
   seqlengths(seqinfo(geneGr))<-rep(NA, length(seqinfo(geneGr)))
   introns <- gaps(geneGr)
   # Remove first intron: it is not useful in any way.
   # TODO: if there are chr lenghts we will get also a last intron? Yes. Bad fix: remove them.
   introns <- tail(introns, n=length(introns)-1) # TODO add check to avoid removing introns if first exon begin == 0/1
   fakeApaEndGene <- GRanges(
      seqnames = chr,
      strand = strand,
      ranges = IRanges(
         start = end(tail(geneGr, n=1)),
         width = 1,
         names = ""
      )
   )
   mcols(fakeApaEndGene) <- mcols(apaGr)[1,]
   mcols(fakeApaEndGene)[1,"apa"] <- ""
   apaGr <- c(apaGr, fakeApaEndGene)
   apaFragmentsPrePost <- vector("list", length(apaGr)-1)
   apaFrI <- 1
   # We start by doing it iteratively and non R/Bioc stylishly because I've got
   # no idea how. 
   # New idea: we combine exons and introns adding them mcols to their kind
   # and with info about being overlappant or not.
   # Then we run over this GRanges that should have all the needed info
   # to generate a GRanges res with intervals and maybe also a list of 
   # pre/post indexes for every APA.
   mcols(geneGr) <- NULL
   mcols(geneGr)$type <- 'e'
   if (length(introns) != 0) {
      mcols(introns)$type <- 'i'  
   }   
   whole <- sort(c(geneGr, introns))
   hits <- findOverlaps(whole, apaGr)
   if (!all(countSubjectHits(hits) == 1)) {
      stop("Error: a given apa does not overlap its gene or an apa is not of width 1")
   }
   
   foundov <- length(unique(whole[queryHits(hits)]))
   mcols(whole)$overlap <- rep(FALSE, length(whole))   
   mcols(whole[unique(queryHits(hits))])$overlap <- rep(TRUE, foundov)
   # Growing objects sin.
   begins <- c()
   ends <- c()
   begin <- -1
   lastExonB <- -1
   # Write logic and invariant here: begin is always the beginning of the last
   # needed fragment (i.e. start of exon with an overlap or with an overlap in
   # the next intron or just seen apa). We add an end when 
   # we get an exon/intron with overlapping apas, and a begin for exons
   # with overlap or preceding an overlapping intron.
   for (i in 1:length(whole)) {
      if (whole[i]$type=='e') {
         if (whole[i]$overlap || (i+1<=length(whole) && whole[i+1]$overlap)) {
            if (begin != -1) {
               begins <- c(begins, begin)
               ends <- c(ends, end(whole[i-1]))
            }
            begin <- start(whole[i])
            lastExonB <- length(begins)+1
         }
      }
      if (whole[i]$overlap) {
         overlapping_apas <- apaGr[subjectHits(hits[queryHits(hits)==i])]
         # Here we have a shot of writing the pre portions of counts
         # for each APA.
         for (j in 1:length(overlapping_apas)) {
            begins <- c(begins, begin)
            ends <- c(ends, start(overlapping_apas[j]))
            # We add 1 to avoid overlapping PRE/POST fragments 
            # - APA are considered before the cut.
            # In the previous gtf we skipped the "cut" base altogether.
            begin <- end(overlapping_apas[j])+1
            firstApa_name <- unlist(
               strsplit(mcols(overlapping_apas[j])$apa, '_', fixed=TRUE))[1]
            apa_name <- firstApa_name
            apaFragmentsPrePost[[apaFrI]] <- new("ApaFragmentPrePost",
                                                   name=apa_name,
                                                   PREstart=lastExonB, 
                                                   PREend=length(begins))
            apaFrI <- apaFrI+1
         }
      }
   }
   fragments <- GRanges(seqnames=chr, strand=strand, 
                        ranges=IRanges(start=begins, end=ends))
   ovHits <- findOverlaps(fragments, geneGr)
   ovLen <- width(overlapsRanges(ranges(fragments), ranges(geneGr), hits=ovHits))
   mcols(fragments)$length <- rep(0, length(fragments))
   # A single fragment can overlap multiple exons and we need to sum here their lengths:
   # i.e. ovlen is longer than length(fragments), with queryHits we get the right reps but
   # not sums. The only way I am able to imagine a solution is with a for.
   indexes_fr <- queryHits(ovHits)
   for (k in 1:length(ovLen)) {
      mcols(fragments[indexes_fr[k]])$length  <- mcols(fragments[indexes_fr[k]])$length + ovLen[k]
   }
   # The last apaFragment is build from a single apa (end of gene) only and we don't want it
   #fragments <- head(fragments, n=length(fragments)-1)
   # But the fragment is ok!
   apaFragmentsPrePost <- head(apaFragmentsPrePost,
                              n=apaFrI-2)
   return(list(fragments, apaFragmentsPrePost))
}
# Testing will be hideous.

# Strand -. We are being not DRY but i prefer to start in this way and refactor
# after the first runs/tests. During active development I will have to remember
# to change code (if needed) in two places...but right now I'm not sure
# about how many differences I need to insert in the code.
getApaGenesFractionsMinusStrand <- function(geneGr, apaGr, chr, strand, gene_id)
{   
   geneGr <- sort(geneGr, decreasing=TRUE)
   apaGr <- sort(apaGr, decreasing=TRUE)
   introns <- sort(gaps(geneGr), decreasing=TRUE)
   seqlengths(seqinfo(geneGr))<-rep(NA, length(seqinfo(geneGr)))
   # Remove last intron: it is not useful in any way.
   introns <- head(introns, n=length(introns)-1)
   fakeApaEndGene <- GRanges(
      seqnames = Rle(c(chr)),
      strand = strand,
      ranges = IRanges(
         start= start(tail(geneGr, n=1)),
         width= 1,
         names = ""
      )
   )
   mcols(fakeApaEndGene) <- mcols(apaGr)[1,]
   mcols(fakeApaEndGene)[1,"apa"] <- ""
   apaGr <- c(apaGr, fakeApaEndGene)
   apaFragmentsPrePost <- vector("list", length(apaGr)-1)
   apaFrI <- 1
   # We start by doing it iteratively and non R/Bioc stylishly because I've got
   # no idea how. 
   # New idea: we combine exons and introns adding them mcols to their kind
   # and with info about being overlappant or not.
   # Then we run over this GRanges that should have all the needed info
   # to generate a GRanges res with intervals and maybe also a list of 
   # pre/post indexes for every APA.
   mcols(geneGr) <- NULL
   mcols(geneGr)$type <- 'e'   
   if (length(introns) != 0) {
      mcols(introns)$type <- 'i'  
   }   
   whole <- sort(c(geneGr, introns), decreasing=TRUE)
   hits <- findOverlaps(whole, apaGr)
   if (!all(countSubjectHits(hits) == 1)) {
      stop("Error: a given apa does not overlap its gene or an apa is not of width 1")
   }
   
   foundov <- length(unique(whole[queryHits(hits)]))
   mcols(whole)$overlap <- rep(FALSE, length(whole))   
   mcols(whole[unique(queryHits(hits))])$overlap <- rep(TRUE, foundov)
   # Growing objects sin.
   begins <- c()
   ends <- c()
   begin <- -1
   lastExonB <- -1
   # Write logic and invariant here: begin is always the beginning of the last
   # needed fragment (i.e. start of exon with an overlap or with an overlap in
   # the next intron or just seen apa). We add an end when 
   # we get an exon/intron with overlapping apas, and a begin for exons
   # with overlap or preceding an overlapping intron.
   for (i in 1:length(whole)) {
      if (whole[i]$type=='e') {
         if (whole[i]$overlap || (i+1<=length(whole) && whole[i+1]$overlap)) {
            if (begin != -1) {
               begins <- c(begins, begin)
               ends <- c(ends, start(whole[i-1]))
            }
            begin <- end(whole[i])
            lastExonB <- length(begins)+1
         }
      }
      if (whole[i]$overlap) {
         overlapping_apas <- apaGr[subjectHits(hits[queryHits(hits)==i])]
         overlapping_apas <- sort(overlapping_apas, decreasing=TRUE)
         # Here we have a shot of writing the pre portions of counts
         # for each APA.
         for (j in 1:length(overlapping_apas)) {
            begins <- c(begins, begin)
            ends <- c(ends, start(overlapping_apas[j]))
            # We add 1 to avoid overlapping PRE/POST fragments 
            # - APA are considered before the cut.
            # In the previous gtf we skipped the "cut" base altogether.
            begin <- end(overlapping_apas[j])-1
            firstApa_name <- unlist(
               strsplit(mcols(overlapping_apas[j])$apa, '_', fixed=TRUE))[1]
            apa_name <- firstApa_name
            apaFragmentsPrePost[[apaFrI]] <- new("ApaFragmentPrePost",
                                                 name=apa_name,
                                                 PREstart=lastExonB, 
                                                 PREend=length(begins))
            apaFrI <- apaFrI+1
         }
      }
   }
   # here begins and ends are to be switched.
   fragments <- GRanges(seqnames=chr, strand=strand, 
                        ranges=IRanges(start=ends, end=begins))
   ovHits <- findOverlaps(fragments, geneGr)
   ovLen <- width(overlapsRanges(ranges(fragments), ranges(geneGr), hits=ovHits))
   
   mcols(fragments)$length <- rep(0, length(fragments))
   
   # A single fragment can overlap multiple exons and we need to sum here their lengths:
   # i.e. ovlen is longer than length(fragments), with queryHits we get the right reps but
   # not sums. The only way I am able to imagine a solution is with a for.
   indexes_fr <- queryHits(ovHits)
   for (k in 1:length(ovLen)) {
      mcols(fragments[indexes_fr[k]])$length  <- mcols(fragments[indexes_fr[k]])$length + ovLen[k]
   }
   #mcols(fragments[queryHits(ovHits)])$length  <- ovLen # But for length is here XXX FIXME
   
   # The last apaFragment is build from a single apa only and we don't want it
   #fragments <- head(fragments, n=length(fragments)-1)
   # But the fragment is ok!
   apaFragmentsPrePost <- head(apaFragmentsPrePost,
                               n=apaFrI-2)
   return(list(fragments, apaFragmentsPrePost))
}


obtainPrePost <- function(prepost, fragments)
{
   sn <- unique(seqnames(fragments))
   strand <- unique(strand(fragments)) 
   # We have already checked about strand uniqueness.
   gene_id=c(paste0(prepost@name, "_PRE"),
             paste0(prepost@name, "_POST"))
   # We do not really need the right coords, but still: I was fool.
   # We need them to correct in the right way for lengths!
   if (strand == "+") {
      # This logic does not work for minus strand.
      res <- GRanges(seqnames=rep(sn,2), # 22% of the time here?
                     strand=rep(strand,2),
            ranges=IRanges(start=c(start(fragments[prepost@PREstart]),
                                   start(fragments[prepost@PREend+1])),
                           end=c(end(fragments[prepost@PREend]),
                                 end(tail(fragments, n=1)))))
      length <- c(sum(mcols(fragments[seq(prepost@PREstart, prepost@PREend)])$length),
                             sum(mcols(fragments[seq(prepost@PREend+1, length(fragments))])$length))
   } else { # if (strand == "-") { # and also only + and -
      res <- GRanges(seqnames=rep(sn,2), # 22% of the time here?
                     strand=rep(strand,2),
                     ranges=IRanges(start=c(start(fragments[prepost@PREend]),
                                            start(tail(fragments, n=1))),
                                    end=c(end(fragments[prepost@PREstart]),
                                          end(fragments[prepost@PREend+1]))))
      length <- c(sum(mcols(fragments[seq(prepost@PREstart, prepost@PREend)])$length),
                  sum(mcols(fragments[seq(prepost@PREend+1, length(fragments))])$length))
   }
   mcols(res) <- DataFrame(gene_id, length)
   return(res)
}

sumFragmentCounts<- function(prepost, counts, kind)
{
   if (kind == "pre") {
      return(sum(counts[c(seq(prepost@PREstart,prepost@PREend, by=1))]))
   } else if (kind == "post") {
      return(sum(counts[c(seq(prepost@PREend+1,length(counts), by=1))]))
   } else {
      stop ("Inner error in helper functions to sum counts")
   }
}

createRoarSingleBam <- function(name, mulRds, treatmentSE, controlSE)
{
   prePostDef <- mulRds@prePostDef[[name]]
   fragments <- mulRds@fragments[[name]]
   #prePostCoords = "GRanges",
   #postCoords = "GRanges",
   #countsTreatment = "RangedSummarizedExperiment",
   #countsControl = "RangedSummarizedExperiment",
   prePostCoords <- do.call(c, sapply(prePostDef, obtainPrePost, fragments))
   # We need to extract the right values from treatmentSE/controlSE, should
   # clearly be more efficient than looking for the coords in fragments.
   #  assays(treatmentSE,1)$counts[rownames(assays(treatmentSE,1)$counts)=="10771"]
   postElems <- grep("_POST$", mcols(prePostCoords)$gene_id)
   preElems <- grep("_PRE$", mcols(prePostCoords)$gene_id)
   preCoords <- prePostCoords[preElems,]
   rds <- new("RoarDataset", 
              SummarizedExperiment(rowRanges=preCoords,
                                   colData=DataFrame(row.names=c("treatment_pre","treatment_post","control_pre", "control_post"))),
              prePostCoords=prePostCoords)
   rds@postCoords <- rds@prePostCoords[postElems,]
   se <- SummarizedExperiment(assays = rep(list(matrix(nrow=length(rds@prePostCoords)/2, ncol=4)),2),
                              rowRanges=preCoords, 
                              colData=DataFrame(row.names=c("treatment_pre","treatment_post","control_pre", "control_post"))
   )
   # Other than assay rownames we could have used:
   #> rownames(assays(pada,1)$counts)
   #[1] "100996928" "100996928" "101927572" "101927572" "104909134" "104909134"
   #[7] "9"         "9"         "10"        "10"        "12"        "12"       
   #> names(rowRanges(pada))
   #[1] "100996928" "100996928" "101927572" "101927572" "104909134" "104909134"
   #[7] "9"         "9"         "10"        "10"        "12"        "12"      
   #treatment <- assays(treatmentSE,1)$counts[rownames(assays(treatmentSE,1)$counts)==name]
   # > assays(pada,1)$counts[names(rowRanges(pada))=="12"]
   # [1] 0 0
   # But which is faster?
   control <- assays(controlSE,1,withDimnames=TRUE)$counts[rownames(assays(controlSE,1,withDimnames=TRUE)$counts)==name]
   treatment <- assays(treatmentSE,1,withDimnames=TRUE)$counts[rownames(assays(treatmentSE,1,withDimnames=TRUE)$counts)==name]
   assay(se,1, withDimnames=TRUE)[,"treatment_pre"] <- sapply(prePostDef, sumFragmentCounts, treatment, "pre")
   assay(se,1, withDimnames=TRUE)[,"treatment_post"] <- sapply(prePostDef, sumFragmentCounts, treatment, "post")
   assay(se,1, withDimnames=TRUE)[,"control_pre"] <- sapply(prePostDef, sumFragmentCounts, control, "pre")
   assay(se,1, withDimnames=TRUE)[,"control_post"] <- sapply(prePostDef, sumFragmentCounts, control, "post")
   rowRanges(rds) <- rowRanges(se)
   colData(rds) <- colData(se)
   assays(rds, withDimnames=TRUE) <- assays(se, withDimnames=TRUE)
   names(assays(rds,withDimnames=TRUE)) <- "counts"
   rds@step <- 1
   # length should be preserved: XXX multipleSamples!
   rds@treatmentBams <- list(NA)
   rds@controlBams <- list(NA)
   rds@cores <- 1
   return(rds)
}

createRoarMultipleBam <- function(name, mulRds, treatmentSE, controlSE)
{
   prePostDef <- mulRds@prePostDef[[name]]
   fragments <- mulRds@fragments[[name]]
   #prePostCoords = "GRanges",
   #postCoords = "GRanges",
   #countsTreatment = "RangedSummarizedExperiment",
   #countsControl = "RangedSummarizedExperiment",
   prePostCoords <- do.call(c, sapply(prePostDef, obtainPrePost, fragments))
   # We need to extract the right values from treatmentSE/controlSE, should
   # clearly be more efficient than looking for the coords in fragments.
   #  assays(treatmentSE,1)$counts[rownames(assays(treatmentSE,1)$counts)=="10771"]
   postElems <- grep("_POST$", mcols(prePostCoords)$gene_id)
   preElems <- grep("_PRE$", mcols(prePostCoords)$gene_id)
   preCoords <- prePostCoords[preElems,]
   rds <- new("RoarDataset", 
              SummarizedExperiment(rowRanges=preCoords,
                                   colData=DataFrame(row.names=c("treatment_pre","treatment_post","control_pre", "control_post"))),
              prePostCoords=prePostCoords)
   rds@postCoords <- rds@prePostCoords[postElems,]
   se <- SummarizedExperiment(assays = rep(list(matrix(nrow=length(rds@prePostCoords)/2, ncol=4)),2),
                              rowRanges=preCoords, 
                              colData=DataFrame(row.names=c("treatment_pre","treatment_post","control_pre", "control_post"))
   )
   rds@countsTreatment <- SummarizedExperiment(assays = rep(list(matrix(nrow=length(rds@prePostCoords)/2, ncol=2)), length(mulRds@treatmentBams)),
                              rowRanges=preCoords, 
                              colData=DataFrame(row.names=c("pre","post"))
   )
   rds@countsControl <- SummarizedExperiment(assays = rep(list(matrix(nrow=length(rds@prePostCoords)/2, ncol=2)), length(mulRds@controlBams)),
                                           rowRanges=preCoords, 
                                           colData=DataFrame(row.names=c("pre", "post"))
   )
   
   for (i in 1:length(controlSE)) {
      control <- assays(controlSE[[i]],1,withDimnames=TRUE)$counts[rownames(assays(controlSE[[i]],1,withDimnames=TRUE)$counts)==name]
      assay(rds@countsControl,i,withDimnames=TRUE)[,"pre"] <- sapply(prePostDef, sumFragmentCounts, control, "pre")
      assay(rds@countsControl,i,withDimnames=TRUE)[,"post"] <- sapply(prePostDef, sumFragmentCounts, control, "post")
   }
   for (i in 1:length(treatmentSE)) {
      treatment <- assays(treatmentSE[[i]],1,withDimnames=TRUE)$counts[rownames(assays(treatmentSE[[i]],1,withDimnames=TRUE)$counts)==name]
      assay(rds@countsTreatment,i,withDimnames=TRUE)[,"pre"] <- sapply(prePostDef, sumFragmentCounts, treatment, "pre")
      assay(rds@countsTreatment,i,withDimnames=TRUE)[,"post"] <- sapply(prePostDef, sumFragmentCounts, treatment, "post")
   }
   rowRanges(rds) <- rowRanges(se)
   colData(rds) <- colData(se)
   assays(rds,withDimnames=TRUE) <- assays(se,withDimnames=TRUE)
   names(assays(rds,withDimnames=TRUE)) <- "counts"
   rds@step <- 1
   rds@treatmentBams <- vector(mode = "list", length = length(mulRds@treatmentBams))
   rds@controlBams <- vector(mode = "list", length = length(mulRds@controlBams))   
   rds@cores <- 1
   return(rds)
}

#!/usr/bin/env Rscript
# Script to perform Roar analysis. 
# Requires a gtf with _PRE and _POST gene_ids and bam files from the two 
# conditions to be compared.

checkReadable <- function(filename) {
   res <- file.access(names=filename, mode=4) == 0
   if (!res) {
      warning(paste(filename, "is not readable", sep=" "))
   }
   res
}

arguments <- matrix(c(
   'help', 'h', 0, "logical",
   'debug', 'd', 1, "character",
   'gtf' , 'a', 1, "character",
   'treatment'  , 't', 1, "character",
   'control'  , 'c', 1, "character"
), ncol=4, byrow=T)

library(getopt)
opt <- getopt(arguments)

if (!is.null(opt$help)) {
   stop(getopt(arguments, command=get_Rscript_filename(), usage=TRUE))
}

if (is.null(opt$gtf)) {
   stop("Missing gtf [-a filename] annotation option\n")
}

if (is.null(opt$treatment) | is.null(opt$control)) {
   stop("Missing treatment or control [-t, -c followed by comma separated bam files] param")
}

library(roar)
library(rtracklayer)
treatmentBams <- as.vector(unlist(strsplit(opt$treatment, ",")))
controlBams <- as.vector(unlist(strsplit(opt$control, ",")))

if (!all(sapply(c(treatmentBams, controlBams, opt$gtf), checkReadable))) {
   stop("One of the given files does not exist or is not readable")  
}

# Get counts
rds <- RoarDatasetFromFiles(treatmentBams, controlBams, opt$gtf)
rds <- countPrePost(rds, FALSE)

# Get m/M and Roar
rds <- computeRoars(rds)

# Fisher test
rds <- computePvals(rds)

results <- fpkmResults(rds)
write.table(results, sep="\t", quote=FALSE)
         
# filteredResults <- standardFilter(rds, fpkmCutoff=1)
# write.table(filteredResults, sep="\t", quote=FALSE)

# pvals <- pvalueFilter(rds, fpkmCutoff = 1, pvalCutoff = 0.05)
# write.table(pvals, sep="\t", quote=FALSE)

if (!is.null(opt$debug)) {
   save.image(file=opt$debug)
}

setGeneric("countPrePost", def=function(rds, stranded=FALSE) { standardGeneric("countPrePost") }, valueClass=c("RoarDataset", "RoarDatasetMultipleAPA"))
setGeneric("computeRoars", def=function(rds, qwidthTreatment = NA, qwidthControl = NA) { standardGeneric("computeRoars") }, valueClass=c("RoarDataset", "RoarDatasetMultipleAPA"))
setGeneric("computePvals", def=function(rds) { standardGeneric("computePvals") }, valueClass=c("RoarDataset", "RoarDatasetMultipleAPA"))
setGeneric("computePairedPvals", def=function(rds, treatmentSamples, controlSamples) { standardGeneric("computePairedPvals") }, valueClass=c("RoarDataset", "RoarDatasetMultipleAPA"))
setGeneric("totalResults", def=function(rds) { standardGeneric("totalResults") }, valueClass="data.frame")
setGeneric("fpkmResults", def=function(rds) { standardGeneric("fpkmResults") }, valueClass="data.frame")
setGeneric("sumRoarCounts", def=function(rds) { standardGeneric("sumRoarCounts") }, valueClass="data.frame")
setGeneric("countResults", def=function(rds) { standardGeneric("countResults") }, valueClass="data.frame")
setGeneric("standardFilter", def=function(rds, fpkmCutoff) { standardGeneric("standardFilter") }, valueClass="data.frame")
setGeneric("pvalueFilter", def=function(rds, fpkmCutoff, pvalCutoff) { standardGeneric("pvalueFilter") }, valueClass="data.frame")
setGeneric("pvalueCorrectFilter", def=function(rds, fpkmCutoff, pvalCutoff, method) { standardGeneric("pvalueCorrectFilter") }, valueClass="data.frame")
setGeneric("generateRoarsSingleBam", def=function(rds, treatmentSE, controlSE) {
   standardGeneric("generateRoarsSingleBam") }, valueClass="RoarDatasetMultipleAPA")
setGeneric("generateRoarsMultipleBam", def=function(rds, treatmentSE, controlSE) {
   standardGeneric("generateRoarsMultipleBam") }, valueClass="RoarDatasetMultipleAPA")
setGeneric("cores", def=function(rds) { standardGeneric("cores") }, valueClass="numeric")
setGeneric("sumRoarCounts", def=function(rds) { standardGeneric("sumRoarCounts") }, valueClass="data.frame")
#setGeneric("cores<-", def=function(rds, value) { standardGeneric("cores<-") })

# Methods for the RoarDataset class.
# R/AllGenerics.R contains the signatures of the not overriding methods.

RoarDatasetFromFiles <- function(treatmentBams, controlBams, gtf) {
   # The format will be assumed using the file extension. Will work everytime?
   # Do we need to force a genome(eg. hg19)? It doesn't seem so.
   gtfGRanges<- import(gtf)
   ordered <- order(mcols(gtfGRanges)$gene_id)
   gtfGRanges <- gtfGRanges[ordered]
   treatmentBamsGenomicAlignments <- lapply(treatmentBams, readGAlignments)
   controlBamsGenomicAlignments <- lapply(controlBams, readGAlignments)
   se <- getPreCoordsSE(gtfGRanges)
   new("RoarDataset", se, treatmentBams=treatmentBamsGenomicAlignments, controlBams=controlBamsGenomicAlignments, 
       prePostCoords=gtfGRanges, step=0, paired=FALSE, cores=1)
}

RoarDataset <- function(treatmentGappedAlign, controlGappedAlign, gtfGRanges) {
   if (length(treatmentGappedAlign) == 0 || length(controlGappedAlign) == 0) {
      stop("Lists of GAlignments could not be empty")
   }
   ordered <- order(mcols(gtfGRanges)$gene_id)
   gtfGRanges <- gtfGRanges[ordered]
   se <- getPreCoordsSE(gtfGRanges)
   new("RoarDataset", se, treatmentBams=treatmentGappedAlign, controlBams=controlGappedAlign, 
       prePostCoords=gtfGRanges, step=0, paired=FALSE, cores=1)
}

# Could have used setMethod("initialize", "xx",) but in this way should have had a gtf filename slot.
# setValidity instead of checks inside countPrePost? But this would be inefficient/add slots.

getPreCoordsSE <- function(gtfGRanges) {
   # Now we need to keep means and totals of counts over PRE/POST for the two lists.
   # In the simpler case with a single alignment for both conditions we just keep the counts.
   preElems <- grep("_PRE$", mcols(gtfGRanges)$gene_id)
   preCoords <- gtfGRanges[preElems,]
   se <- SummarizedExperiment(rowRanges=preCoords, colData=DataFrame(row.names=c("treatment_pre","treatment_post","control_pre", "control_post")))
   return(se)
}


# I removed stranded="logical" from the signature because it has a default value, which
# is also set in setGeneric.
setMethod("countPrePost", signature(rds="RoarDataset"),
   function(rds, stranded = FALSE) {
      #if (!is(rds, "RoarDataset")) {
      #   stop("countPrePost could be applied only to RoarDataset objects")
      #} # Why is this needed? Is it needed?
      # To me it does not seem nedeed.
      goOn <- checkStep(rds, 0)
      if (!goOn[[1]]) {
         return(rds)
      }
      rds <- goOn[[2]]
      summOv <- function(x) {
         summarizeOverlaps(features=rds@prePostCoords, reads=x, ignore.strand=!stranded, mc.cores=rds@cores)
      }
      summOvPost <- function(x) {
         summarizeOverlaps(features=rds@postCoords, reads=x, ignore.strand=!stranded, mc.cores=rds@cores)
      } 
      
      # Now we need to keep means and totals of counts over PRE/POST for the two lists.
      # In the simpler case with a single alignment for both conditions we just keep the counts.
      preElems <- grep("_PRE$", mcols(rds@prePostCoords)$gene_id)
      postElems <- grep("_POST$", mcols(rds@prePostCoords)$gene_id)
      if (length(preElems)+length(postElems) != length(mcols(rds@prePostCoords)$gene_id)) {
         stop("The prePostCoords given for this RoarDataset are wrong: some of the gene_id
              does not end in _PRE/_POST.")
      }
      if (length(preElems) != length(postElems)) {
         stop("The prePostCoords given for this RoarDataset are wrong: the number of PRE is different
              from the number of POST.")
      }
      # The number of elements has to be checked because some horrible special case with recycling
      # of vector elements in the comparison are possible.
      if (!all(sub("_PRE", "", mcols(rds@prePostCoords)$gene_id[preElems]) ==
                     sub("_POST", "", mcols(rds@prePostCoords)$gene_id[postElems]))) {
         stop("The prePostCoords given for this RoarDataset are wrong: not all prefixes of PRE-POST
              correspond.")
      }
      # Check uniqueness of gene_id.
      geneIds <- sub("_PRE", "", mcols(rds@prePostCoords)$gene_id[preElems])
      if (!all(geneIds==make.unique(geneIds))) {
         stop("The prePostCoords given for this RoarDataset are wrong: gene_ids (prefixes of PRE-POST)
               are not unique.")
      }
      preCoords <- rowRanges(rds)
      rds@postCoords <- rds@prePostCoords[postElems,]
      se <- SummarizedExperiment(assays = rep(list(matrix(nrow=length(rds@prePostCoords)/2, ncol=4)),2),
                                 rowRanges=preCoords, 
                                 colData=DataFrame(row.names=c("treatment_pre","treatment_post","control_pre", "control_post"))
                                 )
      if (length(rds@treatmentBams) == 1 && length(rds@controlBams) == 1) {
         # We obtain counts for both conditions on PRE and POST coords.
         treatmentSE <- summOv(rds@treatmentBams[[1]])
         controlSE <- summOv(rds@controlBams[[1]])    
         # We obtain counts for both conditions only on POST coords (notwithstanding overlap with PRE) -
         # this is needed to implement the prefer-POST policy where a read overlapping PRE and POST
         # is counted on the POST portion (it has to be derived from a long isoform).
         # Note that in the first case we need to use the whole gtf with PRE and POST to avoid
         # assigning to PRE the overlapping reads, even if the counts on POST gotten afterwards
         # will be discarded.
         # This works but if a PRE portion of a gene overlaps with a POST portion of another one 
         # reads falling there are counted two times.
         treatmentSEpost <- summOvPost(rds@treatmentBams[[1]])
         controlSEpost <- summOvPost(rds@controlBams[[1]])
         assay(se,1,withDimnames=TRUE)[,"treatment_pre"] <- assays(treatmentSE,withDimnames=TRUE)$counts[preElems,]
         assay(se,1,withDimnames=TRUE)[,"treatment_post"] <- assays(treatmentSEpost,withDimnames=TRUE)$counts 
         assay(se,1,withDimnames=TRUE)[,"control_pre"] <- assays(controlSE,withDimnames=TRUE)$counts[preElems,]
         assay(se,1,withDimnames=TRUE)[,"control_post"] <- assays(controlSEpost,withDimnames=TRUE)$counts
         rowRanges(rds) <- rowRanges(se)
         colData(rds) <- colData(se)
         assays(rds,withDimnames=TRUE) <- assays(se,withDimnames=TRUE)
         names(assays(rds,withDimnames=TRUE)) <- "counts"
      } else {
         # As long as we need all the raw counts for the Fisher tests it is better to
         # keep them here as separate assays and compute (and keep) means in the computeRoar function.
         # The structure here is different from the single sample case for each condition, we will
         # see if it will be enough general to be used even there. Right now I ought to fallback
         # to that structure with the average counts.
         len <- length(rds@treatmentBams) + length(rds@controlBams)
         # Do we need preallocation for the list of matrixes?
         # x <- vector(mode = "list", length = 10)
         # ma <- matrix(nrow=5, ncol=2)
         # test <- list(ma, ma)
         # testse <- SummarizedExperiment(assays=test, rowRanges=gtfGRanges[c(1,2,3,4,5)], colData=DataFrame(row.names=c("a","b")))
         countsControl <- SummarizedExperiment(assays = rep(list(matrix(nrow=length(rds@prePostCoords)/2, ncol=2)), 
                                                            length(rds@controlBams)),
                                            rowRanges=preCoords, 
                                            colData=DataFrame(row.names=c("pre","post"))
                                            )
         countsTreatment <- SummarizedExperiment(assays = rep(list(matrix(nrow=length(rds@prePostCoords)/2, ncol=2)), 
                                                              length(rds@treatmentBams)),
                                             rowRanges=preCoords, 
                                             colData=DataFrame(row.names=c("pre","post"))
                                             )

         for (i in 1:length(rds@treatmentBams)) {
            treatmentSE <- summOv(rds@treatmentBams[[i]])
            treatmentSEpost <- summOvPost(rds@treatmentBams[[i]])
            #assay(countsTreatment,i) <- matrix(nrow=length(rds@prePostCoords)/2, ncol=2)
            #assay(countsTreatment,i) <- matrix(nrow=length(rds@prePostCoords)/2, ncol=2, 
            #                                   dimnames=list(rep(NULL,length(rds@prePostCoords)/2), c("pre","post")))
            assay(countsTreatment,i,withDimnames=TRUE)[,"pre"] <- assays(treatmentSE,withDimnames=TRUE)$counts[preElems,]
            assay(countsTreatment,i,withDimnames=TRUE)[,"post"] <- assays(treatmentSEpost,withDimnames=TRUE)$counts 
         }
         for (i in 1:length(rds@controlBams)) {
            controlSE <- summOv(rds@controlBams[[i]])
            controlSEpost <- summOvPost(rds@controlBams[[i]])
            #assay(countsControl,i) <- matrix(nrow=length(rds@prePostCoords)/2, ncol=2)
            assay(countsControl,i,withDimnames=TRUE)[,"pre"] <- assays(controlSE,withDimnames=TRUE)$counts[preElems,]
            assay(countsControl,i,withDimnames=TRUE)[,"post"] <- assays(controlSEpost,withDimnames=TRUE)$counts 
         }
         rds@countsTreatment <- countsTreatment
         rds@countsControl <- countsControl
      }
      rowRanges(rds) <- rowRanges(se)
      colData(rds) <- colData(se)
      assays(rds,withDimnames=TRUE) <- assays(se,withDimnames=TRUE)
      names(assays(rds,withDimnames=TRUE)) <- "counts"
      # We keep as "our" rds objects this SE, while those with counts are just slots.
      # In the case of multiple samples the primary object will be empty treatment now and will be filled
      # during the computeRoars step.
      rds@step <- 1
      return(rds)
   }
)

setMethod("computeRoars", signature(rds="RoarDataset"),
   function(rds, qwidthTreatment = NA, qwidthControl = NA) {
      goOn <- checkStep(rds, 1)
      if (!goOn[[1]]) {
         return(rds)
      }
      rds <- goOn[[2]]
      # roar is the m/M of treatment condition divided by the m/M of the control one.
      # m/M = ((Lpost*Cpre)/(Lpre*Cpost))-1
      # Negative m/M are discarded.
      #roar = (m/M_treatment)/(m/M_control)
      # We must obtain the list of lengths from rds@postCoords and rowRanges(rds) (which is pre).
      if ("length" %in% names(mcols(rds))) {
         preLen <- as.numeric(mcols(rds)$length)
         postLen <- as.numeric(mcols(rds@postCoords)$length)
      } else {
         preLen <- end(rowRanges(rds)) - start(rowRanges(rds)) + 1
         postLen <- end(rds@postCoords) - start(rds@postCoords) + 1
      }
      # I had to add "+1" as long as the end coords are not inclusive.
      # Then the bam lengths to correct our lengths, ie: postLen+ReadLength-1
      if (length(rds@treatmentBams) > 1 || length(rds@controlBams) > 1) {
         # Compute means and put them in place for roar calculations (in the rds-se object).
         # meanAcrossAssays given a list/SimpleList of assays performs the mean on a 
         # given col name (ie. "pre").
         # lapply(assays(testse), function(x) { x[,"a"]})
         # rowMeans(as.data.frame(a))
         assay(rds,1,withDimnames=TRUE)[,"treatment_pre"] <- meanAcrossAssays(assays(rds@countsTreatment,withDimnames=TRUE), "pre") # here peak of memory usage?
         assay(rds,1,withDimnames=TRUE)[,"treatment_post"] <- meanAcrossAssays(assays(rds@countsTreatment,withDimnames=TRUE),"post")
         assay(rds,1,withDimnames=TRUE)[,"control_pre"] <- meanAcrossAssays(assays(rds@countsControl,withDimnames=TRUE), "pre")
         assay(rds,1,withDimnames=TRUE)[,"control_post"] <- meanAcrossAssays(assays(rds@countsControl,withDimnames=TRUE), "post")
         # Also the length correction should consider all the samples!
         if (is.na(qwidthTreatment)) {
            lenTreatment <- unlist(lapply(rds@treatmentBams, qwidth))
            corrTreatment <- mean(lenTreatment)
         } else {
            corrTreatment <- qwidthTreatment
         }
            
         if (is.na(qwidthControl)) {
            lenControl <- unlist(lapply(rds@controlBams, qwidth))
            corrControl <- mean(lenControl)
         } else {
            corrControl <- qwidthControl
         }
      } else {
         # qwidth(x): Returns an integer vector of length length(x) containing the length 
         # of the query *after* hard clipping (i.e. the length of the query sequence 
         # that is stored in the corresponding SAM/BAM record).
         if (is.na(qwidthTreatment)) {
            corrTreatment <- mean(qwidth(rds@treatmentBams[[1]]))
         } else {
            corrTreatment <- qwidthTreatment
         }
         
         if (is.na(qwidthControl)) {
            corrControl <- mean(qwidth(rds@controlBams[[1]])) 
         } else {
            corrControl <- qwidthControl
         }
      }
      # Ok, now if we had a single sample for both conditions we had the data charged in
      # countPrePost, otherwise we have the means (in the same SE/RDS object).
      # If there is a single sample for one condition and more than one for the other there is
      # a little (I hope) unuseful overload to get the mean for the single sample. 
      # The countsTreatment/control slot are still kept as long as we will need them in computePvals.
      postLenTreatment <- postLen + corrTreatment - 1
      postLenControl <- postLen + corrControl - 1
      mMtreatment <- (assay(rds,1,withDimnames=TRUE)[,"treatment_pre"]*postLenTreatment)/(assay(rds,1,withDimnames=TRUE)[,"treatment_post"]*preLen)-1
      mMcontrol <- (assay(rds,1,withDimnames=TRUE)[,"control_pre"]*postLenControl)/(assay(rds,1,withDimnames=TRUE)[,"control_post"]*preLen)-1
      roar <- mMtreatment / mMcontrol
      pVal <- rep(NA, length(roar))
      assay(rds,2,withDimnames=TRUE) <- as.matrix(data.frame(treatment_pre=mMtreatment, treatment_post=mMcontrol, control_pre=roar, control_post=pVal, row.names=NULL))
      names(assays(rds,withDimnames=TRUE)) <- c("counts", "stats")
      rds@step <- 2
      return(rds)
   }
)

setMethod("computePvals", signature(rds="RoarDataset"),
   function(rds) {
      goOn <- checkStep(rds, 2)
      if (!goOn[[1]]) {
         return(rds)
      }
      rds <- goOn[[2]]
      if (length(rds@treatmentBams) == 1 && length(rds@controlBams) == 1) {
         if (cores(rds) == 1) {
            assay(rds,2,withDimnames=TRUE)[,"control_post"] <- apply(assay(rds,1,withDimnames=TRUE), 1, getFisher)
         } else {
            stop("TODO")
         }
      } else {      
         # We have raw counts in two SE slots (countsTreatment/Control) and need to
         # compute pvalues for every combination of treatment/control samples.
         # We need a function that given two assays returns the fisher pvalue
         # and we need to pass every combination there. The results will be but
         # in still another SE slot with a number of columns in the matrix equal to
         # the number of combinations. The product of all the pvalues will be put in the
         # rds/SE object in place of the pvalue for the single sample case.
         countsTreatmentAssays <- assays(rds@countsTreatment,withDimnames=TRUE)
         countsControlAssays <- assays(rds@countsControl,withDimnames=TRUE)
         nTreatment <- length(countsTreatmentAssays)
         nControl <- length(countsControlAssays)
         comparisons <- nTreatment*nControl
         rds@pVals <- SummarizedExperiment(assays = matrix(nrow=length(rds@prePostCoords)/2, ncol=comparisons),
                                           rowRanges=rowRanges(rds), 
                                           # To obtain all combination of two vectors (x,y) in the treatment order:
                                           # as.vector(t(outer(x,y,paste,sep=""))
                                           colData=DataFrame(row.names=paste("pvalue_", 
                                                                             as.vector(t(outer(seq(1,nTreatment), seq(1,nControl), paste, sep="_"))),
                                                                             sep=""))
                                          )
         # Ok, I know that we are in R, but these two for seems straightforward to me.
         for (i in 1:nTreatment) { # the y
            for (j in 1:nControl) { # the x
               mat <- cbind(countsTreatmentAssays[[i]], countsControlAssays[[j]])
               assay(rds@pVals,1,withDimnames=TRUE)[,nControl*(i-1)+j] <- apply(mat, 1, getFisher)
            }
         }
         assay(rds, 2,withDimnames=TRUE)[,"control_post"] <- apply(assay(rds@pVals,1,withDimnames=TRUE), 1, prod)
         # Here in theory we could remove countsTreatment/control slots, TODO check memory footprint and decide.
      }
      rds@step <- 3
      return(rds)
   }
)

setMethod("computePairedPvals", signature(rds="RoarDataset", treatmentSamples="numeric", controlSamples="numeric"),
   function(rds, treatmentSamples, controlSamples) {
      goOn <- checkStep(rds, 2)
      if (!goOn[[1]]) {
         return(rds)
      }
      rds <- goOn[[2]]
      if (length(rds@treatmentBams) == 1 || length(rds@controlBams) == 1) {
         stop("computePairedPvals can be used only with RoarDataset objects with multiple samples for both conditions")
      } else if (any(table(treatmentSamples)>1) || any(table(controlSamples)>1)) {
         stop("treatmentSamples and controlSamples should not contain repeated elements: each sample of a given group could be paired with only one of the other")
      } else if (length(treatmentSamples) != length(controlSamples)) {
         stop("treatmentSamples and controlSamples should have the same lengths")
      } else if (max(treatmentSamples) > length(rds@treatmentBams) || max(controlSamples) > length(rds@controlBams)) {
         stop("The given treatmentSamples or controlSamples numbers are wrong: the max is bigger than the given number of alignments")
      } else {     
         countsTreatmentAssays <- assays(rds@countsTreatment,withDimnames=TRUE)
         countsControlAssays <- assays(rds@countsControl,withDimnames=TRUE)
         comparisons <- length(treatmentSamples)
         rds@pVals <- SummarizedExperiment(assays = matrix(nrow=length(rds@prePostCoords)/2, ncol=comparisons),
                                           rowRanges=rowRanges(rds), 
                                           # To obtain all combination of two vectors (x,y) in the treatment order:
                                           # as.vector(t(outer(x,y,paste,sep=""))
                                           colData=DataFrame(row.names=paste("pvalue_", 
                                                                             mapply(FUN=function(x,y) {paste(x, y, sep="_")}, treatmentSamples, controlSamples),
                                                                             sep=""))
                                          )
         # Ok, I know that we are in R, but this seems straightforward to me.
         for (i in 1:length(treatmentSamples)) { 
            mat <- cbind(countsTreatmentAssays[[treatmentSamples[i]]], countsControlAssays[[controlSamples[i]]])
            assay(rds@pVals,1,withDimnames=TRUE)[,i] <- apply(mat, 1, getFisher)
         }
         assay(rds, 2,withDimnames=TRUE)[,"control_post"] <- apply(assay(rds@pVals,1,withDimnames=TRUE), 1, combineFisherMethod)
         # Here in theory we could remove countsTreatment/control slots, TODO check memory footprint and decide.
      }
      rds@paired <- TRUE
      rds@step <- 3
      return(rds)
   }
)

setMethod("totalResults", signature(rds="RoarDataset"),
   function(rds) {
      goOn <- checkStep(rds, 3)
      rds <- goOn[[2]]
      res <- data.frame(row.names=sub("^\\s+","", sub("_POST","", mcols(rds@postCoords)$gene_id)), 
                        mM_treatment=assay(rds,2,withDimnames=TRUE)[,"treatment_pre"], 
                        mM_control=assay(rds,2,withDimnames=TRUE)[,"treatment_post"],
                        roar=assay(rds,2,withDimnames=TRUE)[,"control_pre"],
                        pval=assay(rds,2,withDimnames=TRUE)[,"control_post"])
      if (length(rds@treatmentBams) != 1 || length(rds@controlBams) != 1) {
         pvals <- data.frame(assay(rds@pVals,1,withDimnames=TRUE))
         colnames(pvals) <- rownames(colData(rds@pVals))
         res <- cbind(res, pvals)
      }
      return(res)
   }
)


setMethod("sumRoarCounts", signature(rds="RoarDataset"),
          # Used by multipleAPA where we summarize gene info using counts/FPKM for different possible
          # PRE for genes. So we get the counts on the different PRE portions of roar objects.
          function(rds) {
             res <- data.frame(row.names=sub("^\\s+","",sub("_POST","", mcols(rds@postCoords)$gene_id)), 
                               counts_treatment=assay(rds,1,withDimnames=TRUE)[,1],
                               counts_control=assay(rds,1,withDimnames=TRUE)[,3])
             # In this case several rows of the same roar obj will have counts for different choices
             # and we choose to report them all (reporting the "whole" gene counts is not possible at
             # this stage cause if we sum up with colSums counts we will count some reads multiple times
             # and we do not have counts for fragments available anymore.)
             return(res)
          }
)

# This function will add to the totalResults dataframe RPKM gotten on the pre portions, then
# the user will be able to apply its preferred filtering/pvalue correction strategy.
# RPKM are gotten from counts over the PRE portions working on means across replicates.
# As total number of mapped reads we use the total number of reads mapped over all PRE portions.
setMethod("fpkmResults", signature(rds="RoarDataset"),
   function(rds) {
      dat <- totalResults(rds)
      if ("length" %in% names(mcols(rds))) {
         preLen <- as.numeric(mcols(rds)$length)
         } else {
         preLen <- end(rowRanges(rds)) - start(rowRanges(rds)) + 1
      }
      sumPreTreatment <- sum(assay(rds, 1, withDimnames=TRUE)[,"treatment_pre"])
      sumPreControl <- sum(assay(rds, 1, withDimnames=TRUE)[,"control_pre"])
      dat$treatmentValue <- (assay(rds, 1, withDimnames=TRUE)[,"treatment_pre"]*1000000000)/(preLen*sumPreTreatment)
      dat$controlValue <- (assay(rds, 1, withDimnames=TRUE)[,"control_pre"]*1000000000)/(preLen*sumPreControl)
      return(dat)
   }
)

setMethod("countResults", signature(rds="RoarDataset"),
   function(rds) {
      dat <- totalResults(rds)
      dat$treatmentValue <- assay(rds, 1, withDimnames=TRUE)[,"treatment_pre"]
      dat$controlValue <- assay(rds, 1, withDimnames=TRUE)[,"control_pre"]
      return(dat)
   }
)
   
.standardFilter <- function(rds, fpkmCutoff) {
   # Here we need to: remove all genes with a mean FPKM <= fpkmCutoff, 
   # a negative/NA m/M-roar.
   # P-value correction? In the single samples case it seems sensible to do that,
   # otherwise we will report all pvalues (and correct their product.)
   # Due to chr by chr scanning bonferroni correction has been removed.
   dat <- fpkmResults(rds)
   # mM_treatment, mM_control , roar columns filtering (< 0 / NA)
   # dat <- subset(dat, mM_treatment >= 0) # subset is ok for interactive use only
   dat <- dat[is.finite(dat$mM_treatment) & dat$mM_treatment >= 0,]
   #dat <- subset(dat, mM_control >= 0)
   dat <- dat[is.finite(dat$mM_control) & dat$mM_control >= 0,]
   #dat <- subset(dat, !is.na(roar))
   # Changed is.na to is.finite to avoid Inf/-Inf, did not add a unitTest as long as it's trivial.
   dat <- dat[is.finite(dat$roar),]
   # treatmentValue/controlValue filtering (<= fpkmCutoff)
   #dat <- subset(dat, treatmentValue > fpkmCutoff)
   #dat <- subset(dat, controlValue > fpkmCutoff)
   dat <- dat[dat$treatmentValue > fpkmCutoff,]
   dat <- dat[dat$controlValue > fpkmCutoff,]
   return(dat)
}                  
setMethod("standardFilter", signature(rds="RoarDataset", fpkmCutoff="numeric"),
          .standardFilter)
setMethod("standardFilter", signature(rds="RoarDatasetMultipleAPA", fpkmCutoff="numeric"),
          .standardFilter)

setMethod("pvalueFilter", signature(rds="RoarDataset", fpkmCutoff="numeric", pvalCutoff="numeric"),
   function(rds, fpkmCutoff, pvalCutoff) {
      dat <- standardFilter(rds, fpkmCutoff)
      if ((length(rds@treatmentBams) != 1 || length(rds@controlBams) != 1) && !rds@paired) {
         # In this case we add to dat a col that says how many comparisons yielded
         # a pvalue < pvalCutoff.
         # esany <- apply(data, 1, function(x) {any(x[seq(1,12)] < 0.05)})
         if(nrow(dat) != 0) {
            cols <- grep("^pvalue_", colnames(dat))
            sel <- apply(dat, 1, function(x) {x[cols] < pvalCutoff})
            # This yields a transposed dat with cols rows and TRUE/FALSE. ncol = nrows of dat
            dat$nUnderCutoff <- apply(sel, 2, function(x){length(x[x==TRUE])})
         }
      } else {
         dat <- dat[dat$pval < pvalCutoff,]
      }   
      return(dat)
   }                  
)

setMethod("pvalueCorrectFilter", signature(rds="RoarDataset", fpkmCutoff="numeric", pvalCutoff="numeric", method="character"),
   function(rds, fpkmCutoff, pvalCutoff, method) {
      dat <- standardFilter(rds, fpkmCutoff)
      if ((length(rds@treatmentBams) != 1 || length(rds@controlBams) != 1) && !rds@paired) {
         # In this case we add to dat a col that says how many comparisons yielded
         # a pvalue < pvalCutoff.
         # esany <- apply(data, 1, function(x) {any(x[seq(1,12)] < 0.05)})
         if(nrow(dat) != 0) {
            cols <- grep("^pvalue_", colnames(dat))
            sel <- apply(dat, 1, function(x) {x[cols] < pvalCutoff})
            # This yields a transposed dat with cols rows and TRUE/FALSE. ncol = nrows of dat
            dat$nUnderCutoff <- apply(sel, 2, function(x){length(x[x==TRUE])})
         }
      } else {
         dat$pval <- p.adjust(dat$pval, method=method)
         dat <- dat[dat$pval < pvalCutoff,]
      }   
      return(dat)
   }                 
)

# Simple getters and setters.
setMethod("cores",  signature(rds="RoarDataset"),
 function(rds) {
    return(rds@cores)
 }
)
 
# setReplaceMethod("cores",  signature(rds="RoarDataset", value="numeric"),
#  function(rds, value) {
#     if (value < 1) {
#        stop("You can specify only a positive number of cores")
#     }
#     rds@cores <- value
#     return(rds)
#  }
# )

# Here signature does not work. We are overriding show, that's why?
setMethod("show", "RoarDataset",
   function(object) {
      cat("RoarDataset object\n")
      cat("N. of treatment alignments:", length(object@treatmentBams), "\n")
      cat("N. of control alignments:", length(object@controlBams), "\n")
      cat("N. of genes in study:", length(object@prePostCoords)/2 , "\n")
      cat("N. of cores:", object@cores, "\n")
      cat("Analysis step reached [0-3]:", object@step, "\n")
      cat("Paired?", object@paired, "\n")
      cat("\n")     
   }
)

# Bad! What should I do? Export this function is not an option.
getPreCoordsSE <- function(gtfGRanges) {
   # Now we need to keep means and totals of counts over PRE/POST for the two lists.
   # In the simpler case with a single alignment for both conditions we just keep the counts.
   preElems <- grep("_PRE$", mcols(gtfGRanges)$gene_id)
   preCoords <- gtfGRanges[preElems,]
   se <- SummarizedExperiment(rowRanges=preCoords, colData=DataFrame(row.names=c("treatment_pre","treatment_post","control_pre", "control_post")))
   return(se)
}

test_that("test_computePairedPvals", {
    gene_id <- c("A_PRE", "A_POST")
    features <- GRanges(seqnames = Rle(c("chr1", "chr1")), strand = strand(rep("+", 
        length(gene_id))), ranges = IRanges(start = c(1000, 2000), 
        width = c(1000, 900)), DataFrame(gene_id))
    se <- getPreCoordsSE(features)
    rds <- new("RoarDataset", se, treatmentBams = list(), controlBams = list(), 
        prePostCoords = features, step = 2, cores = 1)
    preElems <- grep("_PRE$", mcols(rds@prePostCoords)$gene_id)
    postElems <- grep("_POST$", mcols(rds@prePostCoords)$gene_id)
    preCoords <- rds@prePostCoords[preElems, ]
    se <- SummarizedExperiment(assays = rep(list(matrix(nrow = 1, 
        ncol = 4)), 2), rowRanges = preCoords, colData = DataFrame(row.names = c("treatment_pre", 
        "treatment_post", "control_pre", "control_post")))
    rowRanges(rds) <- rowRanges(se)
    colData(rds) <- colData(se)
    assays(rds) <- assays(se)
    names(assays(rds)) <- "counts"
    length(rds@treatmentBams) <- 3
    length(rds@controlBams) <- 2
    rds@countsTreatment <- SummarizedExperiment(assays = rep(list(matrix(nrow = 1, 
        ncol = 2)), 3), rowRanges = preCoords, colData = DataFrame(row.names = c("pre", 
        "post")))
    rds@countsControl <- SummarizedExperiment(assays = rep(list(matrix(nrow = 1, 
        ncol = 2)), 2), rowRanges = preCoords, colData = DataFrame(row.names = c("pre", 
        "post")))
    assay(rds@countsTreatment, 1)[1, ] <- c(10, 10)
    assay(rds@countsTreatment, 2)[1, ] <- c(1, 10)
    assay(rds@countsTreatment, 3)[1, ] <- c(20, 10)
    assay(rds@countsControl, 1)[1, ] <- c(10, 5)
    assay(rds@countsControl, 2)[1, ] <- c(10, 20)
    rds <- computePairedPvals(rds, c(2, 3), c(1, 2))
    expect_equal(0.00522109, assay(rds@pVals, 1)[1, 1], tolerance=1e-6, check.attributes=F)
    expect_equal(0.01938319, assay(rds@pVals, 1)[1, 2], tolerance=1e-6, check.attributes=F)
    expect_equal(0.001032092, assay(rds, 2)[1, 4], tolerance=1e-6, check.attributes=F)
})

test_that("test_computePvals_multipleSamples", {
    gene_id <- c("A_PRE", "A_POST")
    features <- GRanges(seqnames = Rle(c("chr1", "chr1")), strand = strand(rep("+", 
        length(gene_id))), ranges = IRanges(start = c(1000, 2000), 
        width = c(1000, 900)), DataFrame(gene_id))
    se <- getPreCoordsSE(features)
    rds <- new("RoarDataset", se, treatmentBams = list(), controlBams = list(), 
        prePostCoords = features, step = 2, cores = 1)
    preElems <- grep("_PRE$", mcols(rds@prePostCoords)$gene_id)
    postElems <- grep("_POST$", mcols(rds@prePostCoords)$gene_id)
    preCoords <- rds@prePostCoords[preElems, ]
    se <- SummarizedExperiment(assays = rep(list(matrix(nrow = 1, 
        ncol = 4)), 2), rowRanges = preCoords, colData = DataFrame(row.names = c("treatment_pre", 
        "treatment_post", "control_pre", "control_post")))
    rowRanges(rds) <- rowRanges(se)
    colData(rds) <- colData(se)
    assays(rds) <- assays(se)
    names(assays(rds)) <- "counts"
    length(rds@treatmentBams) <- 3
    length(rds@controlBams) <- 2
    rds@countsTreatment <- SummarizedExperiment(assays = rep(list(matrix(nrow = 1, 
        ncol = 2)), 3), rowRanges = preCoords, colData = DataFrame(row.names = c("pre", 
        "post")))
    rds@countsControl <- SummarizedExperiment(assays = rep(list(matrix(nrow = 1, 
        ncol = 2)), 2), rowRanges = preCoords, colData = DataFrame(row.names = c("pre", 
        "post")))
    assay(rds@countsTreatment, 1)[1, ] <- c(10, 10)
    assay(rds@countsTreatment, 2)[1, ] <- c(1, 10)
    assay(rds@countsTreatment, 3)[1, ] <- c(20, 10)
    assay(rds@countsControl, 1)[1, ] <- c(10, 5)
    assay(rds@countsControl, 2)[1, ] <- c(10, 20)
    rds <- computePvals(rds)
    expect_equal(0.491599, assay(rds@pVals, 1)[1, 1], tolerance=1e-6, check.attributes=F)
    expect_equal(0.257549, assay(rds@pVals, 1)[1, 2], tolerance=1e-6, check.attributes=F)
    expect_equal(0.00522109, assay(rds@pVals, 1)[1, 3], tolerance=1e-6, check.attributes=F)
    expect_equal(0.23264154, assay(rds@pVals, 1)[1, 4], tolerance=1e-6, check.attributes=F)
    expect_equal(1, assay(rds@pVals, 1)[1, 5], tolerance=1e-6, check.attributes=F)
    expect_equal(0.01938319, assay(rds@pVals, 1)[1, 6], tolerance=1e-6, check.attributes=F)
})

test_that("test_computePvals_single", {
    gene_id <- c("A_PRE", "A_POST", "B_PRE", "B_POST")
    features <- GRanges(seqnames = Rle(c("chr1", "chr1", "chr2", 
        "chr2")), strand = strand(rep("+", length(gene_id))), 
        ranges = IRanges(start = c(1000, 2000, 3000, 3600), width = c(1000, 
            900, 600, 300)), DataFrame(gene_id))
    se <- getPreCoordsSE(features)
    rds <- new("RoarDataset", se, treatmentBams = list(), controlBams = list(), 
        prePostCoords = features, step = 2, cores = 1)
    preElems <- grep("_PRE$", mcols(rds@prePostCoords)$gene_id)
    postElems <- grep("_POST$", mcols(rds@prePostCoords)$gene_id)
    preCoords <- rds@prePostCoords[preElems, ]
    se <- SummarizedExperiment(assays = rep(list(matrix(nrow = 2, 
        ncol = 4)), 2), rowRanges = preCoords, colData = DataFrame(row.names = c("treatment_pre", 
        "treatment_post", "control_pre", "control_post")))
    rowRanges(rds) <- rowRanges(se)
    colData(rds) <- colData(se)
    assays(rds) <- assays(se)
    names(assays(rds)) <- "counts"
    length(rds@treatmentBams) <- 1
    length(rds@controlBams) <- 1
    assay(rds, 1)[1, 1] <- 10
    assay(rds, 1)[1, 2] <- 100
    assay(rds, 1)[1, 3] <- 20
    assay(rds, 1)[1, 4] <- 20
    assay(rds, 1)[2, 1] <- 10
    assay(rds, 1)[2, 2] <- 20
    assay(rds, 1)[2, 3] <- 20
    assay(rds, 1)[2, 4] <- 20
    rds <- computePvals(rds)
    expect_equal(2.212406e-07, assay(rds, 2)[1, 4], tolerance=1e-6, check.attributes=F)
    expect_equal(0.22337243, assay(rds, 2)[2, 4], tolerance=1e-6, check.attributes=F)
})

test_that("test_computePvals_singlevsMul", {
    gene_id <- c("A_PRE", "A_POST")
    features <- GRanges(seqnames = Rle(c("chr1", "chr1")), strand = strand(rep("+", 
        length(gene_id))), ranges = IRanges(start = c(1000, 2000), 
        width = c(1000, 900)), DataFrame(gene_id))
    se <- getPreCoordsSE(features)
    rds <- new("RoarDataset", se, treatmentBams = list(), controlBams = list(), 
        prePostCoords = features, step = 2, cores = 1)
    preElems <- grep("_PRE$", mcols(rds@prePostCoords)$gene_id)
    postElems <- grep("_POST$", mcols(rds@prePostCoords)$gene_id)
    preCoords <- rds@prePostCoords[preElems, ]
    se <- SummarizedExperiment(assays = rep(list(matrix(nrow = 1, 
        ncol = 4)), 2), rowRanges = preCoords, colData = DataFrame(row.names = c("treatment_pre", 
        "treatment_post", "control_pre", "control_post")))
    rowRanges(rds) <- rowRanges(se)
    colData(rds) <- colData(se)
    assays(rds) <- assays(se)
    names(assays(rds)) <- "counts"
    length(rds@treatmentBams) <- 1
    length(rds@controlBams) <- 3
    rds@countsTreatment <- SummarizedExperiment(assays = matrix(nrow = 1, 
        ncol = 2), rowRanges = preCoords, colData = DataFrame(row.names = c("pre", 
        "post")))
    rds@countsControl <- SummarizedExperiment(assays = rep(list(matrix(nrow = 1, 
        ncol = 2)), 3), rowRanges = preCoords, colData = DataFrame(row.names = c("pre", 
        "post")))
    assay(rds@countsTreatment, 1)[1, ] <- c(10, 10)
    assay(rds@countsControl, 1)[1, ] <- c(10, 10)
    assay(rds@countsControl, 2)[1, ] <- c(10, 5)
    assay(rds@countsControl, 3)[1, ] <- c(10, 20)
    rds <- computePvals(rds)
    expect_equal(1, assay(rds@pVals, 1)[1, 1], tolerance=1e-6, check.attributes=F)
    expect_equal(0.491599, assay(rds@pVals, 1)[1, 2], tolerance=1e-6, check.attributes=F)
    expect_equal(0.257549, assay(rds@pVals, 1)[1, 3], tolerance=1e-6, check.attributes=F)
})

test_that("test_computeRoars_multipleSamples", {
    gene_id <- c("A_PRE", "A_POST")
    features <- GRanges(seqnames = Rle(c("chr1", "chr1")), strand = strand(c("+", 
        "+")), ranges = IRanges(start = c(1, 10), width = c(10, 
        5)), DataFrame(gene_id))
    a_pre <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(2), 
        cigar = "5M", strand = strand("+"))
    a_post <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(11), 
        cigar = "3M", strand = strand("+"))
    treatmentAlign <- list(rep(a_pre, 2), rep(a_post, 3))
    controlAlign <- list(a_post, rep(a_pre, 4), a_post)
    rds <- RoarDataset(treatmentAlign, controlAlign, features)
    rds <- countPrePost(rds, FALSE)
    rds <- computeRoars(rds)
    expect_equal(-0.48, assay(rds, 2)[1, 1], tolerance=1e-6, check.attributes=F)
    expect_equal(0.666666666, assay(rds, 2)[1, 2], tolerance=1e-6, check.attributes=F)
    expect_equal(-0.72, assay(rds, 2)[1, 3], tolerance=1e-6, check.attributes=F)
    expect_true(is.na(assay(rds, 2)[1, 4]))
})

test_that("test_computeRoars_singleSamples", {
    gene_id <- c("A_PRE", "A_POST", "B_POST", "B_PRE", "C_PRE", 
        "C_POST")
    features <- GRanges(seqnames = Rle(rep("chr1", length(gene_id))), 
        strand = strand(c("+", "+", "-", "-", "+", "+")), ranges = IRanges(start = c(1, 
            10, 20, 40, 42, 52), width = c(10, 5, 20, 5, 10, 
            10)), DataFrame(gene_id))
    a_pre <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(2), 
        cigar = "5M", strand = strand("+"))
    a_post <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(11), 
        cigar = "3M", strand = strand("+"))
    a_pre_post <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(9), 
        cigar = "3M", strand = strand("+"))
    b_pre <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(40), 
        cigar = "1M", strand = strand("-"))
    b_post <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(21), 
        cigar = "3M", strand = strand("-"))
    overlapbc <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(40), 
        cigar = "5M", strand = strand("+"))
    c_post <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(53), 
        cigar = "50M", strand = strand("+"))
    treatmentAlign <- list(c(rep(a_pre, 2), rep(a_post, 3), a_pre_post, 
        rep(b_pre, 5), b_post, overlapbc))
    controlAlign <- list(c(a_post, rep(a_pre, 4), a_pre_post, 
        rep(b_post, 5), b_pre, c_post))
    rds <- RoarDataset(treatmentAlign, controlAlign, features)
    rds <- countPrePost(rds, FALSE)
    rds <- computeRoars(rds)
    expect_equal(-0.66538461538462, assay(rds, 2)[1, 1], tolerance=1e-6, check.attributes=F)
    expect_equal(1.21538461538462, assay(rds, 2)[1, 2], tolerance=1e-6, check.attributes=F)
    expect_equal(-0.54746835443038, assay(rds, 2)[1, 3], tolerance=1e-6, check.attributes=F)
    expect_true(is.na(assay(rds, 2)[1, 4]))
    expect_equal(20.6923076923077, assay(rds, 2)[2, 1], tolerance=1e-6, check.attributes=F)
    expect_equal(0.04307692307692, assay(rds, 2)[2, 2], tolerance=1e-6, check.attributes=F)
    expect_equal(480.357142857144, assay(rds, 2)[2, 3], tolerance=1e-6, check.attributes=F)
    expect_true(is.na(assay(rds, 2)[3, 1]))
    expect_equal(-1, assay(rds, 2)[3, 2], tolerance=1e-6, check.attributes=F)
    expect_true(is.na(assay(rds, 2)[3, 3]))
    rds2 <- RoarDataset(treatmentAlign, controlAlign, features)
    rds2 <- countPrePost(rds2, TRUE)
    expect_equal(2, assay(rds2, 1)[1, 1], tolerance=1e-6, check.attributes=F)
    expect_equal(4, assay(rds2, 1)[1, 2], tolerance=1e-6, check.attributes=F)
    expect_equal(4, assay(rds2, 1)[1, 3], tolerance=1e-6, check.attributes=F)
    expect_equal(2, assay(rds2, 1)[1, 4], tolerance=1e-6, check.attributes=F)
    expect_equal(5, assay(rds2, 1)[2, 1], tolerance=1e-6, check.attributes=F)
    expect_equal(1, assay(rds2, 1)[2, 2], tolerance=1e-6, check.attributes=F)
    expect_equal(1, assay(rds2, 1)[2, 3], tolerance=1e-6, check.attributes=F)
    expect_equal(5, assay(rds2, 1)[2, 4], tolerance=1e-6, check.attributes=F)
    expect_equal(1, assay(rds2, 1)[3, 1], tolerance=1e-6, check.attributes=F)
    expect_equal(0, assay(rds2, 1)[3, 2], tolerance=1e-6, check.attributes=F)
    expect_equal(0, assay(rds2, 1)[3, 3], tolerance=1e-6, check.attributes=F)
    expect_equal(1, assay(rds2, 1)[3, 4], tolerance=1e-6, check.attributes=F)
})

test_that("test_computeRoars_singleSamples_GRanges_order", {
    gene_id <- c("B_POST", "A_PRE", "C_POST", "B_PRE", "A_POST", 
        "C_PRE")
    features <- GRanges(seqnames = Rle(rep("chr1", length(gene_id))), 
        strand = strand(c("-", "+", "+", "-", "+", "+")), ranges = IRanges(start = c(20, 
            1, 52, 40, 10, 42), width = c(20, 10, 10, 5, 5, 10)), 
        DataFrame(gene_id))
    a_pre <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(2), 
        cigar = "5M", strand = strand("+"))
    a_post <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(11), 
        cigar = "3M", strand = strand("+"))
    a_pre_post <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(9), 
        cigar = "3M", strand = strand("+"))
    b_pre <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(40), 
        cigar = "1M", strand = strand("-"))
    b_post <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(21), 
        cigar = "3M", strand = strand("-"))
    overlapbc <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(40), 
        cigar = "5M", strand = strand("+"))
    c_post <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(53), 
        cigar = "50M", strand = strand("+"))
    treatmentAlign <- list(c(rep(a_pre, 2), rep(a_post, 3), a_pre_post, 
        rep(b_pre, 5), b_post, overlapbc))
    controlAlign <- list(c(a_post, rep(a_pre, 4), a_pre_post, 
        rep(b_post, 5), b_pre, c_post))
    rds <- RoarDataset(treatmentAlign, controlAlign, features)
    rds <- countPrePost(rds)
    rds <- computeRoars(rds)
    expect_equal(-0.66538461538462, assay(rds, 2)[1, 1], tolerance=1e-6, check.attributes=F)
    expect_equal(1.21538461538462, assay(rds, 2)[1, 2], tolerance=1e-6, check.attributes=F)
    expect_equal(-0.54746835443038, assay(rds, 2)[1, 3], tolerance=1e-6, check.attributes=F)
    expect_true(is.na(assay(rds, 2)[1, 4]))
    expect_equal(20.6923076923077, assay(rds, 2)[2, 1], tolerance=1e-6, check.attributes=F)
    expect_equal(0.04307692307692, assay(rds, 2)[2, 2], tolerance=1e-6, check.attributes=F)
    expect_equal(480.357142857144, assay(rds, 2)[2, 3], tolerance=1e-6, check.attributes=F)
    expect_true(is.na(assay(rds, 2)[3, 1]))
    expect_equal(-1, assay(rds, 2)[3, 2], tolerance=1e-6, check.attributes=F)
    expect_true(is.na(assay(rds, 2)[3, 3]))
})

test_that("test_computeRoars_singleSamples_length", {
    gene_id <- c("A_PRE", "A_POST", "B_POST", "B_PRE", "C_PRE", 
        "C_POST")
    length <- c(1, 15, 20, 5, 30, 40)
    features <- GRanges(seqnames = Rle(rep("chr1", length(gene_id))), 
        strand = strand(c("+", "+", "-", "-", "+", "+")), ranges = IRanges(start = c(1, 
            10, 20, 40, 42, 52), width = c(10, 5, 20, 5, 10, 
            10)), DataFrame(gene_id, length))
    a_pre <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(2), 
        cigar = "5M", strand = strand("+"))
    a_post <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(11), 
        cigar = "3M", strand = strand("+"))
    a_pre_post <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(9), 
        cigar = "3M", strand = strand("+"))
    b_pre <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(40), 
        cigar = "1M", strand = strand("-"))
    b_post <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(21), 
        cigar = "3M", strand = strand("-"))
    overlapbc <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(40), 
        cigar = "5M", strand = strand("+"))
    c_post <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(53), 
        cigar = "50M", strand = strand("+"))
    treatmentAlign <- list(c(rep(a_pre, 2), rep(a_post, 3), a_pre_post, 
        rep(b_pre, 5), b_post, overlapbc))
    controlAlign <- list(c(a_post, rep(a_pre, 4), a_pre_post, 
        rep(b_post, 5), b_pre, c_post))
    rds <- RoarDataset(treatmentAlign, controlAlign, features)
    rds <- countPrePost(rds, FALSE)
    rds <- computeRoars(rds)
    expect_equal(7.34615384615385, assay(rds, 2)[1, 1], tolerance=1e-6, check.attributes=F)
    expect_equal(41.1538461538462, assay(rds, 2)[1, 2], tolerance=1e-6, check.attributes=F)
    expect_equal(0.178504672897196, assay(rds, 2)[1, 3], tolerance=1e-6, check.attributes=F)
    expect_true(is.na(assay(rds, 2)[1, 4]))
    expect_equal(20.6923076923077, assay(rds, 2)[2, 1], tolerance=1e-6, check.attributes=F)
    expect_equal(0.04307692307692, assay(rds, 2)[2, 2], tolerance=1e-6, check.attributes=F)
    expect_equal(480.357142857144, assay(rds, 2)[2, 3], tolerance=1e-6, check.attributes=F)
    expect_true(is.na(assay(rds, 2)[3, 1]))
    expect_equal(-1, assay(rds, 2)[3, 2], tolerance=1e-6, check.attributes=F)
    expect_true(is.na(assay(rds, 2)[3, 3]))
})

test_that("test_computeRoars_singlevsMulSamples", {
    gene_id <- c("A_PRE", "A_POST", "B_POST", "B_PRE", "C_PRE", 
        "C_POST")
    features <- GRanges(seqnames = Rle(rep("chr1", length(gene_id))), 
        strand = strand(c("+", "+", "-", "-", "+", "+")), ranges = IRanges(start = c(1, 
            10, 20, 40, 42, 52), width = c(10, 5, 20, 5, 10, 
            10)), DataFrame(gene_id))
    a_pre <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(2), 
        cigar = "5M", strand = strand("+"))
    a_post <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(11), 
        cigar = "3M", strand = strand("+"))
    a_pre_post <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(9), 
        cigar = "3M", strand = strand("+"))
    b_pre <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(40), 
        cigar = "1M", strand = strand("-"))
    b_post <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(21), 
        cigar = "3M", strand = strand("-"))
    overlapbc <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(40), 
        cigar = "5M", strand = strand("+"))
    c_post <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(53), 
        cigar = "50M", strand = strand("+"))
    treatmentAlign <- list(c(rep(a_pre, 2), rep(a_post, 3), a_pre_post, 
        rep(b_pre, 5), b_post, overlapbc))
    controlAlign <- list(a_post, rep(a_pre, 4), a_pre_post, rep(b_post, 
        5), b_pre, c_post)
    rds <- RoarDataset(treatmentAlign, controlAlign, features)
    rds <- countPrePost(rds, FALSE)
    rds <- computeRoars(rds)
    expect_equal(-0.66538461538462, assay(rds, 2)[1, 1], tolerance=1e-6, check.attributes=F)
    expect_equal(1.21538461538462, assay(rds, 2)[1, 2], tolerance=1e-6, check.attributes=F)
    expect_equal(-0.54746835443038, assay(rds, 2)[1, 3], tolerance=1e-6, check.attributes=F)
    expect_true(is.na(assay(rds, 2)[1, 4]))
    expect_equal(20.6923076923077, assay(rds, 2)[2, 1], tolerance=1e-6, check.attributes=F)
    expect_equal(0.04307692307692, assay(rds, 2)[2, 2], tolerance=1e-6, check.attributes=F)
    expect_equal(480.357142857144, assay(rds, 2)[2, 3], tolerance=1e-6, check.attributes=F)
    expect_true(is.na(assay(rds, 2)[3, 1]))
    expect_equal(-1, assay(rds, 2)[3, 2], tolerance=1e-6, check.attributes=F)
    expect_true(is.na(assay(rds, 2)[3, 3]))
})

test_that("test_countPrePost_mulSamples", {
    gene_id <- c("A_PRE", "A_POST", "B_PRE", "B_POST", "D_PRE", 
        "D_POST", "E_PRE", "E_POST")
    features <- GRanges(seqnames = Rle(c("chr1", "chr1", "chr2", 
        "chr2", "chr2", "chr2", "chr1", "chr1")), strand = strand(rep("+", 
        length(gene_id))), ranges = IRanges(start = c(1000, 2000, 
        3000, 3600, 7000, 7500, 4000, 4000), width = c(500, 900, 
        500, 300, 600, 300, 500, 900)), DataFrame(gene_id))
    rd1 <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(1000), 
        cigar = "300M", strand = strand("+"))
    rd2 <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(2000), 
        cigar = "300M", strand = strand("+"))
    rd3 <- GAlignments("a", seqnames = Rle("chr2"), pos = as.integer(3000), 
        cigar = "300M", strand = strand("+"))
    rd4 <- GAlignments("a", seqnames = Rle("chr2"), pos = as.integer(3200), 
        cigar = "300M", strand = strand("+"))
    rds <- RoarDataset(list(rd1, rd2), list(rd3, rd4), features)
    rds <- countPrePost(rds, FALSE)
    expect_equal(1, assay(rds@countsTreatment, 1)[1, 1], tolerance=1e-6, check.attributes=F)
    expect_equal(0, assay(rds@countsTreatment, 1)[1, 2], tolerance=1e-6, check.attributes=F)
    expect_equal(1, assay(rds@countsControl, 2)[2, 1], tolerance=1e-6, check.attributes=F)
})

test_that("test_countPrePost_preferPOST", {
    gene_id <- c("A_PRE", "A_POST", "B_PRE", "B_POST")
    features <- GRanges(seqnames = Rle(c("chr1", "chr1", "chr2", 
        "chr2")), strand = strand(rep("+", length(gene_id))), 
        ranges = IRanges(start = c(1000, 2000, 3000, 3600), width = c(1000, 
            900, 600, 300)), DataFrame(gene_id))
    rd1 <- GAlignments("a", seqnames = factor("chr1", levels = c("chr1", 
        "chr2")), pos = as.integer(1000), cigar = "3000M", strand = strand("+"))
    rd2 <- GAlignments("a", seqnames = factor("chr1", levels = c("chr1", 
        "chr2")), pos = as.integer(1000), cigar = "3000M", strand = strand("+"))
    rd3 <- GAlignments("a", seqnames = factor("chr2", levels = c("chr1", 
        "chr2")), pos = as.integer(2800), cigar = "300M", strand = strand("+"))
    rd4 <- GAlignments("a", seqnames = factor("chr2", levels = c("chr1", 
        "chr2")), pos = as.integer(3500), cigar = "300M", strand = strand("+"))
    rd5 <- GAlignments("a", seqnames = factor("chr1", levels = c("chr1", 
        "chr2")), pos = as.integer(1000), cigar = "300M", strand = strand("+"))
    rds <- RoarDataset(list(c(rd1, rd2)), list(c(rd3, rd4, rd5)), 
        features)
    rds <- countPrePost(rds, FALSE)
    expect_equal(0, assay(rds, 1)[1, 1], tolerance=1e-6, check.attributes=F)
    expect_equal(2, assay(rds, 1)[1, 2], tolerance=1e-6, check.attributes=F)
    expect_equal(1, assay(rds, 1)[1, 3], tolerance=1e-6, check.attributes=F)
    expect_equal(0, assay(rds, 1)[1, 4], tolerance=1e-6, check.attributes=F)
    expect_equal(1, assay(rds, 1)[2, 3], tolerance=1e-6, check.attributes=F)
    expect_equal(1, assay(rds, 1)[2, 4], tolerance=1e-6, check.attributes=F)
    expect_equal(0, assay(rds, 1)[2, 1], tolerance=1e-6, check.attributes=F)
    expect_equal(0, assay(rds, 1)[2, 2], tolerance=1e-6, check.attributes=F)
})

test_that("test_countPrePost_singleSamples", {
    gene_id <- c("A_PRE", "A_POST", "B_PRE", "B_POST", "D_PRE", 
        "D_POST", "E_PRE", "E_POST")
    features <- GRanges(seqnames = Rle(c("chr1", "chr1", "chr2", 
        "chr2", "chr2", "chr2", "chr1", "chr1")), strand = strand(rep("+", 
        length(gene_id))), ranges = IRanges(start = c(1000, 2000, 
        3000, 3600, 7000, 7500, 4000, 4000), width = c(500, 900, 
        500, 300, 600, 300, 500, 900)), DataFrame(gene_id))
    rd1 <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(1000), 
        cigar = "300M", strand = strand("+"))
    rd2 <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(2000), 
        cigar = "300M", strand = strand("+"))
    rd3 <- GAlignments("a", seqnames = Rle("chr2"), pos = as.integer(3000), 
        cigar = "300M", strand = strand("+"))
    rd4 <- GAlignments("a", seqnames = Rle("chr2"), pos = as.integer(3200), 
        cigar = "300M", strand = strand("+"))
    rds <- RoarDataset(list(c(rd1, rd2)), list(c(rd3, rd4)), 
        features)
    rds <- countPrePost(rds, FALSE)
    expect_equal(1, assay(rds, 1)[1, 1], tolerance=1e-6, check.attributes=F)
    expect_equal(1, assay(rds, 1)[1, 2], tolerance=1e-6, check.attributes=F)
    expect_equal(2, assay(rds, 1)[2, 3], tolerance=1e-6, check.attributes=F)
})

test_that("test_countPrePost_stranded", {
    gene_id <- c("A_PRE", "A_POST", "B_PRE", "B_POST")
    features <- GRanges(seqnames = Rle(c("chr1", "chr1", "chr2", 
        "chr2")), strand = strand(rep("+", length(gene_id))), 
        ranges = IRanges(start = c(1000, 2000, 3000, 3600), width = c(1000, 
            900, 600, 300)), DataFrame(gene_id))
    rd1 <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(1000), 
        cigar = "300M", strand = strand("+"))
    rd2 <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(1000), 
        cigar = "300M", strand = strand("-"))
    rd3 <- GAlignments("a", seqnames = Rle("chr2"), pos = as.integer(3000), 
        cigar = "300M", strand = strand("+"))
    rd4 <- GAlignments("a", seqnames = Rle("chr2"), pos = as.integer(3200), 
        cigar = "300M", strand = strand("-"))
    rd5 <- GAlignments("a", seqnames = Rle("chr2"), pos = as.integer(3200), 
        cigar = "300M", strand = strand("+"))
    rd6 <- GAlignments("a", seqnames = Rle("chr2"), pos = as.integer(3200), 
        cigar = "300M", strand = strand("+"))
    rds <- RoarDataset(list(rd1, rd2), list(rd3, rd4, c(rd5, 
        rd6)), features)
    rds <- countPrePost(rds, TRUE)
    expect_equal(1, assay(rds@countsTreatment, 1)[1, 1], tolerance=1e-6, check.attributes=F)
    expect_equal(0, assay(rds@countsTreatment, 1)[1, 2], tolerance=1e-6, check.attributes=F)
    expect_equal(0, assay(rds@countsTreatment, 2)[1, 1], tolerance=1e-6, check.attributes=F)
    expect_equal(0, assay(rds@countsTreatment, 2)[1, 2], tolerance=1e-6, check.attributes=F)
    expect_equal(1, assay(rds@countsControl, 1)[2, 1], tolerance=1e-6, check.attributes=F)
    expect_equal(0, assay(rds@countsControl, 2)[2, 1], tolerance=1e-6, check.attributes=F)
    expect_equal(2, assay(rds@countsControl, 3)[2, 1], tolerance=1e-6, check.attributes=F)
})

test_that("test_fpkmResults_singleSamples", {
    gene_id <- c("A_PRE", "A_POST", "B_PRE", "B_POST")
    features <- GRanges(seqnames = Rle(rep("chr1", length(gene_id))), 
        strand = strand(rep("+", length(gene_id))), ranges = IRanges(start = c(1000, 
            2000, 1, 2), width = c(1000, 900, 100, 2)), DataFrame(gene_id))
    se <- getPreCoordsSE(features)
    rds <- new("RoarDataset", se, treatmentBams = list(), controlBams = list(), 
        prePostCoords = features, step = 3, cores = 1)
    preElems <- grep("_PRE$", mcols(rds@prePostCoords)$gene_id)
    postElems <- grep("_POST$", mcols(rds@prePostCoords)$gene_id)
    preCoords <- rds@prePostCoords[preElems, ]
    se <- SummarizedExperiment(assays = rep(list(matrix(nrow = 2, 
        ncol = 4)), 2), rowRanges = preCoords, colData = DataFrame(row.names = c("treatment_pre", 
        "treatment_post", "control_pre", "control_post")))
    rowRanges(rds) <- rowRanges(se)
    colData(rds) <- colData(se)
    assays(rds) <- assays(se)
    names(assays(rds)) <- "counts"
    rds@postCoords <- rds@prePostCoords[postElems, ]
    length(rds@treatmentBams) <- 1
    length(rds@controlBams) <- 1
    assay(rds, 1)[1, 1] <- 5000
    assay(rds, 1)[1, 2] <- NA
    assay(rds, 1)[1, 3] <- 300
    assay(rds, 1)[1, 4] <- NA
    assay(rds, 1)[2, 1] <- 1000
    assay(rds, 1)[2, 2] <- NA
    assay(rds, 1)[2, 3] <- 10
    assay(rds, 1)[2, 4] <- NA
    dat <- fpkmResults(rds)
    expect_equal(833333, dat[1, "treatmentValue"], tolerance = .5, check.attributes=F)
    expect_equal(967742, dat[1, "controlValue"], tolerance = .5, check.attributes=F)
    expect_equal(1666667, dat[2, "treatmentValue"], tolerance = .5, check.attributes=F)
    expect_equal(322581, dat[2, "controlValue"], tolerance = .5, check.attributes=F)
})

test_that("test_fpkmResults_singleSamples_length", {
    gene_id <- c("A_PRE", "A_POST", "B_PRE", "B_POST")
    length <- c(1, 1, 50, 20)
    features <- GRanges(seqnames = Rle(rep("chr1", length(gene_id))), 
        strand = strand(rep("+", length(gene_id))), ranges = IRanges(start = c(1000, 
            2000, 1, 2), width = c(1000, 900, 100, 2)), DataFrame(gene_id, 
            length))
    se <- getPreCoordsSE(features)
    rds <- new("RoarDataset", se, treatmentBams = list(), controlBams = list(), 
        prePostCoords = features, step = 3, cores = 1)
    preElems <- grep("_PRE$", mcols(rds@prePostCoords)$gene_id)
    postElems <- grep("_POST$", mcols(rds@prePostCoords)$gene_id)
    preCoords <- rds@prePostCoords[preElems, ]
    se <- SummarizedExperiment(assays = rep(list(matrix(nrow = 2, 
        ncol = 4)), 2), rowRanges = preCoords, colData = DataFrame(row.names = c("treatment_pre", 
        "treatment_post", "control_pre", "control_post")))
    rowRanges(rds) <- rowRanges(se)
    colData(rds) <- colData(se)
    assays(rds) <- assays(se)
    names(assays(rds)) <- "counts"
    rds@postCoords <- rds@prePostCoords[postElems, ]
    length(rds@treatmentBams) <- 1
    length(rds@controlBams) <- 1
    assay(rds, 1)[1, 1] <- 5000
    assay(rds, 1)[1, 2] <- NA
    assay(rds, 1)[1, 3] <- 300
    assay(rds, 1)[1, 4] <- NA
    assay(rds, 1)[2, 1] <- 1000
    assay(rds, 1)[2, 2] <- NA
    assay(rds, 1)[2, 3] <- 10
    assay(rds, 1)[2, 4] <- NA
    dat <- fpkmResults(rds)
    expect_equal(833333333, dat[1, "treatmentValue"], tolerance = .5, check.attributes=F)
    expect_equal(967741935, dat[1, "controlValue"], tolerance = .5, check.attributes=F)
    expect_equal(3333333, dat[2, "treatmentValue"], tolerance = .5, check.attributes=F)
    expect_equal(645161, dat[2, "controlValue"], tolerance = .5, check.attributes=F)
})

test_that("test_pvalueCorrectFilter_singleSamples", {
    gene_id <- c("A_PRE", "A_POST", "B_PRE", "B_POST", "C_PRE", 
        "C_POST", "D_PRE", "D_POST")
    features <- GRanges(seqnames = Rle(rep("chr1", length(gene_id))), 
        strand = strand(rep("+", length(gene_id))), ranges = IRanges(start = c(1000, 
            2000, 1, 2, 3, 4, 5, 6), width = rep(1, length(gene_id))), 
        DataFrame(gene_id))
    se <- getPreCoordsSE(features)
    rds <- new("RoarDataset", se, treatmentBams = list(), controlBams = list(), 
        prePostCoords = features, step = 3, cores = 1, paired = FALSE)
    preElems <- grep("_PRE$", mcols(rds@prePostCoords)$gene_id)
    postElems <- grep("_POST$", mcols(rds@prePostCoords)$gene_id)
    preCoords <- rds@prePostCoords[preElems, ]
    se <- SummarizedExperiment(assays = rep(list(matrix(nrow = 4, 
        ncol = 4)), 2), rowRanges = preCoords, colData = DataFrame(row.names = c("treatment_pre", 
        "treatment_post", "control_pre", "control_post")))
    rowRanges(rds) <- rowRanges(se)
    colData(rds) <- colData(se)
    assays(rds) <- assays(se)
    names(assays(rds)) <- "counts"
    rds@postCoords <- rds@prePostCoords[postElems, ]
    length(rds@treatmentBams) <- 1
    length(rds@controlBams) <- 1
    assay(rds, 1)[, "treatment_pre"] <- c(1, 1e+09, 1e+09, 20)
    assay(rds, 1)[, "control_pre"] <- c(20, 1e+09, 1e+09, 20)
    assay(rds, 2)[, "treatment_pre"] <- c(1, 1, 1, 1)
    assay(rds, 2)[, "treatment_post"] <- c(1, 1, 1, 1)
    assay(rds, 2)[, "control_pre"] <- c(1, 1, 1, 1)
    assay(rds, 2)[, "control_post"] <- c(2e-05, 0.001, 0.001, 
        0.01)
    dat <- pvalueCorrectFilter(rds, 1, 0.02, "bonferroni")
    dat_wanted <- data.frame(row.names = c("B", "C"), mM_treatment = c(1, 
        1), mM_control = c(1, 1), roar = c(1, 1), pval = c(0.003, 
        0.003), treatmentValue = c(5e+08, 5e+08), controlValue = c(5e+08, 
        5e+08))
    expect_equal(dat_wanted, dat, tolerance = 1e-6, check.attributes=F)
})

test_that("test_pvalueFilter_mulSamples", {
    gene_id <- c("A_PRE", "A_POST", "B_PRE", "B_POST")
    features <- GRanges(seqnames = Rle(rep("chr1", length(gene_id))), 
        strand = strand(rep("+", length(gene_id))), ranges = IRanges(start = c(1000, 
            2000, 1, 2), width = rep(1, length(gene_id))), DataFrame(gene_id))
    se <- getPreCoordsSE(features)
    rds <- new("RoarDataset", se, treatmentBams = list(), controlBams = list(), 
        prePostCoords = features, step = 3, cores = 1, paired = FALSE)
    preElems <- grep("_PRE$", mcols(rds@prePostCoords)$gene_id)
    postElems <- grep("_POST$", mcols(rds@prePostCoords)$gene_id)
    preCoords <- rds@prePostCoords[preElems, ]
    se <- SummarizedExperiment(assays = rep(list(matrix(nrow = 2, 
        ncol = 4)), 2), rowRanges = preCoords, colData = DataFrame(row.names = c("treatment_pre", 
        "treatment_post", "control_pre", "control_post")))
    rowRanges(rds) <- rowRanges(se)
    colData(rds) <- colData(se)
    assays(rds) <- assays(se)
    names(assays(rds)) <- "counts"
    rds@postCoords <- rds@prePostCoords[postElems, ]
    length(rds@treatmentBams) <- 2
    length(rds@controlBams) <- 2
    assay(rds, 1)[, "treatment_pre"] <- c(1, 1e+09)
    assay(rds, 1)[, "control_pre"] <- c(20, 1e+09)
    assay(rds, 2)[, "treatment_pre"] <- c(1, 1)
    assay(rds, 2)[, "treatment_post"] <- c(2, 1)
    assay(rds, 2)[, "control_pre"] <- c(10, 1)
    assay(rds, 2)[, "control_post"] <- c(0.01, 0.1)
    rds@pVals <- SummarizedExperiment(assays = matrix(nrow = 2, 
        ncol = 4), rowRanges = preCoords, colData = DataFrame(row.names = c("pvalue_1_1", 
        "pvalue_1_2", "pvalue_2_1", "pvalue_2_2")))
    assay(rds@pVals, 1)[1, ] <- seq(1, 4)
    assay(rds@pVals, 1)[2, ] <- c(1, 1, 1, 5)
    dat <- pvalueFilter(rds, 0.01, 3)
    dat_wanted <- data.frame(row.names = c("A", "B"), mM_treatment = c(1, 
        1), mM_control = c(2, 1), roar = c(10, 1), pval = c(0.01, 
        0.1), pvalue_1_1 = c(1, 1), pvalue_1_2 = c(2, 1), pvalue_2_1 = c(3, 
        1), pvalue_2_2 = c(4, 5), treatmentValue = c(1, 1e+09), 
        controlValue = c(20, 1e+09), nUnderCutoff = c(2, 3))
    expect_equal(dat_wanted, dat, tolerance = 1e-6, check.attributes=F)
})

test_that("test_pvalueFilter_mulSamples_paired", {
    gene_id <- c("A_PRE", "A_POST", "B_PRE", "B_POST")
    features <- GRanges(seqnames = Rle(rep("chr1", length(gene_id))), 
        strand = strand(rep("+", length(gene_id))), ranges = IRanges(start = c(1000, 
            2000, 1, 2), width = rep(1, length(gene_id))), DataFrame(gene_id))
    se <- getPreCoordsSE(features)
    rds <- new("RoarDataset", se, treatmentBams = list(), controlBams = list(), 
        prePostCoords = features, step = 3, cores = 1, paired = TRUE)
    preElems <- grep("_PRE$", mcols(rds@prePostCoords)$gene_id)
    postElems <- grep("_POST$", mcols(rds@prePostCoords)$gene_id)
    preCoords <- rds@prePostCoords[preElems, ]
    se <- SummarizedExperiment(assays = rep(list(matrix(nrow = 2, 
        ncol = 4)), 2), rowRanges = preCoords, colData = DataFrame(row.names = c("treatment_pre", 
        "treatment_post", "control_pre", "control_post")))
    rowRanges(rds) <- rowRanges(se)
    colData(rds) <- colData(se)
    assays(rds) <- assays(se)
    names(assays(rds)) <- "counts"
    rds@postCoords <- rds@prePostCoords[postElems, ]
    length(rds@treatmentBams) <- 2
    length(rds@controlBams) <- 2
    assay(rds, 1)[, "treatment_pre"] <- c(1, 1e+09)
    assay(rds, 1)[, "control_pre"] <- c(20, 1e+09)
    assay(rds, 2)[, "treatment_pre"] <- c(1, 1)
    assay(rds, 2)[, "treatment_post"] <- c(2, 1)
    assay(rds, 2)[, "control_pre"] <- c(10, 1)
    assay(rds, 2)[, "control_post"] <- c(0.01, 0.1)
    rds@pVals <- SummarizedExperiment(assays = matrix(nrow = 2, 
        ncol = 2), rowRanges = preCoords, colData = DataFrame(row.names = c("pvalue_1_2", 
        "pvalue_2_1")))
    assay(rds@pVals, 1)[1, ] <- c(1, 20)
    assay(rds@pVals, 1)[2, ] <- c(5, 5)
    dat <- pvalueFilter(rds, 0.01, 0.05)
    dat_wanted <- data.frame(row.names = "A", mM_treatment = 1, 
        mM_control = 2, roar = 10, pval = 0.01, pvalue_1_2 = 1, 
        pvalue_2_1 = 20, treatmentValue = 1, controlValue = 20)
    expect_equal(dat_wanted, dat, tolerance = 1e-6, check.attributes=F)
})

test_that("test_pvalueFilter_singleSamples", {
    gene_id <- c("A_PRE", "A_POST", "B_PRE", "B_POST", "C_PRE", 
        "C_POST", "D_PRE", "D_POST")
    features <- GRanges(seqnames = Rle(rep("chr1", length(gene_id))), 
        strand = strand(rep("+", length(gene_id))), ranges = IRanges(start = c(1000, 
            2000, 1, 2, 3, 4, 5, 6), width = rep(1, length(gene_id))), 
        DataFrame(gene_id))
    se <- getPreCoordsSE(features)
    rds <- new("RoarDataset", se, treatmentBams = list(), controlBams = list(), 
        prePostCoords = features, step = 3, cores = 1, paired = FALSE)
    preElems <- grep("_PRE$", mcols(rds@prePostCoords)$gene_id)
    postElems <- grep("_POST$", mcols(rds@prePostCoords)$gene_id)
    preCoords <- rds@prePostCoords[preElems, ]
    se <- SummarizedExperiment(assays = rep(list(matrix(nrow = 4, 
        ncol = 4)), 2), rowRanges = preCoords, colData = DataFrame(row.names = c("treatment_pre", 
        "treatment_post", "control_pre", "control_post")))
    rowRanges(rds) <- rowRanges(se)
    colData(rds) <- colData(se)
    assays(rds) <- assays(se)
    names(assays(rds)) <- "counts"
    rds@postCoords <- rds@prePostCoords[postElems, ]
    length(rds@treatmentBams) <- 1
    length(rds@controlBams) <- 1
    assay(rds, 1)[, "treatment_pre"] <- c(1, 1e+09, 1e+09, 20)
    assay(rds, 1)[, "control_pre"] <- c(20, 1e+09, 1e+09, 20)
    assay(rds, 2)[, "treatment_pre"] <- c(1, 1, -1, 2)
    assay(rds, 2)[, "treatment_post"] <- c(2, 1, 1, 1)
    assay(rds, 2)[, "control_pre"] <- c(10, 1, NA, 1)
    assay(rds, 2)[, "control_post"] <- c(0.01, 0.1, 0.1, 0.001)
    dat <- pvalueFilter(rds, 0.01, 0.05)
    dat_wanted <- data.frame(row.names = c("A", "D"), mM_treatment = c(1, 
        2), mM_control = c(2, 1), roar = c(10, 1), pval = c(0.01, 
        0.001), treatmentValue = c(0.5, 10), controlValue = c(10, 
        10))
    expect_equal(dat_wanted, dat, tolerance = 1e-6, check.attributes=F)
})

test_that("test_standardFilter_singleSamples", {
    gene_id <- c("A_PRE", "A_POST", "B_PRE", "B_POST", "C_PRE", 
        "C_POST", "D_PRE", "D_POST")
    features <- GRanges(seqnames = Rle(rep("chr1", length(gene_id))), 
        strand = strand(rep("+", length(gene_id))), ranges = IRanges(start = c(1000, 
            2000, 1, 2, 3, 4, 5, 6), width = rep(1, length(gene_id))), 
        DataFrame(gene_id))
    se <- getPreCoordsSE(features)
    rds <- new("RoarDataset", se, treatmentBams = list(), controlBams = list(), 
        prePostCoords = features, step = 3, cores = 1, paired = FALSE)
    preElems <- grep("_PRE$", mcols(rds@prePostCoords)$gene_id)
    postElems <- grep("_POST$", mcols(rds@prePostCoords)$gene_id)
    preCoords <- rds@prePostCoords[preElems, ]
    se <- SummarizedExperiment(assays = rep(list(matrix(nrow = 4, 
        ncol = 4)), 2), rowRanges = preCoords, colData = DataFrame(row.names = c("treatment_pre", 
        "treatment_post", "control_pre", "control_post")))
    rowRanges(rds) <- rowRanges(se)
    colData(rds) <- colData(se)
    assays(rds) <- assays(se)
    names(assays(rds)) <- "counts"
    rds@postCoords <- rds@prePostCoords[postElems, ]
    length(rds@treatmentBams) <- 1
    length(rds@controlBams) <- 1
    assay(rds, 1)[, "treatment_pre"] <- c(1, 1e+09, 1e+09, 20)
    assay(rds, 1)[, "control_pre"] <- c(20, 1e+09, 1e+09, 20)
    assay(rds, 2)[, "treatment_pre"] <- c(1, -1, 1, 1)
    assay(rds, 2)[, "treatment_post"] <- c(1, 1, 1, 1)
    assay(rds, 2)[, "control_pre"] <- c(1, 1, NA, 1)
    assay(rds, 2)[, "control_post"] <- c(0.1, 0.1, 0.1, 0.1)
    dat <- standardFilter(rds, 1)
    dat_wanted <- data.frame(row.names = "D", mM_treatment = 1, 
        mM_control = 1, roar = 1, pval = 0.1, treatmentValue = 10, 
        controlValue = 10)
    expect_equal(dat_wanted, dat, tolerance = 1e-6, check.attributes=F)
})

test_that("test_totalResults_mulSamples", {
    gene_id <- c("A_PRE", "A_POST", "B_PRE", "B_POST")
    features <- GRanges(seqnames = Rle(rep("chr1", length(gene_id))), 
        strand = strand(rep("+", length(gene_id))), ranges = IRanges(start = c(1000, 
            2000, 1, 2), width = c(1000, 900, 1, 2)), DataFrame(gene_id))
    se <- getPreCoordsSE(features)
    rds <- new("RoarDataset", se, treatmentBams = list(), controlBams = list(), 
        prePostCoords = features, step = 3, cores = 1)
    preElems <- grep("_PRE$", mcols(rds@prePostCoords)$gene_id)
    postElems <- grep("_POST$", mcols(rds@prePostCoords)$gene_id)
    preCoords <- rds@prePostCoords[preElems, ]
    se <- SummarizedExperiment(assays = rep(list(matrix(nrow = 2, 
        ncol = 4)), 2), rowRanges = preCoords, colData = DataFrame(row.names = c("treatment_pre", 
        "treatment_post", "control_pre", "control_post")))
    rowRanges(rds) <- rowRanges(se)
    colData(rds) <- colData(se)
    assays(rds) <- assays(se)
    names(assays(rds)) <- "counts"
    rds@postCoords <- rds@prePostCoords[postElems, ]
    length(rds@treatmentBams) <- 2
    length(rds@controlBams) <- 3
    assay(rds, 2)[1, 1] <- 1
    assay(rds, 2)[1, 2] <- 2
    assay(rds, 2)[1, 3] <- 3
    assay(rds, 2)[1, 4] <- 4
    assay(rds, 2)[2, 1] <- 5
    assay(rds, 2)[2, 2] <- 6
    assay(rds, 2)[2, 3] <- 7
    assay(rds, 2)[2, 4] <- 8
    rds@pVals <- SummarizedExperiment(assays = matrix(nrow = 2, 
        ncol = 6), rowRanges = preCoords, colData = DataFrame(row.names = c("pvalue_1_1", 
        "pvalue_1_2", "pvalue_1_3", "pvalue_2_1", "pvalue_2_2", 
        "pvalue_2_3")))
    assay(rds@pVals, 1)[1, ] <- seq(1, 6)
    assay(rds@pVals, 1)[2, ] <- seq(10, 15)
    dat <- totalResults(rds)
    dat_wanted <- data.frame(row.names = c("A", "B"), mM_treatment = c(1, 
        5), mM_control = c(2, 6), roar = c(3, 7), pval = c(4, 
        8), pvalue_1_1 = c(1, 10), pvalue_1_2 = c(2, 11), pvalue_1_3 = c(3, 
        12), pvalue_2_1 = c(4, 13), pvalue_2_2 = c(5, 14), pvalue_2_3 = c(6, 
        15))
    expect_equal(dat_wanted, dat, tolerance = 1e-6, check.attributes=F)
})

test_that("test_totalResults_singleSamples", {
    gene_id <- c("1_PRE", "1_POST", "2_PRE", "2_POST")
    features <- GRanges(seqnames = Rle(rep("chr1", length(gene_id))), 
        strand = strand(rep("+", length(gene_id))), ranges = IRanges(start = c(1000, 
            2000, 1, 2), width = c(1000, 900, 1, 2)), DataFrame(gene_id))
    se <- getPreCoordsSE(features)
    rds <- new("RoarDataset", se, treatmentBams = list(), controlBams = list(), 
        prePostCoords = features, step = 3, cores = 1)
    preElems <- grep("_PRE$", mcols(rds@prePostCoords)$gene_id)
    postElems <- grep("_POST$", mcols(rds@prePostCoords)$gene_id)
    preCoords <- rds@prePostCoords[preElems, ]
    se <- SummarizedExperiment(assays = rep(list(matrix(nrow = 2, 
        ncol = 4)), 2), rowRanges = preCoords, colData = DataFrame(row.names = c("treatment_pre", 
        "treatment_post", "control_pre", "control_post")))
    rowRanges(rds) <- rowRanges(se)
    colData(rds) <- colData(se)
    assays(rds) <- assays(se)
    names(assays(rds)) <- "counts"
    rds@postCoords <- rds@prePostCoords[postElems, ]
    length(rds@treatmentBams) <- 1
    length(rds@controlBams) <- 1
    assay(rds, 2)[1, 1] <- 1
    assay(rds, 2)[1, 2] <- 2
    assay(rds, 2)[1, 3] <- 3
    assay(rds, 2)[1, 4] <- 4
    assay(rds, 2)[2, 1] <- 5
    assay(rds, 2)[2, 2] <- 6
    assay(rds, 2)[2, 3] <- 7
    assay(rds, 2)[2, 4] <- 8
    dat <- totalResults(rds)
    dat_wanted <- data.frame(row.names = c("1", "2"), mM_treatment = c(1, 
        5), mM_control = c(2, 6), roar = c(3, 7), pval = c(4, 
        8))
    expect_equal(dat_wanted, dat, tolerance = 1e-6, check.attributes=F)
})

#!/usr/bin/env Rscript
# Script to perform Roar analysis. 
# Requires a gtf with _PRE and _POST gene_ids and bam files from the two 
# conditions to be compared.

checkReadable <- function(filename) {
   res <- file.access(names=filename, mode=4) == 0
   if (!res) {
      warning(paste(filename, "is not readable", sep=" "))
   }
   res
}

arguments <- matrix(c(
   'help', 'h', 0, "logical",
   'debug', 'd', 1, "character",
   'gtf' , 'a', 1, "character",
   'treatment'  , 't', 1, "character",
   'control'  , 'c', 1, "character"
), ncol=4, byrow=T)

library(getopt)
opt <- getopt(arguments)

if (!is.null(opt$help)) {
   stop(getopt(arguments, command=get_Rscript_filename(), usage=TRUE))
}

if (is.null(opt$gtf)) {
   stop("Missing gtf [-a filename] annotation option\n")
}

if (is.null(opt$treatment) | is.null(opt$control)) {
   stop("Missing treatment or control [-t, -c followed by comma separated bam files] param")
}
options(error = function() traceback(2))
library(roar)
library(rtracklayer)
library(GenomicAlignments)
treatmentBams <- as.vector(unlist(strsplit(opt$treatment, ",")))
controlBams <- as.vector(unlist(strsplit(opt$control, ",")))
stranded <- FALSE

if (!all(sapply(c(treatmentBams, controlBams, opt$gtf), checkReadable))) {
   stop("One of the given files does not exist or is not readable")  
}


gtfGRanges<- import(opt$gtf)
chrs <- seqlevels(gtfGRanges)

orderBam <- function(bam) {
   tmp <- tempfile()
   ordered <- sortBam(bam, tmp, byQname=FALSE, maxMemory=512)
   garbage <- indexBam(ordered)
   return(ordered)
}

orderedRightBams <- lapply(treatmentBams, orderBam)
orderedLeftBams <- lapply(controlBams, orderBam)

workOnChr <- function(chr) {
   write(paste("Working on", chr), stderr())
   reduced <- keepSeqlevels(gtfGRanges, chr) 
   coords <- c(start(reduced), end(reduced))  # To keep strandness in consideration! Is this needed?
   begin <- min(coords)
   end <- max(coords)
   spanChr <- GRanges(seqnames=chr,ranges=IRanges(start=begin,width=end-begin+1))
   loadBam <- function(bam) {
      param <- ScanBamParam(which=spanChr)
      res <- readGAlignments(file=bam, param = param)
      return(res)
   } 
   
   treatmentBamsGenomicAlignments <- lapply(orderedRightBams, loadBam)
   controlBamsGenomicAlignments <- lapply(orderedLeftBams, loadBam)
   
   rds <- RoarDatasetMultipleAPA(treatmentBamsGenomicAlignments, controlBamsGenomicAlignments, reduced)
   
   # Get counts
   rds <- countPrePost(rds, stranded)
   # Get m/M and Roar
   rds <- computeRoars(rds)
   # Fisher test
   rds <- computePvals(rds)
   res <- countResults(rds)
   return(res)
}

allRes <- lapply(chrs, workOnChr)
meltedRes <- do.call("rbind", allRes)

sumTreatment <- sum(meltedRes[,"counts_treatment"])
sumControl <- sum(meltedRes[,"counts_control"])
meltedRes$treatmentFpkm <- (meltedRes[,"counts_treatment"]*1000000000)/(meltedRes[,"length"]*sumTreatment)
meltedRes$controlFpkm <- (meltedRes[,"counts_control"]*1000000000)/(meltedRes[,"length"]*sumControl)
meltedRes$treatmentValue <- NULL
meltedRes$controlValue <- NULL
write.table(meltedRes, sep="\t", quote=FALSE)

unlink(orderedRightBams)
unlink(orderedLeftBams)
treatmentBai <- sub(".bam", ".bai", orderedRightBams)
controlBai <- sub(".bam", ".bai", orderedLeftBams)
unlink(treatmentBai)
unlink(controlBai)

if (!is.null(opt$debug)) {
   save.image(file=opt$debug)
}

%\VignetteIndexEntry{Identify differential APA usage from RNA-seq alignments}
%\VignettePackage{roar}
\documentclass{article}
\author{Elena Grassi\\
Department of Molecular Biotechnologies and Health Sciences,\\ MBC, University of Turin, Italy\\
}
\usepackage{Sweave}
\usepackage{hyperref}
% From dexseq vignette
\newcommand{\Robject}[1]{{\small\texttt{#1}}}
\newcommand{\Rfunction}[1]{\Robject{#1}}
\newcommand{\Rpackage}[1]{\textit{#1}}
\newcommand{\Rclass}[1]{\textit{#1}}

\title{Identify differential APA usage from RNA-seq alignments}

\date{\Rpackage{roar} version \Sexpr{packageDescription("roar")$Version} (Last revision 2014-09-24)}
\begin{document}
\SweaveOpts{concordance=TRUE}

\maketitle

\begin{abstract}
  This vignette describes how to use the Bioconductor package
  \Rpackage{roar} to detect preferential usage of shorter isoforms via alternative
  poly-adenylation from RNA-seq data. The approach is based on Fisher test
  to detect disequilibriums in the number of reads falling over the 3'UTRs
  when comparing two biological conditions. The name roar means
  ``ratio of a ratio'': counts and fragments lengths 
  are used to calculate the prevalence
  of the short isoform over the long one in both conditions, therefore the ratio of
  these ratios represents the relative ``shortening'' (or lengthening) in one
  condition with respect to the other.
  As input, \Rpackage{roar}
  uses alignments files for the two conditions and coordinates of the 3'UTRs with
  alternative polyadenylation sites. 
  Here, the method is demonstrated on the data from the package \Rpackage{RNAseqData.HNRNPC.bam.chr14}.
  To cite this software, please refer to \Rfunction{citation("roar")}.
\end{abstract}

\tableofcontents

\section{Introduction}
The alternative polyadenylation mechanism at the basis of the existence of short and long isoforms
of the same gene is reviewed in Elkon et al~\cite{Elkon2013}.
For the relevance of this phenomenon to development and cancer see~\cite{Sandberg2008,Mayr2009,Ji2009}.

\section{Input data}
A roar analysis starts from some alignments, obtained via standard RNAseq experiments
and data processing.
It is possible to build a \Rclass{RoarDataset} object giving its constructor
two lists of bam files obtained from samples of the two conditions to be compared or to
use another constructor that takes directly two lists of \Rclass{GAlignments}.

\subsection{Alternative polyadenylation annotations}
The other information needed to build a \Rclass{RoarDataset} object are
3'UTRs coordinates (with canonical and alternative polyadenylation sites) 
for the genes that one wants to analyze - these could be given using a gtf 
file or a GRanges object.

The gtf file should have an attribute (\emph{metadata column} for the \Rclass{GRanges} object) called 
``gene\_id'' that ends with ``\_PRE'' or ``\_POST'' to address respectively the short and the long isoforms.

An element in the annotation is considered ``PRE'' 
(i.e. common to the short and long isoform of the transcript)  if its gene\_id ends with ``\_PRE''. 
If it ends with ``\_POST'' it is considered the portion present only in the long isoform.
The prefix of gene\_id should be a unique identifier for the gene and each identifier has to be
associated with only one ``\_PRE'' and one ``\_POST'', leading to two genomic regions associated to each   
gene\_id.

The gtf can also contain an attribute (\emph{metadata column} for the \Rclass{GRanges} object) 
that represents the lengths of PRE and POST portions on the transcriptome.
If this is omitted the lengths on the genome are used instead (this will results in somewhat inaccurate
length corrections for genes with either the PRE or the POST portion comprising an intron). 
Note that right now every gtf entry (or none of them) should have it.

In the package we added a file (examples/apa.gtf) that follows these directives: it is build
upon the hg19 human genome release using PolyA\_DB (\cite{Cheng2006}) version 2 to obtain
coordinates of alternative polyadenylation (from now on addressed as APA) sites.
Every gene with at least one APA site is considered; 
their longest transcript is used to get the ``classical'' polyadenylation syte (canonical end
of the longest transcript stored
in UCSC) and the most proximal  (with respect to the TSS, that is the farther to the canonical end
of the transcript; but preferring sites still in the 3'UTR) reported APA
site found in PolyA\_DB is used to define the end of the short isoform. 
Therefore
we define the ``PRE'' portion as the stretch of DNA starting from the beginning of the exon containing 
(or proximal to) the APA site and ending at the site position, while the POST portion starts 
there and ends
at the canonical transcript end. Using only the nearest exon to the APA site and not the entire
transcript to define the short isoform avoids spurious signals deriving from other alternative
splicing events.

To summarize the package requirements are: every entry in the gtf should have an attribute called ``gene\_id''
formed by a prefix representing unique gene identifiers and a suffix. The suffix defines if the given 
coordinates refer to the portion of this gene common between the short and the long isoform (``\_PRE'')
or to the portion pertaining only to the long isoform (``\_POST''). Every gene identifier must appear
two (and only two) times in the gtf, one with the suffix ``\_PRE'' and one with ``\_POST''. An example
of how PRE and POST are defined is reported in Figure~\ref{Fig1}.

\begin{figure}[h]
  \includegraphics[width=\textwidth]{prepost.pdf}
  \caption{An example of PRE/POST portions definition}
  \label{Fig1}
\end{figure}

\section{Analysis steps}
The suggestion is to use one of the two wrapper scripts 
(\emph{roarWrapper} or \emph{roarWrapper\_chrBychr}) or at least follow their tracks. 
The principal steps of the analysis are performed
by different methods that receive a \Rclass{RoarDataset} object as an argument
and returns it with the needed step performed. It is also possible
to call directly the methods to get the results (eg. \emph{totalResults}),
in this way all the preceding steps will be performed automatically.

Loading and handling alignments data require a lot of memory: if you have small datasets
(up until 2.5Gb, ~58.5 million reads with a length of 50 bp)
4 Gb of RAM will be more than sufficient (peak memory 
usage, gotten via the ps command with the keyword rss, when comparing two samples of 2.5Gb: ~
1.3Gb) and you can use the \emph{roarWrapper}  script.
For bigger datasets (eg. 20Gb, ~413 million reads with a length of 101bp) 
you will have to split your alignments in 
small chunks: \emph{roarWrapper\_chrBychr} works on single chromosomes.

Please note than when working in this way FPKM
values returned by \emph{fpkmResults} will be different from those obtained with the analysis
performed using a single \Rclass{RoarDataset} on the whole alignments - in this case you can extract counts 
with \emph{countResults} and knit together real FPKM values afterwards, if you need them (\emph{roarWrapper\_chrBychr} does this).

In the following sections we will show an example of this package usage on the 
\Rpackage{RNAseqData.HNRNPC.bam.chr14} package data and the annotation given in the package itself.

\subsection{Creating a RoarDataset object}
The analysis begins by creating a \Rclass{RoarDataset} object that holds
the alignment data (4 HNRNPC ko samples and 3 control samples) 
and the annotation regarding 3'UTRs coordinates.

<<loadLibAndCreateRoarObject>>=
library(roar)
library(rtracklayer)
library(RNAseqData.HNRNPC.bam.chr14)
gtf <- system.file("examples", "apa.gtf", package="roar")
# HNRNPC ko data
bamTreatment <- RNAseqData.HNRNPC.bam.chr14_BAMFILES[seq(5,8)]
# control (HeLa wt)
bamControl <- RNAseqData.HNRNPC.bam.chr14_BAMFILES[seq(1,3)]
rds <- RoarDatasetFromFiles(bamTreatment, bamControl, gtf)
@
\subsection{Obtaining counts}
This is the first step in the Roar analysis: it counts reads overlapping with the
PRE/POST portions defined in the given gtf/GRanges annotation.
Reads of the given bam are accounted for with the following rules:

\begin{enumerate}
       \item reads that align on only one of the given features are assigned to that feature, even if the
       overlap is not complete 
       \item reads that align on both a PRE and a POST feature of the same gene (spanning reads) are 
       assigned
       to the POST one, considering that they have clearly been obtained from the longer isoform
\end{enumerate}
If the alignments were derived from a strand-aware protocol it is possible
to consider strandness when counting reads using the stranded=TRUE argument; in this
case we use FALSE as long as we don't want to consider strandness.

<<countReads>>= 
rds <- countPrePost(rds, FALSE)
@

\subsection{Computing roar}
This is the second step in the Roar analysis: it computes the ratio of prevalence
of the short and long isoforms for every gene in the treatment and control condition (m/M) and their
ratio, roar, that indicates if there is a relative shortening-lengthening in a condition
versus the other one (the choice of calling them ``treatment'' and ``control'' is simply a convention
and reflects the fact that to calculate the roar the m/M of the treatment condition is used as the 
numerator).
For details about the m/M calculations see section~\ref{appendix}.

A roar > 1 for a given gene means that in the treatment condition
that gene has an higher ratio of short vs long isoforms with respect to the control condition (and
the opposite for roar < 1). Negative or NA m/M or roar occurs in not definite situations,
such as counts equal to zero for PRE or POST portions.
If for one of the conditions there is more than one sample, like in this example
that has four and three samples for the two conditions, then calculations are performed on average counts.

<<computeRoars>>= 
rds <- computeRoars(rds)
@
\subsection{Computing p-values}
This is the third step in the Roar analyses: it applies a Fisher test comparing counts falling
on the PRE and POST portion in the treatment and control conditions for every gene. If there are multiple 
samples for
a condition every combination of comparisons between the samples lists is considered - for example in this case we have four and three samples, therefore it will calculate 12 p-values.

<<computePvals>>= 
rds <- computePvals(rds)
@

Computing all samples pairing is not the preferrable choice when a paired experimental design exists:
in this case only the correct pairs between control and treatment samples should be compared with the Fisher
test; then their p-values can be combined following the Fisher method (\cite{Fisher}) 
because we have different
independent tests on the same null hypothesis. For these situations there is an alternative way
to obtain the p-values: \emph{computePairedPvals}, which require the user to specify as arguments
not only the \Rclass{RoarDataset} object but also two vectors containining the ordered samples
for treatment and control - see the related man page for details.

\subsection{Obtaining results}
There are various functions aimed at extracting results from a \Rclass{RoarDataset};
the first one is \emph{totalResults}:

<<getResults>>= 
results <- totalResults(rds)
@
It returns a dataframe with gene\_id as rownames and several columns: ``mM\_treatment'', ``mM\_control'', 
``roar'',
``pval'' and a number of columns called ``pvalue\_X\_Y'' 
(when there are multiple samples for at least one condition, X refers to the treatment samples and
Y to the control ones).
The first two columns have the value of the ratio showing the relative abundance of the
short isoform with respect to the long one in the treatment (or control) condition, the third one
represents the roar and it's bigger than one when the shorter isoform is relatively more expressed in the 
treatment
condition (and the other way around when it's smaller than one). The pval column stores the results
of the Fisher tests when both conditions have a single sample, while the multiplication of all the
possible combinations p-values for un-paired multiple samples analysis or the combined
p-values following the Fisher method if \emph{computePairedPvals} has been used.

In this case for example there will be columns ``pvalue\_1\_1'' up to ``pvalue\_4\_3'': the first
number represents the number of the considered sample for the treatment condition, while
the second for the control one (the samples are considered in the same order given in
the lists passed to the \Rclass{RoarDataset} constructors).

All other functions are simply helper functions: \emph{fpkmResults} adds columns "treatmentValue" and 
"controlValue"
representing the level of expression of the relative gene in the two conditions, while
\emph{countResults} puts in that columns the number of reads that were counted on the PRE portion of genes.

\emph{standardFilter} get a parameter with a cutoff for the FPKM value and 
selects only genes with an expression
higher than that and without any NA/Inf value for m/M or roar (deriving from not definite situations
with counts equal to zero). \emph{pvalueFilter} further selects only genes with a Fisher test pvalue
smaller than the given cutoff in the single sample case, while if more than one sample has been given
for one condition it adds a column named "nUnderCutoff" that stores the number of p-values that are
smaller than the given cutoff. It is worth noting that all the p-values are nominal with this
method, while \emph{pvalueCorrectFilter}
applies an user requested multiple correction procedure (after FPKM filtering) for single sample
or paired (that is when \emph{computePairedPvals} has been used) analyses and filters with the
given cutoff on these corrected value (for un-paired multiple samples design this does exactly
the same as \emph{pvalueFilter}).

For example in our case we can see how many genes have a pvalue under 0.05
in every comparisons in this way:

<<getFilteredResults>>= 
results_filtered <- pvalueFilter(rds, fpkmCutoff=-Inf, 
                                 pvalCutoff=0.05)
nrow(results_filtered[results_filtered$nUnderCutoff==12,])
@
Note that for FPKM we used -Inf as a cutoff because in this case we didn't want to filter out
genes considering their expression values (and because we used only partial alignments
deriving from a single chromosome, thus the usual FPKM cutoff are not directly applicable).

Another thing to point out is that the FPKM values derive from the total reads falling
over the genes given in the annotation, therefore when doing the analysis ``stepwise''
(eg. with roarWrapper\_chrByChr) their values will be different than those obtained
performing the analysis all together. \emph{countResults} could be used in this situations
to save single steps counts and then calculate ``whole'' FPKM values.

\subsection{MultipleAPA analyses}

In order to allow users to efficiently study polyadenylation for gene harbouring more than one alternative
polyadenylation site with a single analysis we have developed the \Rclass{RoarDatasetMultipleAPA}
class.
A MultipleAPA analysis computes several roar values and p-values for each gene: one for every 
possible combinations of APA-canonical end of a gene (i.e. the end of its last exon). 
This is more efficient than performing several different ``standard'' roar analyses choosing 
the PRE and POST portions corresponding to different APAs because reads overlaps are computed only once.

The gtf or \Rclass{Granges} used for this kind af analyses are different from the ones described
previously: they should containing a suitable annotation of alternative APA sites and gene exonic structure.
Minimal requirements are: attributes (metadata columns for \Rclass{Granges}) called "gene", "apa" and "type."
APA should be single bases falling over one of the given genes and need to have the
metadata column "type" equal to "apa" and the "apa" column with unambiguous ids and the corresponding gene id pasted together with an underscore. 
The "gene" metadata columns for these entries should not be initialized.
All the studied gene exons need to be reported, in this case the metadata column "gene" should contain 
the gene id (the same one reported for each gene APAs) while "type" should be set to "gene" and "apa" to NA.
All apa entries assigned to a gene should have coordinates that falls inside it and every gene that appears 
should contain at least one APA.

Annotation files that follow these rules can be found at \url{https://github.com/vodkatad/roar/wiki/Identify-differential-APA-usage-from-RNA-seq-alignments}. 
From the user point of view the steps needed to perform a multipleAPA analysis are exactly the same of
the standard one, the only difference lies in the output of methods that report results as long as in this
case we have more than a single roar value for each gene. The details are reported in the corresponding
man pages.

\subsection{Genes with overlapping 3'UTRs}

When two genes overlap reads that align on the shared coordinates could not be 
assigned to either of them in a straightforward way - in the single APA analyses we
discard those reads while for technical reasons in the multiple APA analyses we perform
the alignment separately for genes on the plus and minus strand, thus the
reads that overlap genes on different strands will be counted for all of them, while those
on genes on the same strand will be correctly discarded.
In order to overcome this problem in the multipleAPA analyses it is possible to filter
the gtf or \Rclass{Granges} annotation used before creating the \Rclass{RoarDatasetMultipleAPA} object removing
all genes with overlapping portions. 

Note that for the single APA analyses roar alignment procedure could
detect overlaps only
between the given PRE and POST portions and therefore possible overlaps with other portions of genes
are not considered - again it is possible to filter the used annotations beforehand.


\section{Appendix}\label{appendix}
\subsection{m/M calculations}

To correctly evaluate the quantities ratios of the short and long isoforms counts of the reads
falling over the two
transcript portions (as previously defined: PRE is the portion common between the two isoforms, while POST is
the one present only in the longer transcript) should be accounted for considering the fact that those
falling over PRE could have been obtained from both isoforms while the others falling
on POST derive exclusively 
from the longer one; another more trivial question that has to be addressed is that reads fall with larger
frequencies on longer stretches of RNA.

We can say that the total number of reads falling over a transcript in its entirety ($N$) derives
from the relative abundance of the two isoforms and their potential of generating reads; 
that is: $ N = \epsilon_{M}  M + \epsilon_{m}  m$
where $m$ is the quantity of the short isoform, $M$ of the long one and $\epsilon$ identifies their
efficiency in generating reads.

Assuming the equiprobability of reads distribution (that is each nucleotide has the same probability 
of finding itself in a read) the efficiency in generating reads of the two isoforms is
related to their lengths (with the addition of a constant of proportionality $k$):\vspace{0.4cm} \\
\begin{math}
\epsilon_{M} = k (l_{PRE}+l_{POST})
\end{math}
\vspace{0.4cm}\\
\begin{math}
\epsilon_{m} = k (l_{PRE})
\end{math}
\vspace{0.4cm}\\

Defining $l_{PRE}$ as the length of the PRE portion and $l_{POST}$ as the length of the POST 
we can now obtain the number of reads falling on the two portions as:\vspace{0.4cm}\\
\begin{math}
\#r_{PRE} = \epsilon_{M} M (\frac{l_{PRE}}{l_{PRE}+l_{POST}}) + \epsilon_{m}  m
\end{math}
\vspace{0.2cm}\\
and:
\vspace{0.2cm}\\
\begin{math}
\#r_{POST} = \epsilon_{M} M (\frac{l_{POST}}{l_{PRE}+l_{POST}}) 
\end{math}
\vspace{0.5cm}

These two equations reflect the fact that all the reads deriving from the short isoform 
($\epsilon_{m}  m$) fall on the PRE portion
while the ones deriving from the long one are distributed over the PRE and POST portions depending
on their lengths.

We can now setup a system of equations aimed at obtaining the $m/M$ value in terms of the numbers of
reads falling over the two portions and their lengths.\vspace{0.5cm}

We start from: 
\begin{math} \left\{
  \begin{array}{l}
    \#r_{PRE} = \epsilon_{M} M (\frac{l_{PRE}}{l_{PRE}+l_{POST}}) + \epsilon_{m}  m \\
    \#r_{POST} = \epsilon_{M} M (\frac{l_{POST}}{l_{PRE}+l_{POST}})  
  \end{array} \right.
\end{math}\vspace{0.5cm}\\

Then with simple algebraic steps the system can be solved yielding the formula to obtain $m/M$ using only read counts and lengths:\vspace{0.5cm}\\
\begin{math}
m/M = \frac{l_{POST} \#r_{PRE}}{l_{PRE} \#r_{POST}} -1
\end{math}
\vspace{0.5cm} \\

As a simple emblematic case suppose that the PRE and POST portions have the same lengths and that the
short and long isoforms are in perfect equilibrium (i.e. they are present in a cell in equal numbers).
In this situation we will find on the PRE portion two times the number of reads falling on the POST 
one because half of them will derive from the long isoforms and the other half from the short ones. 
In this case the equation correcly gives us an $m/M$ equal to 1.

In the previous discussion we have ignored reads falling across the PRE/POST boundaries. As long
as they can derive only from the long isoform it is reasonable to assign them to $\#r_{POST}$.

Portion lengths should be corrected to keep in consideration read lengths and the just cited
assignment to POST of the spanning reads, therefore:\vspace{0.5cm}\\
\begin{math}
l^{\prime}_{PRE} = l_{PRE}
\end{math}
\vspace{0.2cm} \\
\begin{math}
l^{\prime}_{POST} = l_{POST} + readLength - 1
\end{math}
\vspace{0.5cm} \\ % find a suitable relative measure XXX

Normally we should have added readLength to both the lengths but in this case we do not expect
reads to fall after the POST portion (that is the end of transcripts) and thus we only have to correct
for the spanning reads. We do not subtract the same value from $l_{PRE}$ as long as in theory we could expect
reads to fall at its 5' (i.e. reads falling across that exon and the previous one or those 
from still unspliced transcripts).\\ These corrected lengths are those used for the $m/M$ calculations.

\bibliography{roar} % bib in the same dir as .Rnw
\bibliographystyle{plain}

\end{document}

\name{meanAcrossAssays}
     \alias{meanAcrossAssays}
     \title{Private/inner/helper method to get average counts across samples}
     \description{
       This method \bold{should not} be used by package users. It gets average counts
       for "pre" or "post" portions (depending on the wantedColumns argument)
       given the list of assays for one of the two conditions.
     }
     \usage{
      meanAcrossAssays(assays, wantedColumns)
     }
     \arguments{
       \item{assays}{  A list of matrixes/dataframes. }
        \item{wantedColumns}{
           The name of the columns ("pre" or "post") whose means should be computed.
           Average will be calculated on the corresponding rows of the list of matrices/dataframe,
           working on the given column.
        }
      }
      \value{
         The pvalue for the two.sided Fisher test.
      }
\keyword{meanAcrossAssays}

\name{getFisher}
     \alias{getFisher}
     \title{Private/inner/helper method to perform Fisher test}
     \description{
       This method \bold{should not} be used by package users. Given a numerical vector
       of length 4 it will perform a Fisher test and return the p-value for the two.sided
       test. Non-integer values will be rounded.
     }
     \usage{
      getFisher(counts)
     }
     \arguments{
       \item{counts}{
           A numerical vector of length 4.
        }
      }
      \value{
         The pvalue for the two.sided Fisher test.
      }
\keyword{getFisher}

\name{totalResults}
     \alias{totalResults}
     \alias{totalResults, RoarDataset}
     \alias{totalResults, RoarDatasetMultipleAPA}
     \title{Returns a dataframe with results of the analysis for a \code{\linkS4class{RoarDataset}}
     or a \code{\linkS4class{RoarDatasetMultipleAPA}} object}
     \description{
         The last step of a classical Roar analyses: it returns a dataframe containing m/M values, roar 
         values and pvalues.
     }
     \usage{
      totalResults(rds)
     }
     \arguments{
       \item{rds}{
           The \code{\linkS4class{RoarDataset}} or \code{\linkS4class{RoarDatasetMultipleAPA}} with all the analysis 
           steps (\code{\link{countPrePost}},
            \code{\link{computeRoars}}, \code{\link{computePvals}}) performed.}
      }
      \value{
         The \code{\linkS4class{RoarDataset}} or the \code{\linkS4class{RoarDatasetMultipleAPA}} object given as rds 
         with all the analysis steps performed.
         If one or more steps hadn't been performed they will be called automatically.
         The resulting dataframe will have the "gene_id" of the initial annotation as row names (without
         the trailing "_PRE"/"_POST") and as columns the m/M ratio for the treatment and control conditions,
         the roar value and the Fisher test pvalue (respectively: mM_treatment, mM_control, roar, pval).
         If more than one sample has been given for a condition the "pval" column will contain the
         product of all the comparisons pvalue and there will be other columns containing
         the pvalues resulting from all the pairwise treatment vs control contrasts, with names "pvalue_X_Y" 
         where
         X represent the position of the sample in the treatment list of bam files (or GappedAlignment) and
         Y the position for the control list.
         When using \code{\linkS4class{RoarDatasetMultipleAPA}} this dataframe will report multiple
         results for each gene that corresponds to the pairings between every APA associated with that
         gene in the gtf and the gene's end - rownames in this case will be in the form geneid_apaid.
         \bold{WARNING}: this method does not filter in any way the results, therefore there will be negative
         m/M values/ROAR and also NA - in these cases there aren't enough information to draw a conclusion
         about the shortening/lengthening of the gene in the given samples and thus the pvalues
         should not be kept in consideration. Furthermore there isn't any filter on the expression level
         of the genes. See \code{\link{fpkmResults}}, \code{\link{standardFilter}} and
         \code{\link{pvalueFilter}} about results filtering possibilities.
      }
     \examples{
   library(GenomicAlignments)
   gene_id <- c("A_PRE", "A_POST", "B_PRE", "B_POST")
   features <- GRanges(
      seqnames = Rle(c("chr1", "chr1", "chr2", "chr2")),
      strand = strand(rep("+", length(gene_id))),
      ranges = IRanges(
         start=c(1000, 2000, 3000, 3600),
         width=c(1000, 900, 600, 300)),
      DataFrame(gene_id)
   )
   rd1 <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(1000), cigar = "300M", strand = strand("+"))
   rd2 <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(2000), cigar = "300M", strand = strand("+"))
   rd3 <- GAlignments("a", seqnames = Rle("chr2"), pos = as.integer(3000), cigar = "300M", strand = strand("+"))
   rds <- RoarDataset(list(c(rd1,rd2)), list(rd3), features)
   rds <- countPrePost(rds, FALSE)
   rds <- computeRoars(rds)
   rds <- computePvals(rds)
   dat <- totalResults(rds)
    }
\keyword{totalResults}

\name{RoarDatasetFromFiles}
     \alias{RoarDatasetFromFiles}
     \title{Creates a \code{\link[=RoarDataset-class]{RoarDataset}} object}
     \description{
       This function creates an \code{\link[=RoarDataset-class]{RoarDataset}} object from two lists and a gtf with a suitable annotation of alternative APA sites.
     }
     \usage{
      RoarDatasetFromFiles(treatmentBams, controlBams, gtf)
     }
     \arguments{
            \item{treatmentBams}{
           A list of filenames of bam alignments with data for the treatment condition  (by convention it is 
            considered the ``treated'' condition: this simply means
            that the package will compute roar values (ratios of the m/M) using this condition as the numerator)
           to be considered.}
        \item{controlBams}{
           A list of filenames of bam alignments with data for the control condition
           to be considered.}
       \item{gtf}{
           A filename of a gtf with coordinates for the portions of transcripts
           that has to be considered pertaining to the short (or long) isoform.
           This gtf must have an attribute called "gene_id" that ends with "_PRE"
           or "_POST" to address respectively the short and the long isoform.
           A ready-to-go gtf, with coordinates derived from the PolyADB on
           the human genome (version hg19), is available in the "examples" package directory.
           An element in the annotation
           is considered "PRE" (i.e. common to the short and long isoform of the transcript) if its gene_id
           feature in the gtf ends with "_PRE". If
           it ends with "_POST" it is considered the portion present only in the long isoform.
           The prefix of gene_id should be an identifier for the gene and each identifier has to be
           associated with only one "_PRE" and one "_POST", leading to two genomic region associated to each   
           gene_id.
           The gtf can also contain an attribute that represents the lengths of PRE and POST portions on the transcriptome.
           If this is omitted the lengths on the genome are used instead. Note that right now every gtf entry
           (or none of them) should have it.
        }
      }
      \value{
         A \code{\link[=RoarDataset-class]{RoarDataset}} object ready to be analyzed via the other methods.
      }
     \seealso{
       \code{\link{RoarDataset}}
     }
     \examples{
     #rds <- RoarDatasetFromFiles(treatmentBams, controlBams, gtf)
    }
\keyword{RoarDatasetFromFiles}

\name{standardFilter}
     \alias{standardFilter}
     \alias{standardFilter, RoarDataset}
     \alias{standardFilter, RoarDatasetMultipleAPA}
     \title{Returns a dataframe with results of the analysis for a \code{\linkS4class{RoarDataset}} object
          or a \code{\linkS4class{RoarDatasetMultipleAPA}} object}
     \description{
         The last step of a classical Roar analyses: it returns a dataframe containing m/M values, roar 
         values, pvalues and estimates of expression (a measure recalling FPKM). Only the genes
         with an expression estimate bigger than a given cutoff will be considered.
         %and the pvalue
         %will be corrected for multiple comparisons (with the Bonferroni method).
     }
     \usage{
      standardFilter(rds, fpkmCutoff)
     }
     \arguments{
       \item{rds}{
           The \code{\linkS4class{RoarDataset}} or the \code{\linkS4class{RoarDatasetMultipleAPA}} 
           with all the analysis steps (\code{\link{countPrePost}},
            \code{\link{computeRoars}}, \code{\link{computePvals}}) performed.
            If one or more steps hadn't been performed they will be called automatically.}
      \item{fpkmCutoff}{
         The cutoff that will be used to determine if a gene is expressed or not.}
      }
      \value{
         For \code{\linkS4class{RoarDataset}} and \code{\linkS4class{RoarDatasetMultipleAPA}}: 
         
         The resulting dataframe will be identical to that returned by \code{\link{fpkmResults}} but
         it will contains rows relative only with genes with an expression estimate (treatment or controlValue)
         bigger than the given fpkmCutoff in both the conditions and with sensitive m/M and roar values (it
         removes negative or NA
         m/M values/roar - these values arise when there aren't enough information to draw a conclusion
         about the shortening/lengthening of the gene).
         %The pvalues (for multiple samples their multiplication, i.e.
         %the value in the "pval" column) will then be corrected for multiple comparisons (Bonferroni method)
         %only considering those genes, that are considered as "expressed" in both the biological conditions
         %in analysis.
      }
     \examples{
   library(GenomicAlignments)
   gene_id <- c("A_PRE", "A_POST", "B_PRE", "B_POST")
   features <- GRanges(
      seqnames = Rle(c("chr1", "chr1", "chr2", "chr2")),
      strand = strand(rep("+", length(gene_id))),
      ranges = IRanges(
         start=c(1000, 2000, 3000, 3600),
         width=c(1000, 900, 600, 300)),
      DataFrame(gene_id)
   )
   rd1 <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(1000), cigar = "300M", strand = strand("+"))
   rd2 <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(2000), cigar = "300M", strand = strand("+"))
   rd3 <- GAlignments("a", seqnames = Rle("chr2"), pos = as.integer(3000), cigar = "300M", strand = strand("+"))
   rds <- RoarDataset(list(c(rd1,rd2)), list(rd3), features)
   rds <- countPrePost(rds, FALSE)
   rds <- computeRoars(rds)
   rds <- computePvals(rds)
   dat <- standardFilter(rds, 1)
    }
\keyword{standardFilter}

\name{countPrePost} 
     \alias{countPrePost}
     \alias{countPrePost,RoarDataset,logical}
     \alias{countPrePost,RoarDatasetMultipleAPA}
     \title{Counts reads falling over PRE/POST portions of the given transcripts}
     \description{
       This is the first step in the Roar analyses: it counts reads overlapping with the
       PRE/POST portions defined in the given gtf/GRanges annotation. See \code{\link{RoarDataset}} for details
       on how to define these portions.
       Reads of the given bam annotation files that falls over this portion are accounted for with the following
       rules:
       
       1- reads that align on only one of the given features are assigned to that feature, even if the
       overlap is not complete 
       2- reads that align on both a PRE and a POST feature of the same gene (spanning reads) are 
       assigned to the POST one, considering that they have clearly been obtained from the longest isoform
       
       If the stranded argument is set to TRUE then strandness is considered when counting reads.
       When rds is a \code{\link{RoarDatasetMultipleAPA}} counts are obtained on more than two portions for
       each transcript in order to be able to efficiently evaluate multiple APA sites. 
       The option stranded=TRUE is still not implemented for \code{\linkS4class{RoarDatasetMultipleAPA}}.
       
     }
     \usage{
      %\S4method{countPrePost}{RoarDataset,logical}(rds, stranded=FALSE)
      %\S4method{countPrePost}{RoarDatasetMultipleAPA,logical}(rds, stranded=FALSE)
      countPrePost(rds, stranded=FALSE)
     }
     \arguments{
       \item{rds}{
           The \code{\linkS4class{RoarDataset}} or the \code{\linkS4class{RoarDatasetMultipleAPA}} which contains the             alignments and annotation informations over which
           counts will be performed.}
       \item{stranded}{
           A logical indicating if strandness should be considered when counting reads or not.
           Default=FALSE. \bold{WARNING}: not implemented (ignored) when using \code{\linkS4class{RoarDatasetMultipleAPA}}.
       }
      }
      \value{
         The \code{\linkS4class{RoarDataset}} object given as rds with the counting reads phase of the analysis done. Counts will be held in the RoarDataset object itself in the case of single samples, while
      }
     \examples{
   library(GenomicAlignments)
   gene_id <- c("A_PRE", "A_POST", "B_PRE", "B_POST")
   features <- GRanges(
      seqnames = Rle(c("chr1", "chr1", "chr2", "chr2")),
      strand = strand(rep("+", length(gene_id))),
      ranges = IRanges(
         start=c(1000, 2000, 3000, 3600),
         width=c(1000, 900, 600, 300)),
      DataFrame(gene_id)
   )
   rd1 <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(1000), cigar = "300M", strand = strand("+"))
   rd2 <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(2000), cigar = "300M", strand = strand("+"))
   rd3 <- GAlignments("a", seqnames = Rle("chr2"), pos = as.integer(3000), cigar = "300M", strand = strand("+"))
   rds <- RoarDataset(list(c(rd1,rd2)), list(rd3), features)
   rds <- countPrePost(rds)
    }
\keyword{countPrePost}

\name{RoarDatasetMultipleAPA-class}
\Rdversion{1.1}
\docType{class}
\alias{RoarDatasetMultipleAPA-class}
\alias{computePvals,RoarDatasetMultipleAPA-method}
\alias{computePairedPvals,RoarDatasetMultipleAPA,numeric,numeric-method}
\alias{computeRoars,RoarDatasetMultipleAPA-method}
\alias{cores,RoarDatasetMultipleAPA-method}
\alias{countPrePost,RoarDatasetMultipleAPA,logical-method}
\alias{countPrePost,RoarDatasetMultipleAPA-method}
\alias{fpkmResults,RoarDatasetMultipleAPA-method}
\alias{countResults,RoarDatasetMultipleAPA-method}
\alias{totalResults,RoarDatasetMultipleAPA-method}
\alias{standardFilter,RoarDatasetMultipleAPA,numeric-method}
\alias{pvalueFilter,RoarDatasetMultipleAPA,numeric,numeric-method}
\alias{pvalueCorrectFilter,RoarDatasetMultipleAPA,numeric,numeric,character-method}

\title{Class \code{"RoarDatasetMultipleAPA"}}
\description{
RoarDataset - a class to perform 3'UTR shortening analyses
}
\section{Objects from the Class}{
Objects of thiss class should be created using the functions
\code{\link{RoarDatasetMultipleAPA}} or \code{\link{RoarDatasetMultipleAPAFromFiles}}, ideally
the raw \code{\link{new}} method should never be invoked by end users. Then
to perform the analysis the user should call, in order: countPrePost, computeRoars, computePvals and
one of the methods to format results. This class is used to allow
efficient analyses that allow to study more than one APA site for each gene: internally
it uses a \code{\link{RoarDataset}} object that stores PRE/POST counts for all possible alternative
APA choices for each gene.
}
\section{Slots}{
  \describe{
    \item{\code{treatmentBams}:}{Object of class \code{"list"} - a list of GappedAlignment objects for the first condition (by convention it is considered the ``treated'' condition) in analysis. }
    \item{\code{controlBams}:}{Object of class \code{"list"} - a list of GappedAlignment objects for the second condition (by convention it is considered the ``control'' condition) in analysis. }
    \item{\code{geneCoords}:}{Object of class \code{"GRangesList"} - private object that represents the exon structures of genes in study. }
    \item{\code{apaCoords}:}{Object of class \code{"GRangesList"} - private object that represents the APA fallin on genes in study.  }
    \item{\code{fragments}:}{Object of class \code{"GRangesList"} - private object used to efficiently count reads falling on short and long isoforms. }
    \item{\code{prePostDef}:}{Object of class \code{"list"} - private object representing all possible short and long isoforms. }
    \item{\code{roars}:}{Object of class \code{"list"} - private object with a list of \code{\link{RoarDataset}} objects, each one representing all possible PRE/POST choices for a single gene. }
    \item{\code{corrTreatment}:}{\code{"numeric"} slot - private, integer representing the mean length of reads for the treatment samples. }
    \item{\code{corrControl}:}{\code{"numeric"} slot - private, integer representing the mean length of reads for the control samples. }
    \item{\code{paired}:}{\code{"logical"} slot - private. }
    \item{\code{step}:}{\code{"numeric"} slot - private. }
    \item{\code{cores}:}{\code{"numeric"} slot - private. }
  }
}

\section{Methods}{
  \describe{
    \item{\code{\link{countPrePost}}}{\code{signature(rds = "RoarDatasetMultipleAPA", stranded = "logical")}: Counts reads falling over all the possible PRE/POST portions of the given transcripts. WARNING: stranded = TRUE is still unsupported and could give unpredictable results. }
    \item{\code{\link{computeRoars}}}{\code{signature(rds = "RoarDatasetMultipleAPA")}: Computes m/M and roar values for this \code{\linkS4class{RoarDatasetMultipleAPA}} object. }
    \item{\code{\link{computePvals}}}{\code{signature(rds = "RoarDatasetMultipleAPA")}: Computes pvalues (Fisher test) for this \code{\linkS4class{RoarDatasetMultipleAPA}} object. }
    \item{totalResults}{\code{signature(rds = "RoarDatasetMultipleAPA")}: Returns a dataframe with results of the analysis for a \code{\linkS4class{RoarDatasetMultipleAPA}} object. }
    \item{\code{\link{fpkmResults}}}{\code{signature(rds = "RoarDatasetMultipleAPA")}: 
    The last step of a classical Roar analyses: it returns a dataframe containing m/M values, roar 
    values, pvalues and estimates of expression (a measure recalling FPKM). }
    \item{\code{\link{countResults}}}{\code{signature(rds = "RoarDatasetMultipleAPA")}: 
    The last step of a classical Roar analyses: it returns a dataframe containing m/M values, roar 
    values, pvalues and estimates of expression (counts of reads falling over a gene). }
    \item{\code{\link{standardFilter}}}{\code{signature(rds = "RoarDatasetMultipleAPA", fpkmCutoff = "double")}: Returns a dataframe with results of the analysis for a \code{\linkS4class{RoarDatasetMultipleAPA}} object. }
    \item{\code{\link{pvalueFilter}}}{\code{signature(rds = "RoarDatasetMultipleAPA", fpkmCutoff = "double", pvalCutoff = "double")}: ... }
    \item{\code{\link{cores}}}{\code{signature(rds = "RoarDatasetMultipleAPA")}: returns the number of cores used for computation, right now always 1. }
    }
}

\author{
Elena Grassi, PhD student in  Biomedical Sciences and Oncology - Dept. of Molecular Biotechnologies and Health Sciences, Molecular Biotechnology Center, Torino
}

\examples{
showClass("RoarDatasetMultipleAPA")
}
\keyword{classes}

\name{RoarDataset-class}
\Rdversion{1.1}
\docType{class}
\alias{RoarDataset-class}
\alias{computePvals,RoarDataset-method}
\alias{computePairedPvals,RoarDataset,numeric,numeric-method}
\alias{computeRoars,RoarDataset-method}
\alias{cores,RoarDataset-method}
\alias{countPrePost,RoarDataset,logical-method}
\alias{countPrePost,RoarDataset-method}
\alias{fpkmResults,RoarDataset-method}
\alias{countResults,RoarDataset-method}
\alias{totalResults,RoarDataset-method}
\alias{standardFilter,RoarDataset,numeric-method}
\alias{pvalueFilter,RoarDataset,numeric,numeric-method}
\alias{pvalueCorrectFilter,RoarDataset,numeric,numeric,character-method}

\title{Class \code{"RoarDataset"}}
\description{
RoarDataset - a class to perform 3'UTR shortening analyses
}
\section{Objects from the Class}{
Objects of thiss class should be created using the functions
\code{\link{RoarDataset}} or \code{\link{RoarDatasetFromFiles}}, ideally
the raw \code{\link{new}} method should never be invoked by end users. Then
to perform the analysis the user should call, in order: countPrePost, computeRoars, computePvals and
one of the methods to format results.
}
\section{Slots}{
  \describe{
    \item{\code{treatmentBams}:}{Object of class \code{"list"} - a list of GappedAlignment objects for the first condition (by convention it is considered the ``treated'' condition) in analysis. }
    \item{\code{controlBams}:}{Object of class \code{"list"} - a list of GappedAlignment objects for the second condition (by convention it is considered the ``control'' condition) in analysis. }
    \item{\code{prePostCoords}:}{Object of class \code{"GRanges"} - represents the APA sites coords, defining "PRE" (last exon coords up until the alternative APA, defining the shorter isoform) and "POST" (from the alternative APA to the ``standard'' one) regions of the genes. }
    \item{\code{postCoords}:}{Object of class \code{"GRanges"} - private object. }
    \item{\code{countsTreatment}:}{Object of class \code{"RangedSummarizedExperiment"} - private object. }
    \item{\code{countsControl}:}{Object of class \code{"RangedSummarizedExperiment"} - private object. }
    \item{\code{pVals}:}{Object of class \code{"RangedSummarizedExperiment"} - private object. }
    \item{\code{paired}:}{\code{"logical"} slot - private. }
    \item{\code{step}:}{\code{"numeric"} slot - private. }
    \item{\code{cores}:}{\code{"numeric"} slot - private. }
    \item{\code{metadata}:}{\code{"list"} slot - private. }
    \item{\code{rowRanges}:}{Object of class \code{"GRangesORGRangesList"} - private object. }
    \item{\code{colData}:}{Object of class \code{"DataFrame"} - private object. }
    \item{\code{assays}:}{Object of class \code{"Assays"} - private object. }
  }
}
\section{Extends}{
Class \code{"\linkS4class{RangedSummarizedExperiment}"}, directly.
}
\section{Methods}{
  \describe{
    \item{\code{\link{countPrePost}}}{\code{signature(rds = "RoarDataset", stranded = "logical")}: Counts reads falling over PRE/POST portions of the given transcripts. }
    \item{\code{\link{computeRoars}}}{\code{signature(rds = "RoarDataset")}: Computes m/M and roar values for this \code{\linkS4class{RoarDataset}} object. }
    \item{\code{\link{computePvals}}}{\code{signature(rds = "RoarDataset")}: Computes pvalues (Fisher test) for this \code{\linkS4class{RoarDataset}} object. }
    \item{totalResults}{\code{signature(rds = "RoarDataset")}: Returns a dataframe with results of the analysis for a \code{\linkS4class{RoarDataset}} object. }
    \item{\code{\link{fpkmResults}}}{\code{signature(rds = "RoarDataset")}: 
    The last step of a classical Roar analyses: it returns a dataframe containing m/M values, roar 
    values, pvalues and estimates of expression (a measure recalling FPKM). }
    \item{\code{\link{countResults}}}{\code{signature(rds = "RoarDataset")}: 
    The last step of a classical Roar analyses: it returns a dataframe containing m/M values, roar 
    values, pvalues and estimates of expression (counts over PRE portions). }
    \item{\code{\link{standardFilter}}}{\code{signature(rds = "RoarDataset", fpkmCutoff = "double")}: Returns a dataframe with results of the analysis for a \code{\linkS4class{RoarDataset}} object. }
    \item{\code{\link{pvalueFilter}}}{\code{signature(rds = "RoarDataset", fpkmCutoff = "double", pvalCutoff = "double")}: ... }
    \item{\code{\link{cores}}}{\code{signature(rds = "RoarDataset")}: returns the number of cores used for computation, right now always 1. }
    }
}
%\references{
% website for now? XXX
%}
\author{
Elena Grassi, PhD student in  Biomedical Sciences and Oncology - Dept. of Molecular Biotechnologies and Health Sciences, Molecular Biotechnology Center, Torino
}

\examples{
showClass("RoarDataset")
}
\keyword{classes}

\name{combineFisherMethod}
     \alias{combineFisherMethod}
     \title{Private/inner/helper method to combine pvalues of independent test with the Fisher method}
     \description{
       This method \bold{should not} be used by package users. Given a numerical vector
       of pvalues, which should be obtained from independent tests on the same null hypothesis, 
         this will give the combined pvalue following the Fisher method.
     }
     \usage{
      combineFisherMethod(pvals)
     }
     \arguments{
       \item{pvals}{
           A numerical vector with pvalues of independent tests on the same H0.
      }}
      \value{
         The combined pvalue given by the Fisher method.
      }
\keyword{combineFisherMethod}

\name{RoarDatasetMultipleAPAFromFiles}
     \alias{RoarDatasetMultipleAPAFromFiles}
     \title{Creates a \code{\link[=RoarDatasetMultipleAPA-class]{RoarDatasetMultipleAPA}} object}
     \description{
       This function creates an \code{\link[=RoarDatasetMultipleAPA-class]{RoarDatasetMultipleAPA}} object from two lists and a gtf with a suitable annotation of 
       alternative APA sites and exonic structures of genes.
                  A MultipleAPA analysis computes several roar values and p-values for each gene: one 
           for every possible combination of APA-canonical end of a gene (i.e. the
           end of its last exon). This is more efficient than performing several different ``standard'' roar analyses
           choosing the PRE and POST portions corresponding to different APAsbecause reads overlaps are computed only once.
     }
     \usage{
      RoarDatasetMultipleAPAFromFiles(treatmentBams, controlBams, gtf)
     }
     \arguments{
            \item{treatmentBams}{
           A list of filenames of bam alignments with data for the treatment condition  (by convention it is 
            considered the ``treated'' condition: this simply means
            that the package will compute roar values (ratios of the m/M) using this condition as the numerator)
           to be considered.}
        \item{controlBams}{
           A list of filenames of bam alignments with data for the control condition
           to be considered.}
       \item{gtf}{
           A filename of a gtf with coordinates for alternative APA sites and gene exonic structure.
           This gtf must have three attributes called "gene", "apa" and "type" to distinguish different features.
           APA should be single bases falling over one of the given genes and need to have the attribute "type" equal 
           to "apa" and the "apa" attribute
           composed of unambiguous id and the corresponding gene id pasted together with an underscore. The "gene"
           attributes for these entries should not be initialized.
           All the studied gene exons need to be reported, in this case the attribute "gene" should contain the gene id (the same
           one reported for each gene APAs) while "type" should be set to "gene" and "apa" to NA.
           All apa entries assigned to a gene should have coordinates that falls inside it and every gene that
           appears should contain at least one APA.
           A ready-to-go gtf, with coordinates derived from the PolyADB on
           the human genome (version hg19), is available in the "examples" package directory.
        }
      }
      \value{
         A \code{\link[=RoarDatasetMultipleAPA-class]{RoarDatasetMultipleAPA}} object ready to be analyzed via the other methods.
      }
     \seealso{
       \code{\link{RoarDatasetMultipleAPA}}
     }
     \examples{
     #rds <- RoarDatasetMultipleAPAFromFiles(treatmentBams, controlBams, gtf)
    }
\keyword{RoarDatasetFromFiles}

\name{checkStep}
     \alias{checkStep}
     \title{Private/inner/helper method to check the order of the invoked analysis methods}
     \description{
       This method \bold{should not} be used by package users. It gets an rds object and a
       required number of analysis step and, if possible, calls the requested method to reach
       that step. It returns the object and a logical value that tells if the analysis can go on.
     }
     \usage{
      checkStep(rds, neededStep)
     }
     \arguments{
       \item{rds}{
           A \code{\linkS4class{RoarDataset}} object.}
       \item{neededStep}{
           The analysis step where rds should be/arrive.    
       }
      }
      \value{
         A list containing a logical that shows if the needed step could be reached
         with rds and the object at the requested step. Check step won't repeat
         a step already done and the logical value will be FALSE in this case (and rds
         won't be returned modified).
      }
\keyword{checkStep}

\name{computePairedPvals}
     \alias{computePairedPvals}
     \alias{computePairedPvals, RoarDataset}
     \alias{computePairedPvals, RoarDatasetMultipleAPA}
     \title{Computes pvalues (Fisher test) on the read counts in this roar analysis}
     \description{
       This is the third step in the Roar analyses: it applies a Fisher test comparing counts falling
       on the PRE and POST portion in the treatment and control conditions for every gene. 
       The paired method should be used when the experimental setup offers multiple paired samples
       for the two conditions: that is foreach sample of the control condition there is a naturally
       paired one for the treatment (i.e. cells derived from the same plate divided in two groups
       and treated or not). For example in the below code sample treatment sample n.1 (rd1) is paired with
       control n.2 (rd4)
       and rd2 with rd3. The pvalue resulting from Fisher test applied on the different samples pairings will
       be combined with the Fisher method, therefore the pairs of samples should be independent between each other.
      }
     \usage{
      computePairedPvals(rds, treatmentSamples, controlSamples)
     }
     \arguments{
       \item{rds}{
           The \code{\linkS4class{RoarDataset}} or the \code{\linkS4class{RoarDatasetMultipleAPA}} 
           which contains the counts over PRE-POST portions
           in the two conditions to be compared via pvalues.}
      \item{treatmentSamples}{Numbers that represent the indexes of the treatmentBams/GappedAlign parameter given to
         the RoarDataset costructor and the order in which they are paired with control samples.}
      \item{controlSamples}{Numbers that represent the indexes of the controlBams/GappedAlign parameter given to
         the RoarDataset costructor and the order in which they are paired with treatment samples.}
      }      
      \value{
         The \code{\linkS4class{RoarDataset}} or the \code{\linkS4class{RoarDatasetMultipleAPA}} object given as rds
         with the compute pvalues phase of the analysis done. Pvalues will be held in the RoarDataset object itself in 
         the case of single samples, while
         in a separate slot otherwise, but end user normally should not analyze those directly but use
         \code{\link{totalResults}} or \code{\link{fpkmResults}} at the end of the analysis. 
      }
     \examples{
   library(GenomicAlignments)
   gene_id <- c("A_PRE", "A_POST", "B_PRE", "B_POST")
   features <- GRanges(
      seqnames = Rle(c("chr1", "chr1", "chr2", "chr2")),
      strand = strand(rep("+", length(gene_id))),
      ranges = IRanges(
         start=c(1000, 2000, 3000, 3600),
         width=c(1000, 900, 600, 300)),
      DataFrame(gene_id)
   )
   rd1 <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(1000), cigar = "300M", strand = strand("+"))
   rd2 <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(2000), cigar = "300M", strand = strand("+"))
   rd3 <- GAlignments("a", seqnames = Rle("chr2"), pos = as.integer(3000), cigar = "300M", strand = strand("+"))
   rd4 <- GAlignments("a", seqnames = Rle("chr2"), pos = as.integer(3400), cigar = "300M", strand = strand("+"))
   rds <- RoarDataset(list(rd1,rd2), list(rd3, rd4), features)
   rds <- countPrePost(rds, FALSE)
   rds <- computeRoars(rds)
   rds <- computePairedPvals(rds, c(1,2), c(2,1))
    }
\keyword{computePairedPvals}

\name{countResults}
     \alias{countResults}
     \alias{countResults, RoarDataset}
     \alias{countResults, RoarDatasetMultipleAPA}
     \title{Returns a dataframe with results of the analysis for a \code{\linkS4class{RoarDataset}} object
          or a \code{\linkS4class{RoarDatasetMultipleAPA}} object}
     \description{
         The last step of a classical Roar analyses: it returns a dataframe containing m/M values, roar 
         values, pvalues and estimates of expression (number of reads falling over the PRE portions).
     }
     \usage{
      countResults(rds)
     }
     \arguments{
       \item{rds}{
           The \code{\linkS4class{RoarDataset}} or the \code{\linkS4class{RoarDatasetMultipleAPA}} 
           with all the analysis steps (\code{\link{countPrePost}},
            \code{\link{computeRoars}}, \code{\link{computePvals}}) performed.
            If one or more steps hadn't been performed they will be called automatically.}
      }
      \value{
         The resulting dataframe will be identical to that returned by \code{link{totalResults}} but
         with two columns added: "treatmentValue" and "controlValue". These columns will contain a number
         that indicates the level of expression of the relative gene in the treatment (or control) condition.
         For \code{\linkS4class{RoarDataset}} this number represents the counts (averaged across samples when       
         applicable) 
         obtained for the PRE portion of the gene. For \code{\linkS4class{RoarDatasetMultipleAPA}} every
         possible PRE choice will have its corresponding reads counts assigned and also the length of
         the PRE portion (counting only exonic bases). See the vignette
         for more details.
      }
     \examples{
   library(GenomicAlignments)
   gene_id <- c("A_PRE", "A_POST", "B_PRE", "B_POST")
   features <- GRanges(
      seqnames = Rle(c("chr1", "chr1", "chr2", "chr2")),
      strand = strand(rep("+", length(gene_id))),
      ranges = IRanges(
         start=c(1000, 2000, 3000, 3600),
         width=c(1000, 900, 600, 300)),
      DataFrame(gene_id)
   )
   rd1 <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(1000), cigar = "300M", strand = strand("+"))
   rd2 <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(2000), cigar = "300M", strand = strand("+"))
   rd3 <- GAlignments("a", seqnames = Rle("chr2"), pos = as.integer(3000), cigar = "300M", strand = strand("+"))
   rds <- RoarDataset(list(c(rd1,rd2)), list(rd3), features)
   rds <- countPrePost(rds, FALSE)
   rds <- computeRoars(rds)
   rds <- computePvals(rds)
   dat <- countResults(rds)
    }
\keyword{countResults}

\name{roar-package}
\alias{roar-package}
\alias{roar}
\docType{package}
\title{
Identify differential APA usage from RNA-seq alignments
}
\description{
Identify preferential usage of APA sites, 
comparing two biological conditions, starting from known alternative sites and
alignments obtained from standard RNA-seq experiments.
}
\details{
The code{\linkS4class{RoarDataset}} class exposes methods to perform the whole analysis, in order to 
identify genes with preferential expression of long/short isoforms in a condition with respect to 
another one. The needed input data are alignments deriving from RNA-seq experiments of the
two conditions and a set of coordinates of APA sites for genes with an alternative APA site
proximal to the one used ``normally''.
}
\author{
Elena Grassi <grassi.e@gmail.com>
}
\keyword{ package }

\name{pvalueFilter}
     \alias{pvalueFilter}
     \alias{pvalueFilter, RoarDataset}
     \alias{pvalueFilter, RoarDatasetMultipleAPA}
     \title{Returns a dataframe with results of the analysis for a \code{\linkS4class{RoarDataset}} object
          or a \code{\linkS4class{RoarDatasetMultipleAPA}} object}
     \description{
         The last step of a classical Roar analyses: it returns a dataframe containing m/M values, roar 
         values, pvalues and estimates of expression (a measure recalling FPKM). Only the genes
         with an expression estimate bigger than a given cutoff will be considered. Also pvalues
         will be considered for filtering.
     }
     \usage{
      pvalueFilter(rds, fpkmCutoff, pvalCutoff)
     }
     \arguments{
       \item{rds}{
           The \code{\linkS4class{RoarDataset}} or the \code{\linkS4class{RoarDatasetMultipleAPA}} 
           with all the analysis steps (\code{\link{countPrePost}},
            \code{\link{computeRoars}}, \code{\link{computePvals}}) performed.
            If one or more steps hadn't been performed they will be called automatically.}
       \item{fpkmCutoff}{
         The cutoff that will be used to determine if a gene is expressed or not.}
       \item{pvalCutoff}{
         The cutoff that will be used to determine if a pvalue is significative or not.}
      }
      \value{
         For \code{\linkS4class{RoarDataset}}: 
         
         The resulting dataframe will be identical to that returned by \code{\link{standardFilter}} but
         after gene expression and m/M values filtering another step will be performed:
         for single samples comparisons only genes with a nominal pvalue smaller than the given cutoff
         will be considered, while for multiple samples a column (nUnderCutoff) will be added to the
         dataframe. This column will contain an integer number representing the number of comparisons
         between the samples of the two conditions that results in a nominal pvalue lower than the given    
         cutoff (pvalCutoff). For multiple samples with a paired design (i.e. if computePairedPvals was used)
         the pvalues of the requested pairings will be listed together with the combined pvalued
         obtained with the Fisher method and the filtering will be done on this pvalue.
        % \bold{WARNING}: the given cutoff will be used to determine significativeness of corrected pvalues in one situation (single sample for both conditions) and of nominal one in the other one.
        
         For \code{\linkS4class{RoarDatasetMultipleAPA}}: 
         for each gene we select the APA choice that is associated with the smallest p-value then proceed exactly as above.
      } 
     \examples{
   library("GenomicAlignments")
   gene_id <- c("A_PRE", "A_POST", "B_PRE", "B_POST")
   features <- GRanges(
      seqnames = Rle(c("chr1", "chr1", "chr2", "chr2")),
      strand = strand(rep("+", length(gene_id))),
      ranges = IRanges(
         start=c(1000, 2000, 3000, 3600),
         width=c(1000, 900, 600, 300)),
      DataFrame(gene_id)
   )
   rd1 <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(1000), cigar = "300M", strand = strand("+"))
   rd2 <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(2000), cigar = "300M", strand = strand("+"))
   rd3 <- GAlignments("a", seqnames = Rle("chr2"), pos = as.integer(3000), cigar = "300M", strand = strand("+"))
   rds <- RoarDataset(list(c(rd1,rd2)), list(rd3), features)
   rds <- countPrePost(rds, FALSE)
   rds <- computeRoars(rds)
   rds <- computePvals(rds)
   dat <- pvalueFilter(rds, 1, 0.05)
    }
\keyword{pvalueFilter}

\name{computeRoars}
     \alias{computeRoars}
     \alias{computeRoars, RoarDataset, numeric, numeric}
     \alias{computeRoars, RoarDatasetMultipleAPA, numeric, numeric}
     \title{Computes m/M and roar values}
     \description{
       This is the second step in the Roar analyses: it computes the ratio of prevalence
       of the short and long isoforms for every gene in the treatment and control condition (m/M) and their
       ratio, roar, that indicates if there is a relative shortening-lengthening in a condition
       over the other one. A roar > 1 for a given gene means that in the treatment condition
       that gene has an higher ratio of short vs long isoforms with respect to the control condition (and
       the opposite for roar < 1). Negative or NA m/M or roar occurs in not definite situations,
       such as counts equal to zero for PRE or POST portions.
       If for one of the conditions there are more than one samples then calculations are performed
       on average counts.
     }
     \usage{
         %\S4method{computeRoars}{RoarDataset,numeric,numeric}(rds, qwidthTreatment=NA, qwidthControl=NA)
         %\S4method{computeRoars}{RoarDataset,numeric,numeric}(rds, qwidthTreatment, qwidthControl)
         computeRoars(rds, qwidthTreatment=NA, qwidthControl=NA)
         computeRoars(rds, qwidthTreatment, qwidthControl)
     }
     \arguments{
       \item{rds}{
           The \code{\linkS4class{RoarDataset}} or the \code{\linkS4class{RoarDatasetMultipleAPA}} which contains the   
           counts over PRE-POST portions in the two conditions to be compared via roar.}
         \item{qwidthTreatment}{ The mean length of the reads in the treatment bam files
            - used internally for the interaction between
            \code{\linkS4class{RoarDataset}} and \code{\linkS4class{RoarDatasetMultipleAPA}} objects.
            The default (NA) calculates this value from the bam and should not be changed.}
         \item{qwidthControl}{The mean length of the reads in the control bam files 
            - used internally for the interaction between
            \code{\linkS4class{RoarDataset}} and \code{\linkS4class{RoarDatasetMultipleAPA}} objects. 
            The default (NA) calculates this value from the bam and should not be changed.}
      }
      \value{
         The \code{\linkS4class{RoarDataset}} or the \code{\linkS4class{RoarDatasetMultipleAPA}} object given as rds             with the computeRoars phase of the  
         analysis done. m/M and roars will be held in the RoarDataset object itself in the case of single samples,
         while
         in two slots otherwise, but end user normally should not analyze those directly but use
         \code{\link{totalResults}} or \code{\link{fpkmResults}} at the end of the analysis. 
      }
     \examples{
   library(GenomicAlignments)
   gene_id <- c("A_PRE", "A_POST", "B_PRE", "B_POST")
   features <- GRanges(
      seqnames = Rle(c("chr1", "chr1", "chr2", "chr2")),
      strand = strand(rep("+", length(gene_id))),
      ranges = IRanges(
         start=c(1000, 2000, 3000, 3600),
         width=c(1000, 900, 600, 300)),
      DataFrame(gene_id)
   )
   rd1 <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(1000), cigar = "300M", strand = strand("+"))
   rd2 <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(2000), cigar = "300M", strand = strand("+"))
   rd3 <- GAlignments("a", seqnames = Rle("chr2"), pos = as.integer(3000), cigar = "300M", strand = strand("+"))
   rds <- RoarDataset(list(c(rd1,rd2)), list(rd3), features)
   rds <- countPrePost(rds, FALSE)
   rds <- computeRoars(rds)
    }
\keyword{computeRoars}

\name{RoarDatasetMultipleAPA}
     \alias{RoarDatasetMultipleAPA}
     \title{Creates a \code{\link[=RoarDatasetMultipleAPA-class]{RoarDatasetMultipleAPA}} object}
     \description{
       This function creates an \code{\link[=RoarDatasetMultipleAPA-class]{RoarDatasetMultipleAPA}} object from two lists of 
       of \code{\link{GAlignments}} and a \code{\link{GRanges}} containing a suitable annotation of 
       alternative APA sites and gene exon structure.
      A MultipleAPA analysis computes several roar values and p-values for each gene: one 
      for every possible combination of APA-canonical end of a gene (i.e. the
      end of its last exon). This is more efficient than performing several different ``standard'' roar analyses
      choosing the PRE and POST portions corresponding to different APAs because reads overlaps are computed only once.
     }
     \usage{
      RoarDatasetMultipleAPA(treatmentBamsGenomicAlignments, controlBamsGenomicAlignments, gtfGRanges)
     }
     \arguments{
       \item{treatmentBamsGenomicAlignments}{
           A list of \code{\link{GAlignments}} representing alignment of samples for the treatment 
           condition (by convention it is considered the ``treated'' condition: this simply means
            that the package will compute roar values (ratios of the m/M) using this condition as the numerator)
           to be considered.}
        \item{controlBamsGenomicAlignments}{
           A list of \code{\link{GAlignments}} representing alignment of samples for the control condition
           to be considered.}
       \item{gtfGRanges}{
           A \code{\link{GRanges}} containing a suitable annotation of alternative APA sites and gene exonic structure.
           Minimal requirements are: metadata columns called "gene", "apa" and "type."
           APA should be single bases falling over one of the given genes and need to have the
           metadata column "type" equal to "apa" and the "apa"  column
           composed of unambiguous id and the corresponding gene id pasted together with an underscore. The "gene"
           metadata columns for these entries should not be initialized.
           All the studied gene exons need to be reported, in this case the metadata column "gene" should contain 
           the gene id (the same
           one reported for each gene APAs) while "type" should be set to "gene" and "apa" to NA.
           All apa entries assigned to a gene should have coordinates that falls inside it and every gene that
           appears should contain at least one APA.
           
        }
      }
      \value{
         A \code{\link[=RoarDatasetMultipleAPA-class]{RoarDatasetMultipleAPA}} object ready to be analyzed via the other methods.
      }
     \seealso{
       \code{\link{RoarDatasetMultipleAPAFromFiles}}
     }
     \examples{   
      library(GenomicAlignments)
      gene <- c("A", "B", NA, NA)
      type <- c("gene","gene","apa", "apa")
      apa <- c(NA, NA, "apa1_A", "apa2_B")
      features <- GRanges(
         seqnames = Rle(c("chr1", "chr2", "chr1", "chr2")),
         strand = strand(rep("+", length(gene))),
         ranges = IRanges(
            start=c(1000, 2000, 1300, 2050),
            width=c(500, 900, 1, 1)),
         DataFrame(gene, apa, type)
      )
      rd1 <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(1000), cigar = "300M", strand = strand("+"))
      rds <- RoarDatasetMultipleAPA(list(c(rd1,rd1)), list(c(rd1,rd1)), features)
    }
\keyword{RoarDataset}

\name{RoarDataset}
     \alias{RoarDataset}
     \title{Creates a \code{\link[=RoarDataset-class]{RoarDataset}} object}
     \description{
       This function creates an \code{\link[=RoarDataset-class]{RoarDataset}} object from two lists of 
       of \code{\link{GAlignments}} and a \code{\link{GRanges}} containing a suitable annotation of 
       alternative APA sites.
     }
     \usage{
      RoarDataset(treatmentGappedAlign, controlGappedAlign, gtfGRanges)
     }
     \arguments{
       \item{treatmentGappedAlign}{
           A list of \code{\link{GAlignments}} representing alignment of samples for the treatment 
           condition (by convention it is considered the ``treated'' condition: this simply means
            that the package will compute roar values (ratios of the m/M) using this condition as the numerator)
           to be considered.}
        \item{controlGappedAlign}{
           A list of \code{\link{GAlignments}} representing alignment of samples for the control condition
           to be considered.}
       \item{gtfGRanges}{
           A \code{\link{GRanges}} object with coordinates for the portions of transcripts
           that has to be considered pertaining to the short (or long) isoform.
           This \code{\link{GRanges}} object must have a character metadata column called "gene_id" 
           that ends with "_PRE" or "_POST" to address respectively the short and the long isoform.
           An element in the annotation
           is considered "PRE" (i.e. common to the short and long isoform of the transcript) if its gene_id
           ends with "_PRE". If it ends with "_POST" it is considered the portion present only in the long
           isoform.
           The prefix of gene_id should be a unique identifier for the gene and each identifier has to be
           associated with only one "_PRE" and one "_POST", leading to two genomic region associated to each   
           gene_id.
           The \code{\link{GRanges}} object can also contain a numeric 
           metadata column that represents the lengths of PRE and POST portions on the transcriptome.
           If this is omitted the lengths on the genome are used instead. Note that right now every gtf entry 
           (or none of them) should have it.
        }
      }
      \value{
         A \code{\link[=RoarDataset-class]{RoarDataset}} object ready to be analyzed via the other methods.
         %\code{"\linkS4class{RoarDataset}"}
      }
     \seealso{
       \code{\link{RoarDatasetFromFiles}}
     }
     \examples{
   library(GenomicAlignments)
   gene_id <- c("A_PRE", "A_POST", "B_PRE", "B_POST")
   features <- GRanges(
      seqnames = Rle(c("chr1", "chr1", "chr2", "chr2")),
      strand = strand(rep("+", length(gene_id))),
      ranges = IRanges(
         start=c(1000, 2000, 3000, 3600),
         width=c(1000, 900, 600, 300)),
      DataFrame(gene_id)
   )
   rd1 <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(1000), cigar = "300M", strand = strand("+"))
   rd2 <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(2000), cigar = "300M", strand = strand("+"))
   rd3 <- GAlignments("a", seqnames = Rle("chr2"), pos = as.integer(3000), cigar = "300M", strand = strand("+"))
   rds <- RoarDataset(list(c(rd1,rd2)), list(rd3), features)
    }
\keyword{RoarDataset}

\name{computePvals}
     \alias{computePvals}
     \alias{computePvals, RoarDataset}
     \alias{computePvals, RoarDatasetMultipleAPA}
     \title{Computes pvalues (Fisher test) on the read counts in this roar analysis}
     \description{
       This is the third step in the Roar analyses: it applies a Fisher test comparing counts falling
       on the PRE and POST portion in the treatment and control conditions for every gene. 
       If there are multiple samples for
       a condition every combinations of comparisons between the samples lists are considered.
     }
     \usage{
      computePvals(rds)
     }
     \arguments{
       \item{rds}{
           The \code{\linkS4class{RoarDataset}} or the \code{\linkS4class{RoarDatasetMultipleAPA}} 
           which contains the counts over PRE-POST portions
           in the two conditions to be compared via pvalues.}
      }
      \value{
         The \code{\linkS4class{RoarDataset}} or the \code{\linkS4class{RoarDatasetMultipleAPA}} object given as rds
         with the compute pvalue phase of the analysis done. Pvalues will be held in the RoarDataset object itself in 
         the case of single samples, while
         in a separate slot otherwise, but end user normally should not analyze those directly but use
         \code{\link{totalResults}} or \code{\link{fpkmResults}} at the end of the analysis. 
      }
     \examples{
   library(GenomicAlignments)
   gene_id <- c("A_PRE", "A_POST", "B_PRE", "B_POST")
   features <- GRanges(
      seqnames = Rle(c("chr1", "chr1", "chr2", "chr2")),
      strand = strand(rep("+", length(gene_id))),
      ranges = IRanges(
         start=c(1000, 2000, 3000, 3600),
         width=c(1000, 900, 600, 300)),
      DataFrame(gene_id)
   )
   rd1 <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(1000), cigar = "300M", strand = strand("+"))
   rd2 <- GAlignments("a", seqnames = Rle("chr1"), pos = as.integer(2000), cigar = "300M", strand = strand("+"))
   rd3 <- GAlignments("a", seqnames = Rle("chr2"), pos = as.integer(3000), cigar = "300M", strand = strand("+"))
   rds <- RoarDataset(list(c(rd1,rd2)), list(rd3), features)
   rds <- countPrePost(rds, FALSE)
   rds <- computeRoars(rds)
   rds <- computePvals(rds)
    }
\keyword{computePvals}

\name{cores}
     \alias{cores}
     \title{Method to check how many cores are used by a roar analysis - right now not useful}
     \description{
         Right now always returns 1 as long as multi-core support has to be implemented.
     }
     \usage{
      cores(rds)
     }
     \arguments{
       \item{rds}{
           A \code{\linkS4class{RoarDataset}} object.}
      }
      \value{
         The number of cores used by this roar analisys.
      }
\keyword{checkStep}
